@@ -10,7 +10,14 @@ See the webpage for more information and documentation:
     http://code.google.com/p/sympy/
 """
 
-__version__ = "0.6.3"
+__version__ = "0.6.4"
+
+
+def __sympy_debug():
+    # helper function so we don't import os globally
+    import os
+    return eval(os.getenv('SYMPY_DEBUG', 'False'))
+SYMPY_DEBUG = __sympy_debug()
 
 import symbol as stdlib_symbol
 from sympy.core import *
@@ -33,9 +40,9 @@ from printing import pretty, pretty_print, pprint, pprint_use_unicode, \
     pprint_try_use_unicode, print_gtk, print_tree
 from printing import ccode, latex, preview, view, pngview, pdfview, dviview
 from printing import python, print_python, srepr, sstr, sstrrepr
-from polynomials import factor
 
 evalf._create_evalf_table()
 
 # This is slow to import:
 #import abc
+
