@@ -201,10 +201,9 @@ class BasicMeta(BasicType):
 
 
     def __cmp__(cls, other):
-        try:
-            other = sympify(other)
-        except ValueError:
-            #if we cannot sympify it, other is definitely not equal to cls
+        # If the other object is not a Basic subclass, then we are not equal to
+        # it.
+        if not isinstance(other, BasicType):
             return -1
         n1 = cls.__name__
         n2 = other.__name__
@@ -391,7 +390,10 @@ class Basic(AssumeMeths):
 
     # NB: there is no need in protective __setattr__
 
-
+    def __getnewargs__(self):
+        """ Pickling support.
+        """
+        return tuple(self.args)
 
     def __hash__(self):
         # hash cannot be cached using cache_it because infinite recurrence
@@ -1520,15 +1522,21 @@ class Basic(AssumeMeths):
         """
         from sympy import collect
         x = sympify(x)
-        self = self.expand()
+        const = x.as_coeff_terms()[0] # constant multiplying x
+        if const != S.One: # get rid of constants
+            result = self.coeff(x/const)
+            if result is not None:
+                return(result/const)
+            else:
+                return None
         if x.is_Integer:
             return
-        expr = collect(self, x)
-        symbols = list(x.atoms(Symbol))
-        w = Wild("coeff", exclude=symbols)
-        m = expr.match(w*x+Wild("rest"))
-        if m:
-            return m[w]
+        self = self.expand() # collect expects it's arguments in expanded form
+        result = collect(self, x, evaluate=False)
+        if x in result:
+            return result[x]
+        else:
+            return None
 
     def as_coefficient(self, expr):
         """Extracts symbolic coefficient at the given expression. In
@@ -1932,9 +1940,11 @@ class Basic(AssumeMeths):
         from sympy.integrals import integrate
         return integrate(self, *args, **kwargs)
 
-    #XXX fix the removeme
-    def __call__(self, *args, **removeme):
-        return Function(self[0])(*args)
+    def __call__(self, subsdict):
+        """Use call as a shortcut for subs, but only support the dictionary version"""
+        if not isinstance(subsdict, dict):
+            raise TypeError("argument must be a dictionary")
+        return self.subs(subsdict)
 
     def __float__(self):
         result = self.evalf()
@@ -2266,6 +2276,16 @@ class SingletonMeta(BasicMeta):
         # inject singletonic __new__
         cls.__new__      = staticmethod(cls_new)
 
+        # Inject pickling support.
+        def cls_getnewargs(self):
+            return ()
+        cls_getnewargs.__name__ = '%s.__getnewargs__' % cls.__name__
+
+        assert not cls.__dict__.has_key('__getnewargs__'), \
+                'Singleton classes are not allowed to redefine __getnewargs__'
+        cls.__getnewargs__ = cls_getnewargs
+
+
         # tag the class appropriately (so we could verify it later when doing
         # S.<something>
         cls.is_Singleton = True
