@@ -1,20 +1,31 @@
 import math
 import sympy.mpmath as mpmath
-import sympy.mpmath.lib as mlib
+import sympy.mpmath.libmpf as mlib
+from sympy.mpmath.libmpf import mpf_abs
 import sympy.mpmath.libmpc as mlibc
+from sympy.mpmath.libelefun import mpf_pow, mpf_pi, phi_fixed
 import decimal
 
 rnd = mlib.round_nearest
 
-from basic import Basic, Atom, Singleton, S, C, Memoizer, MemoizerArg
+from basic import Basic, Atom, S, C, SingletonMeta, Memoizer, MemoizerArg
 from sympify import _sympify, SympifyError, _sympifyit
-from methods import NoRelMeths, RelMeths, ArithMeths
 from power import integer_nthroot
 
 # from mul import Mul   /cyclic/
 # from power import Pow /cyclic/
 # from function import FunctionClass    /cyclic/
 
+_errdict = {"divide": False}
+def seterr(divide=False):
+    """
+    Should sympy raise an exception on 0/0 or return a nan?
+
+    divide == True .... raise an exception
+    divide == False ... return nan
+    """
+    _errdict["divide"] = divide
+
 # (a,b) -> gcd(a,b)
 _gcdcache = {}
 
@@ -23,7 +34,7 @@ def igcd(a, b):
     """Computes integer greates common divisor of two numbers.
 
        The algorithm is based on the well known Euclid's algorithm. To
-       improve speed igcd() has its own caching mechanizm implemented.
+       improve speed, igcd() has its own caching mechanizm implemented.
     """
     try:
         return _gcdcache[(a,b)]
@@ -45,7 +56,7 @@ def ilcm(a, b):
     if a == 0 and b == 0:
         return 0
     else:
-        return a * b / igcd(a, b)
+        return a * b // igcd(a, b)
 
 def igcdex(a, b):
     """Returns x, y, g such that g = x*a + y*b = gcd(a, b).
@@ -66,9 +77,9 @@ def igcdex(a, b):
         return (0, 1, 0)
 
     if not a:
-        return (0, b/abs(b), abs(b))
+        return (0, b//abs(b), abs(b))
     if not b:
-        return (a/abs(a), 0, abs(a))
+        return (a//abs(a), 0, abs(a))
 
     if a < 0:
         a, x_sign = -a, -1
@@ -83,7 +94,7 @@ def igcdex(a, b):
     x, y, r, s = 1, 0, 0, 1
 
     while b:
-        (c, q) = (a % b, a / b)
+        (c, q) = (a % b, a // b)
         (a, b, r, s, x, y) = (b, c, x-q*r, y-q*s, r, s)
 
     return (x*x_sign, y*y_sign, a)
@@ -128,7 +139,7 @@ def factor_trial_division(n):
     return factors
 
 
-class Number(Atom, RelMeths, ArithMeths):
+class Number(Atom):
     """
     Represents any kind of number in sympy.
 
@@ -229,6 +240,9 @@ class Real(Number):
     __slots__ = ['_mpf_', '_prec']
 
     # mpz can't be pickled
+    def __getnewargs__(self):
+        return (mlib.to_pickable(self._mpf_),)
+
     def __getstate__(self):
         d = Basic.__getstate__(self).copy()
         del d["_mpf_"]
@@ -243,10 +257,10 @@ class Real(Number):
     is_Real = True
 
     def floor(self):
-        return C.Integer(int(mlib.to_int(mlib.ffloor(self._mpf_, self._prec))))
+        return C.Integer(int(mlib.to_int(mlib.mpf_floor(self._mpf_, self._prec))))
 
     def ceiling(self):
-        return C.Integer(int(mlib.to_int(mlib.fceil(self._mpf_, self._prec))))
+        return C.Integer(int(mlib.to_int(mlib.mpf_ceil(self._mpf_, self._prec))))
 
     @property
     def num(self):
@@ -259,11 +273,11 @@ class Real(Number):
         return self._mpf_, max(prec, self._prec)
 
     def __new__(cls, num, prec=15):
-        prec = mlib.dps_to_prec(prec)
+        prec = mpmath.settings.dps_to_prec(prec)
         if isinstance(num, (int, long)):
             return Integer(num)
         if isinstance(num, (str, decimal.Decimal)):
-            _mpf_ = mlib.from_str(str(num), prec)
+            _mpf_ = mlib.from_str(str(num), prec, rnd)
         elif isinstance(num, tuple) and len(num) == 4:
             _mpf_ = num
         else:
@@ -287,17 +301,6 @@ class Real(Number):
     def _hashable_content(self):
         return (self._mpf_, self._prec)
 
-    def tostr(self, level=0):
-        r = mlib.to_str(self._mpf_, mlib.prec_to_dps(self._prec))
-        if self.precedence<=level:
-            return '(%s)' % (r)
-        return r
-
-    def torepr(self):
-        dps = mlib.prec_to_dps(self._prec)
-        r = mlib.to_str(self._mpf_, dps+3)
-        return '%s(%r, prec=%i)' % (self.__class__.__name__, r, dps)
-
     def _eval_is_positive(self):
         return self.num > 0
 
@@ -305,7 +308,7 @@ class Real(Number):
         return self.num < 0
 
     def __neg__(self):
-        return Real._new(mlib.fneg(self._mpf_), self._prec)
+        return Real._new(mlib.mpf_neg(self._mpf_), self._prec)
 
     def __mul__(self, other):
         try:
@@ -314,7 +317,7 @@ class Real(Number):
             return NotImplemented
         if isinstance(other, Number):
             rhs, prec = other._as_mpf_op(self._prec)
-            return Real._new(mlib.fmul(self._mpf_, rhs, prec, rnd), prec)
+            return Real._new(mlib.mpf_mul(self._mpf_, rhs, prec, rnd), prec)
         return Number.__mul__(self, other)
 
     def __add__(self, other):
@@ -326,7 +329,7 @@ class Real(Number):
             return S.NaN
         if isinstance(other, Number):
             rhs, prec = other._as_mpf_op(self._prec)
-            return Real._new(mlib.fadd(self._mpf_, rhs, prec, rnd), prec)
+            return Real._new(mlib.mpf_add(self._mpf_, rhs, prec, rnd), prec)
         return Number.__add__(self, other)
 
     def _eval_power(b, e):
@@ -340,18 +343,18 @@ class Real(Number):
         if isinstance(e, Number):
             if isinstance(e, Integer):
                 prec = b._prec
-                return Real._new(mlib.fpowi(b._mpf_, e.p, prec, rnd), prec)
+                return Real._new(mlib.mpf_pow_int(b._mpf_, e.p, prec, rnd), prec)
             e, prec = e._as_mpf_op(b._prec)
             b = b._mpf_
             try:
-                y = mlib.fpow(b, e, prec, rnd)
+                y = mpf_pow(b, e, prec, rnd)
                 return Real._new(y, prec)
             except mlib.ComplexResult:
                 re, im = mlibc.mpc_pow((b, mlib.fzero), (e, mlib.fzero), prec, rnd)
                 return Real._new(re, prec) + Real._new(im, prec) * S.ImaginaryUnit
 
     def __abs__(self):
-        return Real._new(mlib.fabs(self._mpf_), self._prec)
+        return Real._new(mlib.mpf_abs(self._mpf_), self._prec)
 
     def __int__(self):
         return int(mlib.to_int(self._mpf_))
@@ -366,7 +369,7 @@ class Real(Number):
             return other.__eq__(self)
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
-            return bool(mlib.feq(self._mpf_, other._as_mpf_val(self._prec)))
+            return bool(mlib.mpf_eq(self._mpf_, other._as_mpf_val(self._prec)))
         return False    # Real != non-Number
 
     def __ne__(self, other):
@@ -379,7 +382,7 @@ class Real(Number):
             return other.__ne__(self)
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
-            return bool(not mlib.feq(self._mpf_, other._as_mpf_val(self._prec)))
+            return bool(not mlib.mpf_eq(self._mpf_, other._as_mpf_val(self._prec)))
         return True     # Real != non-Number
 
     def __lt__(self, other):
@@ -391,8 +394,8 @@ class Real(Number):
             return other.__ge__(self)
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
-            return bool(mlib.flt(self._mpf_, other._as_mpf_val(self._prec)))
-        return RelMeths.__lt__(self, other)
+            return bool(mlib.mpf_lt(self._mpf_, other._as_mpf_val(self._prec)))
+        return Basic.__lt__(self, other)
 
     def __le__(self, other):
         try:
@@ -403,12 +406,21 @@ class Real(Number):
             return other.__gt__(self)
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
-            return bool(mlib.fle(self._mpf_, other._as_mpf_val(self._prec)))
-        return RelMeths.__le__(self, other)
+            return bool(mlib.mpf_le(self._mpf_, other._as_mpf_val(self._prec)))
+        return Basic.__le__(self, other)
 
     def epsilon_eq(self, other, epsilon="10e-16"):
         return abs(self - other) < Real(epsilon)
 
+    def _sage_(self):
+        import sage.all as sage
+        return sage.RealNumber(str(self))
+
+
+# this is here to work nicely in Sage
+RealNumber = Real
+
+
 def _parse_rational(s):
     """Parse rational number from string representation"""
     # Simple fraction
@@ -437,7 +449,6 @@ def _parse_rational(s):
         else:
             return p, 10**-expt
 
-
 class Rational(Number):
     """Represents integers and rational numbers (p/q) of any size.
 
@@ -460,7 +471,7 @@ class Rational(Number):
         >>> Rational("1.2[05]")
         1193/990
         >>> float(Rational(1193,990))
-        1.2050505050505051
+        1.20505050505
 
 
     Low-level
@@ -483,7 +494,7 @@ class Rational(Number):
 
     is_Rational = True
 
-    @Memoizer(type, (int, long, str), MemoizerArg((int, long, type(None)), name="q"))
+    @Memoizer(type, (int, long, str, 'Integer'), MemoizerArg((int, long, 'Integer', type(None)), name="q"))
     def __new__(cls, p, q = None):
         if q is None:
             if isinstance(p, str):
@@ -491,7 +502,11 @@ class Rational(Number):
             else:
                 return Integer(p)
         if q==0:
-            if p==0: return S.NaN
+            if p==0:
+                if _errdict["divide"]:
+                    raise Exception("Indeterminate 0/0")
+                else:
+                    return S.NaN
             if p<0: return S.NegativeInfinity
             return S.Infinity
         if q<0:
@@ -499,32 +514,21 @@ class Rational(Number):
             p = -p
         n = igcd(abs(p), q)
         if n>1:
-            p /= n
-            q /= n
+            p //= n
+            q //= n
         if q==1: return Integer(p)
         if p==1 and q==2: return S.Half
         obj = Basic.__new__(cls)
-        obj.p = p
-        obj.q = q
+        obj.p = int(p)
+        obj.q = int(q)
         #obj._args = (p, q)
         return obj
 
-    def _hashable_content(self):
+    def __getnewargs__(self):
         return (self.p, self.q)
 
-    def tostr(self, level=0):
-        if self.precedence<=level:
-            return '(%s/%s)' % (self.p, self.q)
-        return '%s/%s' % (self.p, self.q)
-
-    def torepr(self):
-        return '%s(%r, %r)' % (self.__class__.__name__, self.p, self.q)
-
-    @property
-    def precedence(self):
-        if self.p < 0:
-            return Basic.Add_precedence
-        return Basic.Mul_precedence
+    def _hashable_content(self):
+        return (self.p, self.q)
 
     def _eval_is_positive(self):
         return self.p > 0
@@ -622,7 +626,7 @@ class Rational(Number):
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Real):
-                return bool(mlib.feq(self._as_mpf_val(other._prec), other._mpf_))
+                return bool(mlib.mpf_eq(self._as_mpf_val(other._prec), other._mpf_))
             return bool(self.p==other.p and self.q==other.q)
 
         return False    # Rational != non-Number
@@ -653,9 +657,9 @@ class Rational(Number):
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Real):
-                return bool(mlib.flt(self._as_mpf_val(other._prec), other._mpf_))
+                return bool(mlib.mpf_lt(self._as_mpf_val(other._prec), other._mpf_))
             return bool(self.p * other.q < self.q * other.p)
-        return RelMeths.__lt__(self, other)
+        return Basic.__lt__(self, other)
 
     def __le__(self, other):
         try:
@@ -667,25 +671,16 @@ class Rational(Number):
         if other.is_comparable and not isinstance(other, Rational): other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Real):
-                return bool(mlib.fle(self._as_mpf_val(other._prec), other._mpf_))
+                return bool(mlib.mpf_le(self._as_mpf_val(other._prec), other._mpf_))
             return bool(self.p * other.q <= self.q * other.p)
-        return RelMeths.__le__(self, other)
+        return Basic.__le__(self, other)
 
     def factors(self):
         f = factor_trial_division(self.p).copy()
         for p,e in factor_trial_division(self.q).items():
             try: f[p] += -e
             except KeyError: f[p] = -e
-        fi = {}
-        for p,e in f.items():
-            if e==0:
-                del f[p]
-            else:
-                try: fi[e] *= p
-                except KeyError: fi[e] = p
-        f = {}
-        for e,p in fi.items():
-            f[p] = e
+
         if len(f)>1 and f.has_key(1): del f[1]
         return f
 
@@ -701,6 +696,62 @@ class Rational(Number):
 # int -> Integer
 _intcache = {}
 
+
+# TODO move this tracing facility to  sympy/core/trace.py  ?
+def _intcache_printinfo():
+    ints = sorted(_intcache.keys())
+    nhit = _intcache_hits
+    nmiss= _intcache_misses
+
+    if nhit == 0 and nmiss == 0:
+        print
+        print 'Integer cache statistic was not collected'
+        return
+
+    miss_ratio = float(nmiss) / (nhit+nmiss)
+
+    print
+    print 'Integer cache statistic'
+    print '-----------------------'
+    print
+    print '#items: %i' % len(ints)
+    print
+    print ' #hit   #miss               #total'
+    print
+    print '%5i   %5i (%7.5f %%)   %5i'    % (nhit, nmiss, miss_ratio*100, nhit+nmiss)
+    print
+    print ints
+
+_intcache_hits   = 0
+_intcache_misses = 0
+
+def int_trace(f):
+    import os
+    if os.getenv('SYMPY_TRACE_INT', 'no').lower() != 'yes':
+        return f
+
+    def Integer_tracer(cls, i):
+        global _intcache_hits, _intcache_misses
+
+        try:
+            _intcache_hits += 1
+            return _intcache[i]
+        except KeyError:
+            _intcache_hits   -= 1
+            _intcache_misses += 1
+
+            return f(cls, i)
+
+
+    # also we want to hook our _intcache_printinfo into sys.atexit
+    import atexit
+    atexit.register(_intcache_printinfo)
+
+    return Integer_tracer
+
+
+
+
 class Integer(Rational):
 
     q = 1
@@ -714,18 +765,34 @@ class Integer(Rational):
         return mlib.from_int(self.p)
 
     # TODO caching with decorator, but not to degrade performance
+    @int_trace
     def __new__(cls, i):
         try:
             return _intcache[i]
         except KeyError:
-            if not isinstance(i, (int, long)):
+            # The most often situation is when Integers are created from Python
+            # int or long
+            if isinstance(i, (int, long)):
+                obj = Basic.__new__(cls)
+                obj.p = i
+                _intcache[i] = obj
+                return obj
+
+            # Also, we seldomly need the following to work:
+            # UC: Integer(Integer(4))   <-- sympify('4')
+            elif isinstance(i, Integer):
+                return i
+
+            else:
                 raise ValueError('invalid argument for Integer: %r' % (i,))
-            obj = Basic.__new__(cls)
-            obj.p = i
-            _intcache[i] = obj
-            return obj
+
+    def __getnewargs__(self):
+        return (self.p,)
 
     # Arithmetic operations are here for efficiency
+    def __int__(self):
+        return self.p
+
     def __neg__(self):
         return Integer(-self.p)
 
@@ -735,6 +802,12 @@ class Integer(Rational):
         else:
             return Integer(-self.p)
 
+    def __mod__(self, other):
+        return self.p % other
+
+    def __rmod__(self, other):
+        return other % self.p
+
     # TODO make it decorator + bytecodehacks?
     def __add__(a, b):
         if type(b) is int:
@@ -750,6 +823,20 @@ class Integer(Rational):
             return Integer(b.p + a.p)
         return Rational.__add__(a, b)
 
+    def __sub__(a, b):
+        if type(b) is int:
+            return Integer(a.p - b)
+        elif isinstance(b, Integer):
+            return Integer(a.p - b.p)
+        return Rational.__sub__(a, b)
+
+    def __rsub__(a, b):
+        if type(b) is int:
+            return Integer(b - a.p)
+        elif isinstance(b, Integer):
+            return Integer(b.p - a.p)
+        return Rational.__rsub__(a, b)
+
     def __mul__(a, b):
         if type(b) is int:
             return Integer(a.p * b)
@@ -816,114 +903,108 @@ class Integer(Rational):
     def _eval_is_odd(self):
         return bool(self.p % 2)
 
-    @property
-    def precedence(self):
-        if self.p < 0:
-            return 40 # same as Add
-        return Atom.precedence
+    def _eval_power(base, exp):
+        """
+        Tries to do some simplifications on base ** exp, where base is
+        an instance of Rational
 
-    def tostr(self, level=0):
-        if self.precedence<=level:
-            return '(%s)' % (self.p)
-        return str(self.p)
+        Returns None if no further simplifications can be done
 
-    def torepr(self):
-        return '%s(%r)' % (self.__class__.__name__, self.p)
+        When exponent is a fraction (so we have for example a square root),
+        we try to find the simplest possible representation, so that
+          - 4**Rational(1,2) becomes 2
+          - (-4)**Rational(1,2) becomes 2*I
+        """
 
-    def _eval_power(b, e):
-        if isinstance(e, Number):
-            if e is S.NaN: return S.NaN
-            if isinstance(e, Real):
-                return b._eval_evalf(e._prec) ** e
-            if e.is_negative:
-                # (3/4)**-2 -> (4/3)**2
-                ne = -e
-                if ne is S.One:
-                    return Rational(1, b.p)
-                return Rational(1, b.p) ** ne
-            if e is S.Infinity:
-                if b.p > 1:
-                    # (3)**oo -> oo
-                    return S.Infinity
-                if b.p < -1:
-                    # (-3)**oo -> oo + I*oo
-                    return S.Infinity + S.Infinity * S.ImaginaryUnit
-                return S.Zero
-            if isinstance(e, Integer):
-                # (4/3)**2 -> 4**2 / 3**2
-                return Integer(b.p ** e.p)
-            if isinstance(e, Rational):
-                if b == -1:  # any one has tested this ???
-                    # calculate the roots of -1
-                    if e.q.is_odd:
-                        return -1
-                    r = cos(pi/e.q) + S.ImaginaryUnit*sin(pi/e.q)
-                    return r**e.p
-                if b >= 0:
-                    x, xexact = integer_nthroot(b.p, e.q)
-                    if xexact:
-                        res = Integer(x ** abs(e.p))
-                        if e >= 0:
-                            return res
-                        else:
-                            return 1/res
-                    else:
-                        if b > 2**32: #Prevent from factorizing too big integers:
-                            for i in xrange(2, e.q/2 + 1): #OLD CODE
-                                if e.q % i == 0:
-                                    x, xexact = integer_nthroot(b.p, i)
-                                    if xexact:
-                                        return Integer(x)**(e * i)
-                            # Try to get some part of the base out, if exponent > 1
-                            if e.p > e.q:
-                                i = e.p / e.q
-                                r = e.p % e.q
-                                return b**i * b**Rational(r, e.q)
-                            return
-
-
-                        dict = b.factors()
-
-                        out_int = 1
-                        sqr_int = 1
-                        sqr_gcd = 0
-
-                        sqr_dict = {}
-
-                        for prime,exponent in dict.iteritems():
-                            exponent *= e.p
-                            div_e = exponent / e.q
-                            div_m = exponent % e.q
-
-                            if div_e > 0:
-                                out_int *= prime**div_e
-                            if div_m > 0:
-                                sqr_dict[prime] = div_m
-
-                        for p,ex in sqr_dict.iteritems():
-                            if sqr_gcd == 0:
-                                sqr_gcd = ex
-                            else:
-                                sqr_gcd = igcd(sqr_gcd, ex)
-
-                        for k,v in sqr_dict.iteritems():
-                            sqr_int *= k**(v/sqr_gcd)
-
-                        if sqr_int == b.p and out_int == 1:
-                            return None
-
-                        return out_int * Pow(sqr_int , Rational(sqr_gcd, e.q))
+        MAX_INT_FACTOR = 4294967296 # Prevent from factorizing too big integers
+        if not isinstance(exp, Number):
+            c,t = base.as_coeff_terms()
+            if exp.is_even and isinstance(c, Number) and c < 0:
+                return (-c * Mul(*t)) ** exp
+        if exp is S.NaN: return S.NaN
+        if isinstance(exp, Real):
+            return base._eval_evalf(exp._prec) ** exp
+        if exp.is_negative:
+            # (3/4)**-2 -> (4/3)**2
+            ne = -exp
+            if ne is S.One:
+                return Rational(1, base.p)
+            return Rational(1, base.p) ** ne
+        if exp is S.Infinity:
+            if base.p > 1:
+                # (3)**oo -> oo
+                return S.Infinity
+            if base.p < -1:
+                # (-3)**oo -> oo + I*oo
+                return S.Infinity + S.Infinity * S.ImaginaryUnit
+            return S.Zero
+        if isinstance(exp, Integer):
+            # (4/3)**2 -> 4**2 / 3**2
+            return Integer(base.p ** exp.p)
+        if exp == S.Half and base.is_negative:
+            # sqrt(-2) -> I*sqrt(2)
+            return S.ImaginaryUnit * ((-base)**exp)
+        if isinstance(exp, Rational):
+            # 4**Rational(1,2) -> 2
+            result = None
+            x, xexact = integer_nthroot(abs(base.p), exp.q)
+            if xexact:
+                res = Integer(x ** abs(exp.p))
+                if exp >= 0:
+                    result = res
                 else:
-                    if e.q == 2:
-                        return S.ImaginaryUnit ** e.p * (-b)**e
+                    result = 1/res
+            else:
+                if base > MAX_INT_FACTOR:
+                    for i in xrange(2, exp.q//2 + 1): #OLD CODE
+                        if exp.q % i == 0:
+                            x, xexact = integer_nthroot(abs(base.p), i)
+                            if xexact:
+                                result = Integer(x)**(exp * i)
+                                break
+                    # Try to get some part of the base out, if exponent > 1
+                    if exp.p > exp.q:
+                        i = exp.p // exp.q
+                        r = exp.p % exp.q
+                        result = base**i * base**Rational(r, exp.q)
+
+                    return
+
+                dict = base.factors()
+                out_int = 1
+                sqr_int = 1
+                sqr_gcd = 0
+                sqr_dict = {}
+                for prime,exponent in dict.iteritems():
+                    exponent *= exp.p
+                    div_e = exponent // exp.q
+                    div_m = exponent % exp.q
+                    if div_e > 0:
+                        out_int *= prime**div_e
+                    if div_m > 0:
+                        sqr_dict[prime] = div_m
+                for p,ex in sqr_dict.iteritems():
+                    if sqr_gcd == 0:
+                        sqr_gcd = ex
                     else:
-                        return None
+                        sqr_gcd = igcd(sqr_gcd, ex)
+                for k,v in sqr_dict.iteritems():
+                    sqr_int *= k**(v // sqr_gcd)
+                if sqr_int == base.p and out_int == 1:
+                    result = None
+                else:
+                    result = out_int * Pow(sqr_int , Rational(sqr_gcd, exp.q))
 
-        c,t = b.as_coeff_terms()
-        if e.is_even and isinstance(c, Number) and c < 0:
-            return (-c * Mul(*t)) ** e
-
-        return
+            if result is not None:
+                if base.is_positive:
+                    return result
+                elif exp.q == 2:
+                    return result * S.ImaginaryUnit
+                else:
+                    return result * ((-1)**exp)
+            else:
+                if exp == S.Half and base.is_negative:
+                    return S.ImaginaryUnit * Pow(-base, exp)
 
     def _eval_is_prime(self):
         if self.p < 0:
@@ -938,7 +1019,8 @@ class Integer(Rational):
     def __rfloordiv__(self, other):
         return Integer(Integer(other).p // self.p)
 
-class Zero(Singleton, Integer):
+class Zero(Integer):
+    __metaclass__ = SingletonMeta
 
     p = 0
     q = 1
@@ -947,7 +1029,7 @@ class Zero(Singleton, Integer):
     is_finite = False
     is_zero = True
     is_prime = False
-    is_composite = True
+    is_composite = False
 
     __slots__ = []
 
@@ -979,7 +1061,8 @@ class Zero(Singleton, Integer):
         # Order(0,x) -> 0
         return self
 
-class One(Singleton, Integer):
+class One(Integer):
+    __metaclass__ = SingletonMeta
 
     p = 1
     q = 1
@@ -1005,7 +1088,12 @@ class One(Singleton, Integer):
     def _eval_order(self, *symbols):
         return
 
-class NegativeOne(Singleton, Integer):
+    @staticmethod
+    def factors():
+        return {1: 1}
+
+class NegativeOne(Integer):
+    __metaclass__ = SingletonMeta
 
     p = -1
     q = 1
@@ -1044,7 +1132,8 @@ class NegativeOne(Singleton, Integer):
                     return b ** q * b ** (e - q)
         return
 
-class Half(Singleton, Rational):
+class Half(Rational):
+    __metaclass__ = SingletonMeta
 
     p = 1
     q = 2
@@ -1056,7 +1145,8 @@ class Half(Singleton, Rational):
         return S.Half
 
 
-class Infinity(Singleton, Rational):
+class Infinity(Rational):
+    __metaclass__ = SingletonMeta
 
     p = 1
     q = 0
@@ -1066,7 +1156,10 @@ class Infinity(Singleton, Rational):
     is_commutative = True
     is_positive = True
     is_bounded = False
-    is_finite = None
+    is_finite   = False
+    is_infinitesimal = False
+    is_integer  = None
+    is_rational = None
     is_odd = None
 
     @staticmethod
@@ -1077,9 +1170,6 @@ class Infinity(Singleton, Rational):
     def __neg__():
         return S.NegativeInfinity
 
-    def tostr(self, level=0):
-        return 'oo'
-
     def _eval_power(b, e):
         """
         e is symbolic object but not equal to 0, 1
@@ -1106,7 +1196,8 @@ class Infinity(Singleton, Rational):
         import sage.all as sage
         return sage.oo
 
-class NegativeInfinity(Singleton, Rational):
+class NegativeInfinity(Rational):
+    __metaclass__ = SingletonMeta
 
     p = -1
     q = 0
@@ -1118,8 +1209,9 @@ class NegativeInfinity(Singleton, Rational):
     is_positive = False
     is_bounded = False
     is_finite = False
-
-    precedence = 40 # same as Add
+    is_infinitesimal = False
+    is_integer  = None
+    is_rational = None
 
     @staticmethod
     def __abs__():
@@ -1129,9 +1221,6 @@ class NegativeInfinity(Singleton, Rational):
     def __neg__():
         return S.Infinity
 
-    def tostr(self, level=0):
-        return '-oo'
-
     def _eval_power(b, e):
         """
         e is symbolic object but not equal to 0, 1
@@ -1157,22 +1246,26 @@ class NegativeInfinity(Singleton, Rational):
     def _as_mpf_val(self, prec):
         return mlib.fninf
 
-class NaN(Singleton, Rational):
+class NaN(Rational):
+    __metaclass__ = SingletonMeta
 
     p = 0
     q = 0
 
     is_commutative = True
     is_real = None
-    is_comparable = None
+    is_rational = None
+    is_integer  = None
+    is_comparable = False
+    is_finite   = None
     is_bounded = None
     #is_unbounded = False
+    is_zero     = None
+    is_prime    = None
+    is_positive = None
 
     __slots__ = []
 
-    def tostr(self, level=0):
-        return 'nan'
-
     def _as_mpf_val(self, prec):
         return mlib.fnan
 
@@ -1185,7 +1278,8 @@ class NaN(Singleton, Rational):
         import sage.all as sage
         return sage.NaN
 
-class ComplexInfinity(Singleton, Atom, NoRelMeths, ArithMeths):
+class ComplexInfinity(Atom):
+    __metaclass__ = SingletonMeta
 
     is_commutative = True
     is_comparable = None
@@ -1202,9 +1296,6 @@ class ComplexInfinity(Singleton, Atom, NoRelMeths, ArithMeths):
     def __neg__():
         return S.ComplexInfinity
 
-    def tostr(self, level=0):
-        return 'zoo'
-
     def _eval_power(b, e):
         if e is S.ComplexInfinity:
             return S.NaN
@@ -1218,7 +1309,8 @@ class ComplexInfinity(Singleton, Atom, NoRelMeths, ArithMeths):
                 else:
                     return S.Zero
 
-class NumberSymbol(Singleton, Atom, RelMeths, ArithMeths):
+class NumberSymbol(Atom):
+    __metaclass__ = SingletonMeta
 
     is_commutative = True
     is_comparable = True
@@ -1277,7 +1369,7 @@ class NumberSymbol(Singleton, Atom, RelMeths, ArithMeths):
         if other.is_comparable:
             other = other.evalf()
             return self.evalf()<other
-        return RelMeths.__lt__(self, other)
+        return Basic.__lt__(self, other)
     def __le__(self, other):
         try:
             other = _sympify(other)
@@ -1287,7 +1379,7 @@ class NumberSymbol(Singleton, Atom, RelMeths, ArithMeths):
         if other.is_comparable: other = other.evalf()
         if isinstance(other, Number):
             return self.evalf()<=other
-        return RelMeths.__le__(self, other)
+        return Basic.__le__(self, other)
     def __gt__(self, other):
         return (-self) < (-other)
     def __ge__(self, other):
@@ -1307,9 +1399,6 @@ class Exp1(NumberSymbol):
     def __abs__():
         return S.Exp1
 
-    def tostr(self, level=0):
-        return 'E'
-
     def _as_mpf_val(self, prec):
         return mlib.fe(prec)
 
@@ -1340,7 +1429,7 @@ class Pi(NumberSymbol):
         return S.Pi
 
     def _as_mpf_val(self, prec):
-        return mlib.fpi(prec)
+        return mpf_pi(prec)
 
     def approximation_interval(self, number_cls):
         if issubclass(number_cls, Integer):
@@ -1348,9 +1437,6 @@ class Pi(NumberSymbol):
         elif issubclass(number_cls, Rational):
             return (Rational(223,71), Rational(22,7))
 
-    def tostr(self, level=0):
-        return 'pi'
-
     def _sage_(self):
         import sage.all as sage
         return sage.pi
@@ -1365,7 +1451,7 @@ class GoldenRatio(NumberSymbol):
     __slots__ = []
 
     def _as_mpf_val(self, prec):
-        return mlib.from_man_exp(mpmath.specfun.phi_fixed(prec+10), -prec-10)
+        return mlib.from_man_exp(phi_fixed(prec+10), -prec-10)
 
     def _eval_expand_func(self, *args):
         return S.Half + S.Half*S.Sqrt(5)
@@ -1376,9 +1462,6 @@ class GoldenRatio(NumberSymbol):
         elif issubclass(number_cls, Rational):
             pass
 
-    def tostr(self, level=0):
-        return 'GoldenRatio'
-
     def _sage_(self):
         import sage.all as sage
         return sage.golden_ratio
@@ -1393,7 +1476,7 @@ class EulerGamma(NumberSymbol):
     __slots__ = []
 
     def _as_mpf_val(self, prec):
-        return mlib.from_man_exp(mpmath.specfun.euler_fixed(prec+10), -prec-10)
+        return mlib.from_man_exp(mpmath.gammazeta.euler_fixed(prec+10), -prec-10)
 
     def approximation_interval(self, number_cls):
         if issubclass(number_cls, Integer):
@@ -1401,9 +1484,6 @@ class EulerGamma(NumberSymbol):
         elif issubclass(number_cls, Rational):
             return (S.Half, Rational(3, 5))
 
-    def tostr(self, level=0):
-        return 'EulerGamma'
-
     def _sage_(self):
         import sage.all as sage
         return sage.euler_gamma
@@ -1418,7 +1498,7 @@ class Catalan(NumberSymbol):
     __slots__ = []
 
     def _as_mpf_val(self, prec):
-        return mlib.from_man_exp(mpmath.specfun.catalan_fixed(prec+10), -prec-10)
+        return mlib.from_man_exp(mpmath.gammazeta.catalan_fixed(prec+10), -prec-10)
 
     def approximation_interval(self, number_cls):
         if issubclass(number_cls, Integer):
@@ -1426,14 +1506,12 @@ class Catalan(NumberSymbol):
         elif issubclass(number_cls, Rational):
             return (Rational(9, 10), S.One)
 
-    def tostr(self, level=0):
-        return 'Catalan'
-
     def _sage_(self):
         import sage.all as sage
         return sage.catalan
 
-class ImaginaryUnit(Singleton, Atom, RelMeths, ArithMeths):
+class ImaginaryUnit(Atom):
+    __metaclass__ = SingletonMeta
 
     is_commutative = True
     is_imaginary = True
@@ -1449,9 +1527,6 @@ class ImaginaryUnit(Singleton, Atom, RelMeths, ArithMeths):
     def _eval_evalf(self, prec):
         return self
 
-    def tostr(self, level=0):
-        return 'I'
-
     def _eval_conjugate(self):
         return -S.ImaginaryUnit
 
@@ -1537,3 +1612,4 @@ Basic.singleton['zoo'] = ComplexInfinity
 Basic.singleton['GoldenRatio'] = GoldenRatio
 Basic.singleton['EulerGamma'] = EulerGamma
 Basic.singleton['Catalan'] = Catalan
+
