@@ -17,7 +17,9 @@ class AssocOp(Basic):
     Base class for Add and Mul.
     """
 
-    __slots__ = []
+    # for performance reason, we don't let is_commutative go to assumptions,
+    # and keep it right here
+    __slots__ = ['is_commutative']
 
     @cacheit
     def __new__(cls, *args, **assumptions):
@@ -27,18 +29,39 @@ class AssocOp(Basic):
             return cls.identity()
         if len(args)==1:
             return _sympify(args[0])
-        c_part, nc_part, lambda_args, order_symbols = cls.flatten(map(_sympify, args))
+        c_part, nc_part, order_symbols = cls.flatten(map(_sympify, args))
         if len(c_part) + len(nc_part) <= 1:
             if c_part: obj = c_part[0]
             elif nc_part: obj = nc_part[0]
             else: obj = cls.identity()
         else:
-            assumptions['commutative'] = not nc_part
             obj = Basic.__new__(cls, *(c_part + nc_part), **assumptions)
+            obj.is_commutative = not nc_part
+
         if order_symbols is not None:
             obj = C.Order(obj, *order_symbols)
-        if lambda_args is not None:
-            obj = Lambda(obj, *lambda_args)
+        return obj
+
+
+    def _new_rawargs(self, *args):
+        """create new instance of own class with args exactly as provided by caller
+
+           This is handy when we want to optimize things, e.g.
+
+           >>> from sympy import Mul, symbols
+           >>> x,y = symbols('xy')
+           >>> e = Mul(3,x,y)
+           >>> e.args
+           (3, x, y)
+           >>> Mul(*e.args[1:])
+           x*y
+           >>> e._new_rawargs(*e.args[1:])  # the same as above, but faster
+           x*y
+
+        """
+        obj = Basic.__new__(type(self), *args)  # NB no assumptions for Add/Mul
+        obj.is_commutative = self.is_commutative
+
         return obj
 
     @classmethod
@@ -60,7 +83,7 @@ class AssocOp(Basic):
                 seq = list(o[:]) + seq
                 continue
             new_seq.append(o)
-        return [], new_seq, None, None
+        return [], new_seq, None
 
     _eval_subs = Basic._seq_subs
 
@@ -83,9 +106,14 @@ class AssocOp(Basic):
         exact_part = []
         for p in pattern.args:
             if p.atoms(Wild, WildFunction):
-                wild_part.append(p)
-            else:
-                exact_part.append(p)
+                # not all Wild should stay Wilds, for example:
+                # (w2+w3).matches(w1) -> (w1+w3).matches(w1) -> w3.matches(0)
+                if (not p in repl_dict) and (not p in expr):
+                    wild_part.append(p)
+                    continue
+
+            exact_part.append(p)
+
         if exact_part:
             newpattern = pattern.__class__(*wild_part)
             newexpr = pattern.__class__._combine_inverse(expr, pattern.__class__(*exact_part))
