@@ -1,7 +1,6 @@
 
 from basic import Basic, S, C
 from sympify import _sympify
-from methods import ArithMeths, RelMeths
 from cache import cacheit
 
 from sympy import mpmath
@@ -25,39 +24,53 @@ def integer_nthroot(y, n):
     >>> integer_nthroot(26,2)
     (5, False)
     """
-    if y < 0: raise ValueError("y must not be negative")
+    if y < 0: raise ValueError("y must be nonnegative")
     if n < 1: raise ValueError("n must be positive")
     if y in (0, 1): return y, True
     if n == 1: return y, True
+    if n == 2:
+        x, rem = mpmath.libmpf.sqrtrem(y)
+        return int(x), not rem
     if n > y: return 1, False
     # Get initial estimate for Newton's method. Care must be taken to
     # avoid overflow
     try:
         guess = int(y ** (1./n)+0.5)
     except OverflowError:
-        try:
-            guess = int(_exp(_log(y)/n)+0.5)
-        except OverflowError:
-            guess = 1 << int(_log(y, 2)/n)
-    # Newton iteration
-    xprev, x = -1, guess
-    while abs(x - xprev) > 1:
-        t = x**(n-1)
-        xprev, x = x, x - (t*x-y)//(n*t)
+        expt = _log(y,2)/n
+        if expt > 53:
+            shift = int(expt-53)
+            guess = int(2.0**(expt-shift)+1) << shift
+        else:
+            guess = int(2.0**expt)
+    #print n
+    if guess > 2**50:
+        # Newton iteration
+        xprev, x = -1, guess
+        while 1:
+            t = x**(n-1)
+            #xprev, x = x, x - (t*x-y)//(n*t)
+            xprev, x = x, ((n-1)*x + y//t)//n
+            #print n, x-xprev, abs(x-xprev) < 2
+            if abs(x - xprev) < 2:
+                break
+    else:
+        x = guess
     # Compensate
     t = x**n
+    while t < y:
+        x += 1
+        t = x**n
     while t > y:
         x -= 1
         t = x**n
     return x, t == y
 
-class Pow(Basic, ArithMeths, RelMeths):
+class Pow(Basic):
 
     is_Pow = True
 
-    precedence = Basic.Pow_precedence
-
-    __slots__ = []
+    __slots__ = ['is_commutative']
 
     @cacheit
     def __new__(cls, a, b, **assumptions):
@@ -72,6 +85,7 @@ class Pow(Basic, ArithMeths, RelMeths):
         obj = a._eval_power(b)
         if obj is None:
             obj = Basic.__new__(cls, a, b, **assumptions)
+            obj.is_commutative = (a.is_commutative and b.is_commutative)
         return obj
 
     @property
@@ -83,33 +97,18 @@ class Pow(Basic, ArithMeths, RelMeths):
         return self._args[1]
 
     def _eval_power(self, other):
-        if other.is_Number:
-            if self.base.is_real:
-                if self.exp.is_Number:
-                    # (a ** 2) ** 3 -> a ** (2 * 3)
-                    return Pow(self.base, self.exp * other)
-            if other.is_Rational:
-                if self.exp.is_even and Integer(other.q).is_even:
-                    return abs( Pow(self.base, self.exp * other))
-                return Pow(self.base, self.exp * other)
-            if other.is_Integer:
-                # (a ** b) ** 3 -> a ** (3 * b)
-                return Pow(self.base, self.exp * other)
-        elif other.is_Add or other.is_Mul:
-            # (a**b)**c = a**(b*c)
+        if other == S.NegativeOne:
             return Pow(self.base, self.exp * other)
-
-        if other.atoms(Wild):
+        if self.exp.is_integer and other.is_integer:
+            return Pow(self.base, self.exp * other)
+        if self.base.is_nonnegative and self.exp.is_real and other.is_real:
+            return Pow(self.base, self.exp * other)
+        if self.exp.is_even and self.base.is_real:
+            return Pow(abs(self.base), self.exp * other)
+        if self.exp.is_real and other.is_real and abs(self.exp) < S.One:
             return Pow(self.base, self.exp * other)
         return
 
-    def _eval_is_commutative(self):
-        c1 = self.base.is_commutative
-        if c1 is None: return
-        c2 = self.base.is_commutative
-        if c2 is None: return
-        return c1 and c2
-
     def _eval_is_comparable(self):
         c1 = self.base.is_comparable
         if c1 is None: return
@@ -121,7 +120,7 @@ class Pow(Basic, ArithMeths, RelMeths):
         if self.exp.is_integer and self.exp.is_positive:
             if self.base.is_even:
                 return True
-            if self.exp.is_integer:
+            if self.base.is_integer:
                 return False
 
     def _eval_is_positive(self):
@@ -158,9 +157,12 @@ class Pow(Basic, ArithMeths, RelMeths):
 
     def _eval_is_integer(self):
         c1 = self.base.is_integer
-        if c1 is None: return
         c2 = self.exp.is_integer
-        if c2 is None: return
+        if c1 is None or c2 is None:
+            return None
+        if not c1:
+            if self.exp.is_nonnegative:
+                return False
         if c1 and c2:
             if self.exp.is_nonnegative or self.exp.is_positive:
                 return True
@@ -175,10 +177,12 @@ class Pow(Basic, ArithMeths, RelMeths):
         if c1 and c2:
             if self.base.is_positive:
                 return True
-
             else:   # negative or zero (or positive)
                 if self.exp.is_integer:
                     return True
+                elif self.base.is_negative:
+                    if self.exp.is_Rational:
+                        return False
 
     def _eval_is_odd(self):
         if not (self.base.is_integer and self.exp.is_nonnegative): return
@@ -198,17 +202,6 @@ class Pow(Basic, ArithMeths, RelMeths):
             if self.exp.is_nonnegative:
                 return True
 
-    def tostr(self, level=0):
-        precedence = self.precedence
-        b = self.base.tostr(precedence)
-        if self.exp is S.NegativeOne:
-            r = '1/%s' % (b)
-        else:
-            r = '%s**%s' % (b,self.exp.tostr(precedence))
-        if precedence <= level:
-            return '(%s)' % (r)
-        return r
-
     def _eval_subs(self, old, new):
         if self==old: return new
         if isinstance(old, self.__class__) and self.base==old.base:
@@ -219,7 +212,7 @@ class Pow(Basic, ArithMeths, RelMeths):
             coeff1,terms1 = old.args[0].as_coeff_terms()
             coeff2,terms2 = (self.exp * C.log(self.base)).as_coeff_terms()
             if terms1==terms2: return new ** (coeff1/coeff2) # (x**(2*y)).subs(exp(3*y*log(x)),z) -> z**(2/3*y)
-        return self.base.subs(old, new) ** self.exp.subs(old, new)
+        return self.base._eval_subs(old, new) ** self.exp._eval_subs(old, new)
 
     def as_powers_dict(self):
         return { self.base : self.exp }
@@ -359,16 +352,8 @@ class Pow(Basic, ArithMeths, RelMeths):
                 #    b[k] = Integer(expansion_dict[k])
                 #return Poly(b, *p).as_basic()
 
-                # use a direct method which is faster:
-                l = []
-                for powers, k in expansion_dict.iteritems():
-                    # assemble a term, e.g. 3*x**2*y
-                    term = [k]
-                    for i,e in enumerate(powers):
-                        term.append(Pow(p[i], powers[i]))
-                    l.append(Mul(*term))
-                # sum up all terms
-                result = Add(*l)
+                from sympy.polys.polynomial import multinomial_as_basic
+                result = multinomial_as_basic(expansion_dict, *p)
                 return result
             else:
                 if n == 2:
@@ -409,8 +394,8 @@ class Pow(Basic, ArithMeths, RelMeths):
     @cacheit
     def count_ops(self, symbolic=True):
         if symbolic:
-            return Add(*[t.count_ops(symbolic) for t in self[:]]) + Symbol('POW')
-        return Add(*[t.count_ops(symbolic) for t in self.args[:]]) + 1
+            return Add(*[t.count_ops(symbolic) for t in self.args]) + Symbol('POW')
+        return Add(*[t.count_ops(symbolic) for t in self.args]) + 1
 
     def _eval_is_polynomial(self, syms):
         if self.exp.has(*syms):
@@ -428,10 +413,23 @@ class Pow(Basic, ArithMeths, RelMeths):
         base, exp = self.as_base_exp()
         c,t = exp.as_coeff_terms()
         n,d = base.as_numer_denom()
-        if c.is_negative:
+        negate = False
+        if exp.is_integer != True:
+            if d.is_negative == True:
+                # Roots need to take care that negative denominators behave
+                # differently than the rest of the complex plane.
+                negate = True
+            elif d.is_negative is None:
+                # Can make no conclusions.
+                return self, S(1)
+        if c.is_negative == True:
             exp = -exp
             n,d = d,n
-        return n ** exp, d ** exp
+        num = n ** exp
+        den = d ** exp
+        if negate:
+            num = -num
+        return num, den
 
     def matches(pattern, expr, repl_dict={}, evaluate=False):
         if evaluate:
@@ -461,35 +459,7 @@ class Pow(Basic, ArithMeths, RelMeths):
             return Basic.matches(pattern, expr, repl_dict, evaluate)
         return d
 
-    def _eval_oseries(self, order):
-        """
-        f**g + O(h) == (f+O(k))**g -> .. -> f**g + O(f**(g-1)*k), hence O(k)==O(h*f**(1-g)).
-        If f->0 as x->0 then
-        """
-        x = order.symbols[0]
-        e = self.exp
-        b = self.base
-        ln = C.log
-        exp = C.exp
-        if e.has(x):
-            return exp(e * ln(b)).oseries(order)
-        if b==x: return self
-        b0 = b.limit(x,0)
-        if b0 is S.Zero or b0.is_unbounded:
-            lt = b.as_leading_term(x)
-            o = order * lt**(1-e)
-            bs = b.oseries(o)
-            if bs.is_Add:
-                # bs -> lt + rest -> lt * (1 + (bs/lt - 1))
-                return lt**e * ((bs/lt).expand()**e).oseries(order * lt**(-e))
-            return bs**e
-        o = order * (b0**-e)
-        # b -> b0 + (b-b0) -> b0 * (1 + (b/b0-1))
-        z = (b/b0-1)
-        r = self._compute_oseries(z, o, self.taylor_term, lambda z: 1+z) * b0**e
-        return r
-
-    def nseries(self, x, x0, n):
+    def _eval_nseries(self, x, x0, n):
         def geto(e):
             "Returns the O(..) symbol, or None if there is none."
             if e.is_Order:
@@ -563,11 +533,13 @@ class Pow(Basic, ArithMeths, RelMeths):
                         p = p.subs(y, -1/log(x))
                         return p
                 prefactor = base.as_leading_term(x)
-                rest = (base/prefactor).expand()
                 # express "rest" as: rest = 1 + k*x**l + ... + O(x**n)
-                rest = rest - 1
+                rest = ((base-prefactor)/prefactor).expand()
                 if rest == 0:
-                    return 1/prefactor
+                    # if prefactor == w**4 + x**2*w**4 + 2*x*w**4, we need to
+                    # factor the w**4 out using collect:
+                    from sympy import collect
+                    return 1/collect(prefactor, x)
                 if rest.is_Order:
                     return ((1+rest)/prefactor).expand()
                 if not rest.has(x):
@@ -575,13 +547,15 @@ class Pow(Basic, ArithMeths, RelMeths):
                 n2 = getn(rest)
                 if n2 is not None:
                     n = n2
-                term2 = rest.as_leading_term(x)
+
+                from sympy import collect
+                term2 = collect(rest.as_leading_term(x), x)
                 k, l = Wild("k"), Wild("l")
                 r = term2.match(k*x**l)
                 k, l = r[k], r[l]
-                if l.is_Integer:
+                if l.is_Integer and l>0:
                     l = int(l)
-                elif l.is_number:
+                elif l.is_number and l>0:
                     l = float(l)
                 else:
                     raise Exception("Not implemented")
@@ -614,7 +588,61 @@ class Pow(Basic, ArithMeths, RelMeths):
         if base == x:
             return self
 
-        return self.series(x, x0, n)
+        order = C.Order(x**n, x)
+        x = order.symbols[0]
+        e = self.exp
+        b = self.base
+        ln = C.log
+        exp = C.exp
+        if e.has(x):
+            return exp(e * ln(b)).nseries(x, x0, n)
+        if b==x: return self
+        b0 = b.limit(x,0)
+        if b0 is S.Zero or b0.is_unbounded:
+            lt = b.as_leading_term(x)
+            o = order * lt**(1-e)
+            bs = b.nseries(x, x0, n-e)
+            if bs.is_Add:
+                bs = bs.removeO()
+            if bs.is_Add:
+                # bs -> lt + rest -> lt * (1 + (bs/lt - 1))
+                return (lt**e * ((bs/lt).expand()**e).nseries(x,
+                        x0, n-e)).expand() + order
+            return bs**e+order
+        o2 = order * (b0**-e)
+        # b -> b0 + (b-b0) -> b0 * (1 + (b/b0-1))
+        z = (b/b0-1)
+        #r = self._compute_oseries3(z, o2, self.taylor_term)
+        x = o2.symbols[0]
+        ln = C.log
+        o = C.Order(z, x)
+        if o is S.Zero:
+            r = (1+z)
+        else:
+            if o.expr.is_number:
+                e2 = ln(o2.expr*x)/ln(x)
+            else:
+                e2 = ln(o2.expr)/ln(o.expr)
+            n = e2.limit(x,0) + 1
+            if n.is_unbounded:
+                # requested accuracy gives infinite series,
+                # order is probably nonpolynomial e.g. O(exp(-1/x), x).
+                r = (1+z)
+            else:
+                try:
+                    n = int(n)
+                except TypeError:
+                    #well, the n is something more complicated (like 1+log(2))
+                    n = int(n.evalf()) + 1
+                assert n>=0,`n`
+                l = []
+                g = None
+                for i in xrange(n+2):
+                    g = self.taylor_term(i, z, g)
+                    g = g.nseries(x, x0, n)
+                    l.append(g)
+                r = Add(*l)
+        return r * b0**e + order
 
     def _eval_as_leading_term(self, x):
         if not self.exp.has(x):
@@ -636,10 +664,6 @@ import basic as _
 _.Pow =     Pow
 del _
 
-import methods as _
-_.Pow =     Pow
-del _
-
 import mul as _
 _.Pow =     Pow
 del _
@@ -647,3 +671,4 @@ del _
 import numbers as _
 _.Pow =     Pow
 del _
+
