@@ -1,7 +1,6 @@
 
 from basic import Basic, C
 from sympify import _sympify
-from methods import NoRelMeths
 
 from numbers import Number
 
@@ -19,7 +18,7 @@ def Rel(a, b, op):
     """
     return Relational(a,b,op)
 
-def Eq(a, b):
+def Eq(a, b=0):
     """
     A handy wrapper around the Relational class.
     Eq(a,b)
@@ -41,7 +40,7 @@ def Ne(a, b):
     Example:
     >>> from sympy import *
     >>> x,y = symbols('xy')
-    >>> Eq(y, x+x**2)
+    >>> Ne(y, x+x**2)
     y != x + x**2
 
     """
@@ -55,7 +54,7 @@ def Lt(a, b):
     Example:
     >>> from sympy import *
     >>> x,y = symbols('xy')
-    >>> Eq(y, x+x**2)
+    >>> Lt(y, x+x**2)
     y < x + x**2
 
     """
@@ -69,7 +68,7 @@ def Le(a, b):
     Example:
     >>> from sympy import *
     >>> x,y = symbols('xy')
-    >>> Eq(y, x+x**2)
+    >>> Le(y, x+x**2)
     y <= x + x**2
 
     """
@@ -83,7 +82,7 @@ def Gt(a, b):
     Example:
     >>> from sympy import *
     >>> x,y = symbols('xy')
-    >>> Eq(y, x+x**2)
+    >>> Gt(y, x+x**2)
     x + x**2 < y
 
     """
@@ -97,13 +96,13 @@ def Ge(a, b):
     Example:
     >>> from sympy import *
     >>> x,y = symbols('xy')
-    >>> Eq(y, x+x**2)
+    >>> Ge(y, x+x**2)
     x + x**2 <= y
 
     """
     return Relational(a,b,'>=')
 
-class Relational(Basic, NoRelMeths):
+class Relational(Basic):
 
     __slots__ = []
 
@@ -136,19 +135,8 @@ class Relational(Basic, NoRelMeths):
     def rhs(self):
         return self._args[1]
 
-    @property
-    def precedence(self):
-        return 20
-
-    def tostr(self, level=0):
-        precedence = self.precedence
-        r = '%s %s %s' % (self.lhs.tostr(precedence), self.rel_op, self.rhs.tostr(precedence))
-        if precedence <= level:
-            return '(%s)' % r
-        return r
-
-    def subs(self, old, new):
-        return self.__class__(self.lhs.subs(old, new), self.rhs.subs(old, new))
+    def _eval_subs(self, old, new):
+        return self.__class__(self.lhs._eval_subs(old, new), self.rhs._eval_subs(old, new))
 
 class Equality(Relational):
 
@@ -196,9 +184,9 @@ class Inequality(Relational):
 
 
 # /cyclic/
-import methods
-methods.Equality    = Equality
-methods.Unequality  = Unequality
-methods.Inequality  = Inequality
-methods.StrictInequality = StrictInequality
-del methods
+import basic as _
+_.Equality      = Equality
+_.Unequality    = Unequality
+_.Inequality    = Inequality
+_.StrictInequality = StrictInequality
+del _
