@@ -1,11 +1,12 @@
 
 from basic import Basic, Atom, S, C, sympify
-from methods import RelMeths, ArithMeths
 from cache import cacheit
 
 # from function import Function, WildFunction   /cyclic/
 
-class Symbol(Atom, RelMeths, ArithMeths):
+import re
+
+class Symbol(Atom):
     """
     Assumptions::
        real = True
@@ -21,7 +22,7 @@ class Symbol(Atom, RelMeths, ArithMeths):
 
     is_comparable = False
 
-    __slots__ = ['name']
+    __slots__ = ['is_commutative', 'name']
 
     is_Symbol = True
 
@@ -46,32 +47,26 @@ class Symbol(Atom, RelMeths, ArithMeths):
             return Symbol.__xnew_cached_(cls, name, commutative, **assumptions)
 
     def __new_stage2__(cls, name, commutative=True, **assumptions):
-        obj = Basic.__new__(cls,
-                            commutative=commutative,
-                            **assumptions)
         assert isinstance(name, str),`type(name)`
+        obj = Basic.__new__(cls, **assumptions)
+        obj.is_commutative = commutative
         obj.name = name
         return obj
 
     __xnew__       = staticmethod(__new_stage2__)            # never cached (e.g. dummy)
     __xnew_cached_ = staticmethod(cacheit(__new_stage2__))   # symbols are always cached
 
+    def __getnewargs__(self):
+        return (self.name, self.is_commutative)
+
     def _hashable_content(self):
-        return (self.name,)
-
-    def tostr(self, level=0):
-        return self.name
-
-    def torepr(self):
-        return '%s(%r)' % (self.__class__.__name__, self.name)
+        return (self.is_commutative, self.name)
 
     def as_dummy(self):
-        assumptions = self._assumptions.copy()
-        return Dummy(self.name, **assumptions)
+        return Dummy(self.name, **self.assumptions0)
 
     def __call__(self, *args):
-        assumptions = self._assumptions
-        return Function(self.name, nargs=len(args))(*args, **assumptions)
+        return Function(self.name, nargs=len(args))(*args, **self.assumptions0)
 
     def _eval_expand_complex(self, *args):
         return C.re(self) + C.im(self)*S.ImaginaryUnit
@@ -107,10 +102,7 @@ class Dummy(Symbol):
         return obj
 
     def _hashable_content(self):
-        return (self.name, self.dummy_index)
-
-    def tostr(self, level=0):
-        return '_' + self.name
+        return Symbol._hashable_content(self) + (self.dummy_index,)
 
 
 class Temporary(Dummy):
@@ -124,33 +116,45 @@ class Temporary(Dummy):
         obj = Dummy.__new__(cls, 'T%i' % Dummy.dummycount, **assumptions)
         return obj
 
+    def __getnewargs__(self):
+        return ()
+
 
 class Wild(Symbol):
     """
     Wild() matches any expression but another Wild().
     """
 
-    __slots__ = ['exclude']
+    __slots__ = ['exclude', 'properties']
 
-    def __new__(cls, name, exclude=None, **assumptions):
+    def __new__(cls, name, exclude=None, properties=None, **assumptions):
         if type(exclude) is list:
             exclude = tuple(exclude)
+        if type(properties) is list:
+            properties = tuple(properties)
 
-        return Wild.__xnew__(cls, name, exclude, **assumptions)
+        return Wild.__xnew__(cls, name, exclude, properties, **assumptions)
+
+    def __getnewargs__(self):
+        return (self.name, self.exclude, self.properties)
 
     @staticmethod
     @cacheit
-    def __xnew__(cls, name, exclude, **assumptions):
+    def __xnew__(cls, name, exclude, properties, **assumptions):
         obj = Symbol.__xnew__(cls, name, **assumptions)
 
         if exclude is None:
             obj.exclude = None
         else:
             obj.exclude = tuple([sympify(x) for x in exclude])
+        if properties is None:
+            obj.properties = None
+        else:
+            obj.properties = tuple(properties)
         return obj
 
     def _hashable_content(self):
-        return (self.name, self.exclude)
+        return (self.name, self.exclude, self.properties )
 
     # TODO add check against another Wild
     def matches(pattern, expr, repl_dict={}, evaluate=False):
@@ -164,6 +168,10 @@ class Wild(Symbol):
                     return None
                 #else:
                 #    print x, expr, pattern, expr, pattern.exclude
+        if pattern.properties:
+            for f in pattern.properties:
+                if not f(expr):
+                    return None
         repl_dict = repl_dict.copy()
         repl_dict[pattern] = expr
         return repl_dict
@@ -171,82 +179,50 @@ class Wild(Symbol):
     def __call__(self, *args, **assumptions):
         return WildFunction(self.name, nargs=len(args))(*args, **assumptions)
 
-    def tostr(self, level=0):
-        return self.name + '_'
-
 
 def symbols(*names, **kwargs):
-    """Returns a list of symbols with names taken from 'names'
-       argument, which can be a string, then each character
-       forms a separate symbol, or a sequence of strings.
-
-       All newly created symbols have assumptions set accordingly
-       to 'kwargs'. Main intention behind this function is to
-       simplify and shorten examples code in doc-strings.
-
-       >>> x, y, z = symbols('xyz')
-
-       >>> a = symbols('a', integer=True)
-
-       >>> a.is_integer
-       True
-
-       >>> xx, yy, zz = symbols('xx', 'yy', 'zz', real=True)
-
-       >>> xx.is_real and yy.is_real and zz.is_real
-       True
-
     """
-    if len(names) == 1:
-        result = [ Symbol(name, **kwargs) for name in names[0] ]
-    else:
-        result = [ Symbol(name, **kwargs) for name in names ]
+    Return a list of symbols with names taken from 'names'
+    argument, which can be a string, then each character
+    forms a separate symbol, or a sequence of strings.
 
-    if len(result) == 1:
-        return result[0]
-    else:
-        return result
+    >>> x, y, z = symbols('xyz')
 
-def var(s):
+    Please note that this syntax is deprecated and will be dropped in a
+    future version of sympy. Use comma or whitespace separated characters
+    instead. Currently the old behavoiur is standard, this can be changed
+    using the 'each_char' keyword:
+
+    >>> symbols('xyz', each_char=False)
+    xyz
+
+    All newly created symbols have assumptions set accordingly
+    to 'kwargs'. Main intention behind this function is to
+    simplify and shorten examples code in doc-strings.
+
+    >>> a = symbols('a', integer=True)
+    >>> a.is_integer
+    True
+    >>> xx, yy, zz = symbols('xx', 'yy', 'zz', real=True)
+    >>> xx.is_real and yy.is_real and zz.is_real
+    True
     """
-        Create a symbolic variable with the name *s*.
-
-        INPUT:
-            s -- a string, either a single variable name, or
-                 a space separated list of variable names, or
-                 a list of variable names.
-
-        NOTE: The new variable is both returned and automatically injected into
-        the parent's *global* namespace.  It's recommended not to use "var" in
-        library code, it is better to use symbols() instead.
-
-        EXAMPLES:
-        We define some symbolic variables:
-            >>> var('m')
-            m
-            >>> var('n xx yy zz')
-            (n, xx, yy, zz)
-            >>> n
-            n
-
-    """
-    import inspect
-    frame = inspect.currentframe().f_back
-
-    try:
+    # use new behavior if space or comma in string
+    if not 'each_char' in kwargs and len(names) == 1 and \
+    isinstance(names[0], str) and (' ' in names[0] or ',' in names[0]):
+        kwargs['each_char'] = False
+    if not kwargs.pop("each_char", True):
+        # the new way:
+        s = names[0]
         if not isinstance(s, list):
-            s = s.split(" ")
-
+            s = re.split('\s|,', s)
         res = []
-
         for t in s:
             # skip empty strings
             if not t:
                 continue
-            sym = Symbol(t)
-            frame.f_globals[t] = sym
+            sym = Symbol(t, **kwargs)
             res.append(sym)
-
         res = tuple(res)
         if len(res) == 0:   # var('')
             res = None
@@ -254,7 +230,54 @@ def var(s):
             res = res[0]
                             # otherwise var('a b ...')
         return res
+    else:
+        # this is the old, deprecated behavior:
+        if len(names) == 1:
+            result = [ Symbol(name, **kwargs) for name in names[0] ]
+        else:
+            result = [ Symbol(name, **kwargs) for name in names ]
+        if len(result) == 1:
+            return result[0]
+        else:
+            return result
 
+def var(*names, **kwargs):
+    """
+    Create symbols and inject them into global namespace.
+
+    This calls symbols() with the same arguments and puts the results into
+    global namespace. Unlike symbols(), it uses each_char=False by default
+    for compatibility reasons.
+
+    NOTE: The new variable is both returned and automatically injected into
+    the parent's *global* namespace.  It's recommended not to use "var" in
+    library code, it is better to use symbols() instead.
+
+    >>> var('m')
+    m
+    >>> var('n xx yy zz')
+    (n, xx, yy, zz)
+    >>> n
+    n
+    >>> var('x y', real=True)
+    (x, y)
+    >>> x.is_real and y.is_real
+    True
+    """
+    import inspect
+    frame = inspect.currentframe().f_back
+    try:
+        kwargs['each_char'] = False
+        s = symbols(*names, **kwargs)
+        if s is None:
+            return s
+        if isinstance(s, Symbol):
+            s_list = [s]
+        else:
+            s_list = s
+        for t in s_list:
+            frame.f_globals[t.name] = t
+        return s
     finally:
         # we should explicitly break cyclic dependencies as stated in inspect
         # doc
