@@ -1,5 +1,5 @@
 from sympy import Symbol, sin, cos, exp, O, sqrt, Rational, Real, re, pi, \
-        sympify, sqrt, Add, Mul, Pow
+        sympify, sqrt, Add, Mul, Pow, I
 from sympy.utilities.pytest import XFAIL
 
 x = Symbol("x")
@@ -15,8 +15,6 @@ b = Symbol("b", positive=True)
 c = Symbol("c")
 
 def test_Symbol():
-    assert str(a)=="a"
-    assert str(b)=="b"
     e=a*b
     assert e==a*b
     assert a*b*b==a*b**2
@@ -72,7 +70,7 @@ def test_arit0():
     e = Rational(1,2)*(1+a)
     assert e == Rational(1,2) + a/2
 
-def testdiv():
+def test_div():
     e=a/b
     assert e == a*b**(-1)
     e=a/b+c/2
@@ -80,7 +78,7 @@ def testdiv():
     e=(1-b)/(b-1)
     assert e == (1+-b)*((-1)+b)**(-1)
 
-def testpow():
+def test_pow():
     n1 = Rational(1)
     n2 = Rational(2)
     n5 = Rational(5)
@@ -185,8 +183,8 @@ def test_expand():
     assert e.expand() == 5*a+5*b+5*c+2*a*c+b*c+a*b+a**2+c**2
     x=Symbol("x")
     s=exp(x*x)-1
-    e=s.series(x,0,5)/x**2
-    assert e.expand() ==  1+x**2/2+O(x**3)
+    e=s.series(x,0,3)/x**2
+    assert e.expand() ==  1+x**2/2+O(x**4)
 
     # Check that this isn't too slow
     x = Symbol('x')
@@ -303,8 +301,9 @@ def test_Mul_is_even_odd():
     assert (3*x).is_even == None
     assert (3*x).is_odd == None
 
-    assert (k/3).is_even == None
-    assert (k/3).is_odd == None
+    assert (k/3).is_integer == False
+    assert (k/3).is_even    == False
+    assert (k/3).is_odd     == False
 
     assert (2*n).is_even == True
     assert (2*n).is_odd == False
@@ -373,6 +372,9 @@ def test_Mul_is_negative_positive():
     assert (-k).is_negative == False
     assert (2*k).is_negative == True
 
+    assert (2*n)._eval_is_negative() == False
+    assert (2*n).is_negative == False
+
     assert n.is_negative == False
     assert (-n).is_negative == True
     assert (2*n).is_negative == False
@@ -380,7 +382,7 @@ def test_Mul_is_negative_positive():
     assert (n*k).is_negative == True
     assert (2*n*k).is_negative == True
     assert (-n*k).is_negative == False
-    assert (n*k*y).is_negative == None
+    assert (n*k*y).is_negative == False     # y.is_real=F;  !real -> !neg
 
     assert u.is_negative == False
     assert (-u).is_negative == None
@@ -423,7 +425,7 @@ def test_Mul_is_negative_positive():
     assert (n*k).is_positive == False
     assert (2*n*k).is_positive == False
     assert (-n*k).is_positive == True
-    assert (-n*k*y).is_positive == None
+    assert (-n*k*y).is_positive == False    # y.is_real=F;  !real -> !neg
 
     assert u.is_positive == None
     assert (-u).is_positive == False
@@ -455,6 +457,28 @@ def test_Mul_is_negative_positive():
     assert (x*k).is_positive == None
     assert (u*v*n*x*k).is_positive == None
 
+def test_Mul_is_negative_positive_2():
+    a = Symbol('a', nonnegative=True)
+    b = Symbol('b', nonnegative=True)
+    c = Symbol('c', nonpositive=True)
+    d = Symbol('d', nonpositive=True)
+
+    assert (a*b).is_nonnegative == True
+    assert (a*b).is_negative    == False
+    assert (a*b).is_zero        == None
+    assert (a*b).is_positive    == None
+
+    assert (c*d).is_nonnegative == True
+    assert (c*d).is_negative    == False
+    assert (c*d).is_zero        == None
+    assert (c*d).is_positive    == None
+
+    assert (a*c).is_nonpositive == True
+    assert (a*c).is_positive    == False
+    assert (a*c).is_zero        == None
+    assert (a*c).is_negative    == None
+
+
 def test_Mul_is_nonpositive_nonnegative():
     x = Symbol('x', real=True)
 
@@ -738,8 +762,8 @@ def test_Pow_is_integer():
     x = Symbol('x')
 
     k = Symbol('k', integer=True)
-    n = Symbol('n', nni=True)
-    m = Symbol('m', pi=True)
+    n = Symbol('n', integer=True, nonnegative=True)
+    m = Symbol('m', integer=True, positive=True)
 
     assert (k**2).is_integer == True
     assert (k**(-2)).is_integer == False
@@ -775,8 +799,10 @@ def test_Pow_is_real():
     assert (x**x).is_real   == None
     assert (y**x).is_real   == True
 
-    assert (x**Rational(1,3)).is_real   == None
-    assert (y**Rational(1,3)).is_real   == True
+    assert (x**Rational(1,3)).is_real  == None
+    assert (y**Rational(1,3)).is_real  == True
+
+    assert sqrt(-1 - sqrt(2)).is_real == False
 
 @XFAIL
 def test_Pow_is_bounded():
@@ -797,11 +823,12 @@ def test_Pow_is_even_odd():
 
     k = Symbol('k', even=True)
     n = Symbol('n', odd=True)
-    m = Symbol('m', nni=True)
+    m = Symbol('m', integer=True, nonnegative=True)
 
     assert (k**2).is_even == True
     assert (n**2).is_even == False
     assert (2**k).is_even == None
+    assert (x**2).is_even == None
 
     assert (k**m).is_even == True
     assert (n**m).is_even == False
@@ -822,7 +849,7 @@ def test_Pow_is_even_odd():
 def test_Pow_is_negative_positive():
     x = Symbol('x', real=True)
 
-    k = Symbol('k', pi=True)
+    k = Symbol('k', integer=True, positive=True)
     n = Symbol('n', even=True)
     m = Symbol('m', odd=True)
 
@@ -860,7 +887,8 @@ def test_Pow_is_negative_positive():
 def test_Pow_is_nonpositive_nonnegative():
     x = Symbol('x', real=True)
 
-    k = Symbol('k', nni=True)
+    k = Symbol('k', integer=True, nonnegative=True)
+    l = Symbol('l', integer=True, positive=True)
     n = Symbol('n', even=True)
     m = Symbol('m', odd=True)
 
@@ -872,7 +900,9 @@ def test_Pow_is_nonpositive_nonnegative():
     assert (k**2).is_nonnegative == True
     assert (k**(-2)).is_nonnegative == True
 
-    assert (k**x).is_nonnegative == True
+    assert (k**x).is_nonnegative == None    # NOTE (0**x).is_real = U
+    assert (l**x).is_nonnegative == True
+    assert (l**x).is_positive    == True
     assert ((-k)**x).is_nonnegative == None
     assert ((-k)**n).is_nonnegative == True
     assert ((-k)**m).is_nonnegative == None
@@ -890,6 +920,36 @@ def test_Pow_is_nonpositive_nonnegative():
     assert ((-k)**n).is_nonpositive == None
     assert ((-k)**m).is_nonpositive == True
 
+
+def test_Mul_is_imaginary_real():
+    r = Symbol('r', real=True)
+    i = Symbol('i', imaginary=True)
+    ii= Symbol('ii',imaginary=True)
+    x = Symbol('x')
+
+    assert    I  .is_imaginary  == True
+    assert    I  .is_real       == False
+    assert  (-I) .is_imaginary  == True
+    assert  (-I) .is_real       == False
+    assert (3*I) .is_imaginary  == True
+    assert (3*I) .is_real       == False
+    assert (I*I) .is_imaginary  == False
+    assert (I*I) .is_real       == True
+
+    assert (r*i) .is_imaginary  == True
+    assert (r*i) .is_real       == False
+
+    assert (x*i) .is_imaginary  == None
+    assert (x*i) .is_real       == None
+
+    assert (i*ii).is_imaginary  == False
+    assert (i*ii).is_real       == True
+
+    assert (r*i*ii).is_imaginary == False
+    assert (r*i*ii).is_real      == True
+
+
+
 def test_Add_is_comparable():
     assert (x+y).is_comparable  == False
     assert (x+1).is_comparable  == False
@@ -930,6 +990,9 @@ def test_issue432():
     class MightyNumeric(tuple):
         def __rdiv__(self, other):
             return "something"
+
+        def __rtruediv__(self, other):
+            return "something"
     assert sympify(1)/MightyNumeric((1,2)) == "something"
 
 def test_issue432b():
@@ -964,3 +1027,27 @@ def test_suppressed_evaluation():
     assert c != 9
     assert c.func is Pow
     assert c.args == (3,2)
+
+
+def test_Add_as_coeff_terms():
+    assert (x+1).as_coeff_terms()   == ( 1, (x+1,) )
+    assert (x+2).as_coeff_terms()   == ( 1, (x+2,) )
+    assert (x+3).as_coeff_terms()   == ( 1, (x+3,) )
+
+    assert (x-1).as_coeff_terms()   == (-1, (1-x,) )
+    assert (x-2).as_coeff_terms()   == (-1, (2-x,) )
+    assert (x-3).as_coeff_terms()   == (-1, (3-x,) )
+
+    n = Symbol('n', integer=True)
+    assert (n+1).as_coeff_terms()   == ( 1, (n+1,) )
+    assert (n+2).as_coeff_terms()   == ( 1, (n+2,) )
+    assert (n+3).as_coeff_terms()   == ( 1, (n+3,) )
+
+    assert (n-1).as_coeff_terms()   == (-1, (1-n,) )
+    assert (n-2).as_coeff_terms()   == (-1, (2-n,) )
+    assert (n-3).as_coeff_terms()   == (-1, (3-n,) )
+
+
+def test_issue974():
+    assert -1/(-1-x)    == 1/(1+x)
+
