@@ -1,5 +1,5 @@
 from sympy import abc, Function, Symbol, Wild, Derivative, sin, cos, Real, \
-        Rational, exp, I, Integer, diff
+        Rational, exp, I, Integer, diff, Mul, var, oo
 from sympy.utilities.pytest import XFAIL
 
 
@@ -161,12 +161,14 @@ def test_derivative1():
     assert (fd+1).match(p+1) == {p: fd}
     assert (fd).match(fd) == {}
     assert (3*fd).match(p*fd) != None
+    p = Wild("p", exclude=[x])
+    q = Wild("q", exclude=[x])
     assert (3*fd-1).match(p*fd + q) == {p: 3, q: -1}
 
 def test_derivative_bug1():
     f = Function("f")
     x = Symbol("x")
-    a = Wild("a", exclude=[f])
+    a = Wild("a", exclude=[f, x])
     b = Wild("b", exclude=[f])
     pattern = a * Derivative(f(x), x, x) + b
     expr = Derivative(f(x), x)+x**2
@@ -177,7 +179,7 @@ def test_derivative_bug1():
 def test_derivative2():
     f = Function("f")
     x = Symbol("x")
-    a = Wild("a", exclude=[f])
+    a = Wild("a", exclude=[f, x])
     b = Wild("b", exclude=[f])
     e = Derivative(f(x), x)
     assert e.match(Derivative(f(x), x)) == {}
@@ -300,3 +302,34 @@ def test_Derivative_bug1():
     b = Wild("b", exclude=[f(x)])
     eq = f(x).diff(x)
     assert eq.match(a*Derivative(f(x), x) + b) == {a: 1, b: 0}
+
+
+def test_match_wild_wild():
+    p = Wild('p')
+    q = Wild('q')
+    r = Wild('r')
+
+    assert p.match(q+r)  in  [ {q: p, r: 0} , {q: 0, r: p} ]
+    assert p.match(q*r)  in  [ {q: p, r: 1} , {q: 1, r: p} ]
+
+
+    p = Wild('p')
+    q = Wild('q', exclude=[p])
+    r = Wild('r')
+
+    assert p.match(q+r) == {q: 0, r: p}
+    assert p.match(q*r) == {q: 1, r: p}
+
+    p = Wild('p')
+    q = Wild('q', exclude=[p])
+    r = Wild('r', exclude=[p])
+
+    assert p.match(q+r) == None
+    assert p.match(q*r) == None
+
+def test_combine_inverse():
+    x, y = var("x y")
+    assert Mul._combine_inverse(x*I*y, x*I) == y
+    assert Mul._combine_inverse(x*I*y, y*I) == x
+    assert Mul._combine_inverse(oo*I*y, y*I) == oo
+    assert Mul._combine_inverse(oo*I*y, oo*I) == y
