@@ -1,5 +1,5 @@
 from sympy import Rational, Symbol, Real, I, sqrt, oo, nan, pi, E, Integer, \
-        Basic, S
+        Basic, S, factorial
 from sympy.core.power import integer_nthroot
 
 from sympy.core.numbers import igcd, ilcm, igcdex
@@ -171,10 +171,11 @@ def test_powers():
     assert integer_nthroot(c2, 2) == (c, True)
     assert integer_nthroot(c2+1, 2) == (c, False)
     assert integer_nthroot(c2-1, 2) == (c-1, False)
+    assert integer_nthroot(2,10**10) == (1, False)
 
-    assert 64**(Rational(1)/3)==4
-    assert 64**(Rational(2)/3)==16
-    assert 24*64**(-Rational(1)/2)==3
+    p, r = integer_nthroot(int(factorial(10000)), 100)
+    assert p % (10**10) == 5322420655
+    assert not r
 
     assert Rational(5**3, 8**3)**Rational(4,3) == Rational(5**4, 8**4)
     assert Rational(-4,7)**Rational(1,2) == I*Rational(4,7)**Rational(1,2)
@@ -186,6 +187,9 @@ def test_powers():
     assert sqrt((3-sqrt(pi))**2) == 3 - sqrt(pi)
     assert sqrt(Rational(1,2)) == Rational(1,2) * sqrt(2)
 
+    assert (2**64+1)**Rational(4, 3)
+    assert (2**64+1)**Rational(17,25)
+
     # Test that this is fast
     assert integer_nthroot(2,10**10) == (1, False)
 
@@ -325,3 +329,4 @@ def test_IntegerInteger():
     b = Integer(a)
 
     assert a == b
+
