@@ -24,12 +24,14 @@ from elementary.trigonometric import acot, cot, tan, cos, sin, asin, acos, atan,
 from elementary.exponential import exp, log, LambertW
 from elementary.hyperbolic import sinh, cosh, tanh, coth, asinh, acosh, atanh, acoth
 from elementary.integers import floor, ceiling
+from elementary.piecewise import Piecewise
 
 from special.error_functions import erf
 from special.gamma_functions import gamma, lowergamma, uppergamma, polygamma, loggamma
 from special.zeta_functions import dirichlet_eta, zeta
 from special.spherical_harmonics import Ylm, Zlm
 from special.tensor_functions import Dij, Eijk
+from special.delta_functions import DiracDelta, Heaviside
 
 ln = log
 
