@@ -7,9 +7,11 @@ Factorials, binomial coefficients and related functions are located in
 the separate 'factorials' module.
 """
 
-from sympy.core import *
-from sympy.core.function import Function
-from sympy.core.basic import S
+from sympy import Function, S, Symbol, Rational, oo, Integer, C
+
+from sympy.mpmath import bernfrac
+
+from sympy.utilities.decorator import deprecated
 
 def _product(a, b):
     p = 1
@@ -75,7 +77,12 @@ class fibonacci(Function):
         return (prev[-2] + _sym*prev[-1]).expand()
 
     @classmethod
+    @deprecated
     def canonize(cls, n, sym=None):
+        return cls.eval(n, sym)
+
+    @classmethod
+    def eval(cls, n, sym=None):
         if n.is_Integer:
             n = int(n)
             if n < 0:
@@ -117,7 +124,12 @@ class lucas(Function):
     """
 
     @classmethod
+    @deprecated
     def canonize(cls, n):
+        return cls.eval(n)
+
+    @classmethod
+    def eval(cls, n):
         if n.is_Integer:
             return fibonacci(n+1) + fibonacci(n-1)
 
@@ -223,7 +235,12 @@ class bernoulli(Function):
     _highest = {0:0, 2:2, 4:4}
 
     @classmethod
+    @deprecated
     def canonize(cls, n, sym=None):
+        return cls.eval(n, sym)
+
+    @classmethod
+    def eval(cls, n, sym=None):
         if n.is_Number:
             if n.is_Integer and n.is_nonnegative:
                 if n is S.Zero:
@@ -236,6 +253,10 @@ class bernoulli(Function):
                     if n.is_odd:
                         return S.Zero
                     n = int(n)
+                    # Use mpmath for enormous Bernoulli numbers
+                    if n > 500:
+                        p, q = bernfrac(n)
+                        return Rational(int(p), q)
                     case = n % 6
                     highest_cached = cls._highest[case]
                     if n <= highest_cached:
@@ -343,7 +364,12 @@ class bell(Function):
         return (_sym * s).expand()
 
     @classmethod
+    @deprecated
     def canonize(cls, n, sym=None):
+        return cls.eval(n, sym)
+
+    @classmethod
+    def eval(cls, n, sym=None):
         if n.is_Integer and n.is_nonnegative:
             if sym is None:
                 return Integer(cls._bell(int(n)))
@@ -376,7 +402,7 @@ class harmonic(Function):
         >>> [harmonic(n, 2) for n in range(6)]
         [0, 1, 5/4, 49/36, 205/144, 5269/3600]
         >>> harmonic(oo, 2)
-        (1/6)*pi**2
+        pi**2/6
 
     Mathematical description
     ========================
@@ -402,7 +428,12 @@ class harmonic(Function):
     _functions = {}
 
     @classmethod
+    @deprecated
     def canonize(cls, n, m=None):
+        return cls.eval(n, m)
+
+    @classmethod
+    def eval(cls, n, m=None):
         if m is None:
             m = S.One
         if n == oo:
