@@ -5,6 +5,8 @@ from sympy.core.cache import cacheit
 
 from sympy.functions.elementary.miscellaneous import sqrt
 
+from sympy.utilities.decorator import deprecated
+
 ###############################################################################
 ########################### HYPERBOLIC FUNCTIONS ##############################
 ###############################################################################
@@ -27,7 +29,12 @@ class sinh(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return eval(cls, arg)
+
+    @classmethod
+    def eval(cls, arg):
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -52,6 +59,17 @@ class sinh(Function):
                 if coeff.is_negative:
                     return -cls(-arg)
 
+            if isinstance(arg, asinh):
+                return arg.args[0]
+
+            if isinstance(arg, acosh):
+                x = arg.args[0]
+                return sqrt(x-1) * sqrt(x+1)
+
+            if isinstance(arg, atanh):
+                x = arg.args[0]
+                return x/sqrt(1-x**2)
+
     @staticmethod
     @cacheit
     def taylor_term(n, x, *previous_terms):
@@ -127,7 +145,12 @@ class cosh(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -152,6 +175,15 @@ class cosh(Function):
                 if coeff.is_negative:
                     return cls(-arg)
 
+            if isinstance(arg, asinh):
+                return sqrt(1+arg.args[0]**2)
+
+            if isinstance(arg, acosh):
+                return arg.args[0]
+
+            if isinstance(arg, atanh):
+                return 1/sqrt(1-arg.args[0]**2)
+
     @staticmethod
     @cacheit
     def taylor_term(n, x, *previous_terms):
@@ -227,7 +259,12 @@ class tanh(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -252,6 +289,17 @@ class tanh(Function):
                 if coeff.is_negative:
                     return -cls(-arg)
 
+            if isinstance(arg, asinh):
+                x = arg.args[0]
+                return x/sqrt(1+x**2)
+
+            if isinstance(arg, acosh):
+                x = arg.args[0]
+                return sqrt(x-1) * sqrt(x+1) / x
+
+            if isinstance(arg, atanh):
+                return arg.args[0]
+
     @staticmethod
     @cacheit
     def taylor_term(n, x, *previous_terms):
@@ -329,7 +377,12 @@ class coth(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -425,7 +478,12 @@ class asinh(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -500,7 +558,12 @@ class acosh(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -579,7 +642,12 @@ class atanh(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         arg = sympify(arg)
 
         if arg.is_Number:
@@ -640,7 +708,12 @@ class acoth(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         arg = sympify(arg)
 
         if arg.is_Number:
