@@ -1,5 +1,6 @@
 from sympy import symbols, Symbol, sqrt, oo, re, nan, im, sign, I, E, log, \
-        pi, arg, conjugate
+        pi, arg, conjugate, expand
+from sympy.utilities.pytest import XFAIL
 
 
 def test_re():
@@ -91,15 +92,24 @@ def test_abs():
     assert abs(0) == 0
     assert abs(1) == 1
     assert abs(-1)== 1
-    assert sqrt(x**2) == abs(x)
     assert abs(x).diff(x) == sign(x)
     x = Symbol('x',real=True)
     n = Symbol('n',integer=True)
     assert x**(2*n) == abs(x)**(2*n)
 
+def test_abs_real():
+    # test some properties of abs that only apply
+    # to real numbers
+    x = Symbol('x', complex=True)
+    assert sqrt(x**2) != abs(x)
+    assert abs(x**2) != x**2
+
+    x = Symbol('x', real=True)
+    assert sqrt(x**2) == abs(x)
+    assert abs(x**2) == x**2
+
 def test_abs_properties():
     x = Symbol('x')
-
     assert abs(x).is_real == True
     assert abs(x).is_positive == None
     assert abs(x).is_nonnegative == True
@@ -142,3 +152,9 @@ def test_conjugate():
     assert conjugate(x * y) == conjugate(x) * conjugate(y)
     assert conjugate(x / y) == conjugate(x) / conjugate(y)
     assert conjugate(-x) == -conjugate(x)
+
+def test_issue936():
+    x = Symbol('x')
+    assert abs(x).expand(trig=True)     == abs(x)
+    assert sign(x).expand(trig=True)    == sign(x)
+    assert arg(x).expand(trig=True)     == arg(x)
