@@ -4,6 +4,8 @@ from sympy.core.function import Lambda, Function
 from miscellaneous import sqrt
 from sympy.core.cache import cacheit
 
+from sympy.utilities.decorator import deprecated
+
 ###############################################################################
 ########################## TRIGONOMETRIC FUNCTIONS ############################
 ###############################################################################
@@ -60,7 +62,12 @@ class sin(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(args)
+
+    @classmethod
+    def eval(cls, arg):
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
@@ -80,22 +87,41 @@ class sin(Function):
                     if pi_coeff.is_integer:
                         return S.Zero
                     elif pi_coeff.is_Rational:
-                        cst_table = {
+                        cst_table_some = {
                             2 : S.One,
                             3 : S.Half*sqrt(3),
                             4 : S.Half*sqrt(2),
                             6 : S.Half,
                         }
 
-                        try:
-                            result = cst_table[pi_coeff.q]
+                        cst_table_more = {
+                            (1, 5) : sqrt((5 - sqrt(5)) / 8),
+                            (2, 5) : sqrt((5 + sqrt(5)) / 8)
+                        }
 
-                            if (pi_coeff.p // pi_coeff.q) % 2 == 1:
-                                return -result
-                            else:
-                                return result
+                        p = pi_coeff.p
+                        q = pi_coeff.q
+
+                        Q, P = p // q, p % q
+
+                        try:
+                            result = cst_table_some[q]
                         except KeyError:
-                            pass
+                            if abs(P) > q // 2:
+                                P = q - P
+
+                            try:
+                                result = cst_table_more[(P, q)]
+                            except KeyError:
+                                if P != p:
+                                    result = cls(C.Rational(P, q)*S.Pi)
+                                else:
+                                    return None
+
+                        if Q % 2 == 1:
+                            return -result
+                        else:
+                            return result
 
                 if arg.is_Mul and arg.args[0].is_negative:
                     return -cls(-arg)
@@ -123,6 +149,20 @@ class sin(Function):
                             return
                         return -cls(-arg)
 
+            if isinstance(arg, asin):
+                return arg.args[0]
+
+            if isinstance(arg, atan):
+                x = arg.args[0]
+                return x / sqrt(1 + x**2)
+
+            if isinstance(arg, acos):
+                x = arg.args[0]
+                return sqrt(1 - x**2)
+
+            if isinstance(arg, acot):
+                x = arg.args[0];
+                return 1 / (sqrt(1 + 1 / x**2) * x)
 
     @staticmethod
     @cacheit
@@ -213,7 +253,7 @@ class cos(Function):
         >>> from sympy import *
         >>> x = Symbol('x')
         >>> cos(x**2).diff(x)
-        -2*sin(x**2)*x
+        -2*x*sin(x**2)
         >>> cos(1).diff(x)
         0
         >>> cos(pi)
@@ -249,7 +289,12 @@ class cos(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
@@ -267,7 +312,7 @@ class cos(Function):
 
                 if pi_coeff is not None:
                     if pi_coeff.is_Rational:
-                        cst_table = {
+                        cst_table_some = {
                             1 : S.One,
                             2 : S.Zero,
                             3 : S.Half,
@@ -275,15 +320,34 @@ class cos(Function):
                             6 : S.Half*sqrt(3),
                         }
 
-                        try:
-                            result = cst_table[pi_coeff.q]
+                        cst_table_more = {
+                            (1, 5) : (sqrt(5) + 1)/4,
+                            (2, 5) : (sqrt(5) - 1)/4
+                        }
 
-                            if (2*pi_coeff.p // pi_coeff.q) % 4 in (1, 2):
-                                return -result
-                            else:
-                                return result
+                        p = pi_coeff.p
+                        q = pi_coeff.q
+
+                        Q, P = 2*p // q, p % q
+
+                        try:
+                            result = cst_table_some[q]
                         except KeyError:
-                            pass
+                            if abs(P) > q // 2:
+                                P = q - P
+
+                            try:
+                                result = cst_table_more[(P, q)]
+                            except KeyError:
+                                if P != p:
+                                    result = cls(C.Rational(P, q)*S.Pi)
+                                else:
+                                    return None
+
+                        if Q % 4 in (1, 2):
+                            return -result
+                        else:
+                            return result
 
                 if arg.is_Mul and arg.args[0].is_negative:
                     return cls(-arg)
@@ -311,6 +375,21 @@ class cos(Function):
                             return
                         return cls(-arg)
 
+            if isinstance(arg, acos):
+                return arg.args[0]
+
+            if isinstance(arg, atan):
+                x = arg.args[0]
+                return 1 / sqrt(1 + x**2)
+
+            if isinstance(arg, asin):
+                x = arg.args[0]
+                return sqrt(1 - x ** 2)
+
+            if isinstance(arg, acot):
+                x = arg.args[0]
+                return 1 / sqrt(1 + 1 / x**2)
+
 
     @staticmethod
     @cacheit
@@ -402,7 +481,7 @@ class tan(Function):
         >>> from sympy import *
         >>> x = Symbol('x')
         >>> tan(x**2).diff(x)
-        2*x*cos(x**2)**(-2)
+        2*x*(1 + tan(x**2)**2)
         >>> tan(1).diff(x)
         0
 
@@ -432,7 +511,12 @@ class tan(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
@@ -474,6 +558,21 @@ class tan(Function):
                 if coeff.is_negative:
                     return -cls(-arg)
 
+        if isinstance(arg, atan):
+            return arg.args[0]
+
+        if isinstance(arg, asin):
+            x = arg.args[0]
+            return x / sqrt(1 - x**2)
+
+        if isinstance(arg, acos):
+            x = arg.args[0]
+            return sqrt(1 - x**2) / x
+
+        if isinstance(arg, acot):
+            x = arg.args[0]
+            return 1 / x
+
 
     @staticmethod
     @cacheit
@@ -562,7 +661,12 @@ class cot(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
@@ -604,6 +708,21 @@ class cot(Function):
                 if coeff.is_negative:
                     return -cls(-arg)
 
+        if isinstance(arg, acot):
+            return arg.args[0]
+
+        if isinstance(arg, atan):
+            x = arg.args[0]
+            return 1 / x
+
+        if isinstance(arg, asin):
+            x = arg.args[0]
+            return sqrt(1 - x**2) / x
+
+        if isinstance(arg, acos):
+            x = arg.args[0]
+            return x / sqrt(1 - x**2)
+
 
     @staticmethod
     @cacheit
@@ -685,7 +804,12 @@ class asin(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
@@ -782,7 +906,12 @@ class acos(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
@@ -869,7 +998,12 @@ class atan(Function):
         return
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
@@ -951,7 +1085,12 @@ class acot(Function):
             raise ArgumentIndexError(self, argindex)
 
     @classmethod
+    @deprecated
     def canonize(cls, arg):
+        return cls.eval(arg)
+
+    @classmethod
+    def eval(cls, arg):
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
@@ -1029,7 +1168,12 @@ class atan2(Function):
     nargs = 2
 
     @classmethod
+    @deprecated
     def canonize(cls, y, x):
+        return cls.eval(y, x)
+
+    @classmethod
+    def eval(cls, y, x):
         sign_y = C.sign(y)
 
         if y.is_zero:
@@ -1053,3 +1197,4 @@ class atan2(Function):
 
     def _eval_is_real(self):
         return self.args[0].is_real and self.args[1].is_real
+
