@@ -20,7 +20,7 @@ def intersection(*entities):
         >>> intersection(c, Point(1, 0))
         [Point(1, 0)]
         >>> intersection(c, l2)
-        [Point(1 - 1/5*5**(1/2), 1 + (2/5)*5**(1/2)), Point(1 + (1/5)*5**(1/2), 1 - 2/5*5**(1/2))]
+        [Point(1 - 5**(1/2)/5, 1 + 2*5**(1/2)/5), Point(1 + 5**(1/2)/5, 1 - 2*5**(1/2)/5)]
 
     Notes:
     ======
@@ -153,4 +153,4 @@ def are_similar(e1, e2):
         except AttributeError:
             n1 = e1.__class__.__name__
             n2 = e2.__class__.__name__
-            raise GeometryError("Cannot test similarity between %s and %s" % (n1, n2))
\ No newline at end of file
+            raise GeometryError("Cannot test similarity between %s and %s" % (n1, n2))
