@@ -3,6 +3,7 @@ from sympy.core import Basic, S, C, Symbol, Wild, Pow, sympify, diff
 
 from sympy.integrals.trigonometry import trigintegrate
 from sympy.integrals.deltafunctions import deltaintegrate
+from sympy.integrals.rationaltools import ratint
 from sympy.integrals.risch import heurisch
 from sympy.utilities import threaded
 from sympy.simplify import apart
@@ -10,6 +11,7 @@ from sympy.series import limit
 from sympy.polys import Poly
 from sympy.solvers import solve
 from sympy.functions import DiracDelta, Heaviside, Piecewise
+from sympy.geometry import Curve
 
 class Integral(Basic):
     """Represents unevaluated integral."""
@@ -34,9 +36,10 @@ class Integral(Basic):
                     continue
                 elif isinstance(V, (tuple, list)):
                     if len(V) == 3:
-                        nlim = map(sympify, V[1:])
-                        limits.append( (V[0], tuple(nlim) ))
-                        continue
+                        if isinstance(V[0], Symbol):
+                            nlim = map(sympify, V[1:])
+                            limits.append( (V[0], tuple(nlim) ))
+                            continue
                     elif len(V) == 1 or (len(V) == 2 and V[1] is None):
                         if isinstance(V[0], Symbol):
                             limits.append((V[0],None))
@@ -55,6 +58,10 @@ class Integral(Basic):
 
         return obj
 
+    def __getnewargs__(self):
+        function, limits = self.args
+        return (function,) + limits
+
     @property
     def function(self):
         return self._args[0]
@@ -281,8 +288,7 @@ class Integral(Basic):
             # g(x) = -------
             #        poly(x)
             if g.is_fraction(x):
-                h = self._eval_integral(apart(g, x), x)
-                parts.append(coeff * h)
+                parts.append(coeff * ratint(g, x))
                 continue
 
             # g(x) = Mul(trig)
@@ -327,6 +333,19 @@ class Integral(Basic):
         else:
             return integrate(arg.removeO(), x) + arg.getO()*x
 
+    def _eval_subs(self, old, new):
+        arg0 = self.args[0].subs(old, new)
+        arg1 = []
+        for sym, limits in self.args[1]:
+            if sym == old:
+                return self
+            if limits is not None:
+                a, b, = limits
+                arg1.append((sym, a.subs(old, new), b.subs(old, new)))
+            else:
+                arg1.append((sym, limits))
+        return Integral(arg0, *arg1)
+
 @threaded(use_add=False)
 def integrate(*args, **kwargs):
     """integrate(f, var, ...)
@@ -349,7 +368,6 @@ def integrate(*args, **kwargs):
        returned. This is equivalent of integrating f over all it's variables.
 
        Examples
-       --------
 
        >>> from sympy import *
        >>> x, y = symbols('xy')
@@ -377,3 +395,48 @@ def integrate(*args, **kwargs):
         return integral.doit()
     else:
         return integral
+
+
+@threaded(use_add=False)
+def line_integrate(field, curve, vars):
+    """line_integrate(field, Curve, variables)
+
+       Compute the line integral.
+
+       Examples
+       --------
+       >>> from sympy import *
+       >>> x, y, t = symbols('xyt')
+       >>> C = Curve([E**t + 1, E**t - 1], (t, 0, ln(2)))
+       >>> line_integrate(x + y, C, [x, y])
+       3*sqrt(2)
+
+    """
+    F = sympify(field)
+    if not F:
+        raise ValueError("Expecting function specifying field as first argument.")
+    if not isinstance(curve, Curve):
+        raise ValueError("Expecting Curve entity as second argument.")
+    if not isinstance(vars, (list, tuple)):
+        raise ValueError("Expecting list/tuple for variables.")
+    if len(curve.functions) != len(vars):
+        raise ValueError("Field variable size does not match curve dimension.")
+
+    if curve.parameter in vars:
+        raise ValueError("Curve parameter clashes with field parameters.")
+
+    # Calculate derivatives for line parameter functions
+    # F(r) -> F(r(t)) and finally F(r(t)*r'(t))
+    Ft = F
+    dldt = 0
+    for i, var in enumerate(vars):
+        _f = curve.functions[i]
+        _dn = diff(_f, curve.parameter)
+        # ...arc length
+        dldt = dldt + (_dn * _dn)
+        Ft = Ft.subs(var, _f)
+    Ft = Ft * dldt**(S(1)/2)
+
+    integral = Integral(Ft, curve.limits).doit()
+    return integral
+
