@@ -1,3 +1,4 @@
+import warnings
 from sympy import Basic, Symbol
 from sympy.core import sympify
 
@@ -5,6 +6,8 @@ from sympy.core.basic import S, C
 from sympy.polys import Poly, roots
 from sympy.simplify import simplify
 
+# from sympy.printing import StrPrinter /cyclic/
+
 import random
 
 class NonSquareMatrixException(Exception):
@@ -17,21 +20,45 @@ class ShapeError(ValueError):
 class MatrixError(Exception):
     pass
 
-class _MatrixAsBasic(Basic):
-    """Proxy between Matrix & Basic
+def _dims_to_nm( dims ):
+    """Converts dimensions tuple (or any object with length 1 or 2) or scalar
+    in dims to matrix dimensions n and m."""
 
-    The reason, this class is here, is that Matrix does not inherit from Basic.
-    So for common functionality implemented in Basic, we have to provide this
-    proxy.
+    try:
+        l = len( dims )
+    except TypeError:
+        dims = (dims,)
+        l = 1
 
-    see #420
-    """
-    def __init__(self, m):
-        self.m = m
-    def torepr(self):
-        return self.m.torepr()
-    def tostr(self):
-        return self.m.tostr()
+    # This will work for nd-array too when they are added to sympy.
+    try:
+        for dim in dims:
+            assert (dim > 0) and isinstance( dim, int )
+    except AssertionError:
+        raise ValueError("Matrix dimensions should positive integers!")
+
+    if l == 2:
+        n, m = dims
+    elif l == 1:
+        n, m = dims[0], dims[0]
+    else:
+        raise ValueError("Matrix dimensions should be a two-element tuple of ints or a single int!")
+
+    return n, m
+
+def _iszero(x):
+    return x == 0
+
+class DeferredVector(object):
+    def __init__(self,name):
+        self.name=name
+    def __getitem__(self,i):
+        component_name = '%s[%d]'%(self.name,i)
+        return Symbol(component_name)
+    def __str__(self):
+        return StrPrinter.doprint(self)
+    def __repr__(self):
+        return StrPrinter.doprint(self)
 
 class Matrix(object):
 
@@ -44,14 +71,12 @@ class Matrix(object):
         Matrix can be constructed with values or a rule.
 
         >>> from sympy import *
-        >>> Matrix( (1,2+I), (3,4) ) #doctest:+NORMALIZE_WHITESPACE
+        >>> Matrix( ((1,2+I), (3,4)) ) #doctest:+NORMALIZE_WHITESPACE
         [1, 2 + I]
         [3,     4]
         >>> Matrix(2, 2, lambda i,j: (i+1)*j ) #doctest:+NORMALIZE_WHITESPACE
         [0, 1]
         [0, 2]
-
-        Note: in SymPy we count indices from 0. The rule however counts from 1.
         """
         if len(args) == 3 and callable(args[2]):
             operation = args[2]
@@ -70,11 +95,8 @@ class Matrix(object):
             if len(mat) != self.lines*self.cols:
                 raise MatrixError('List length should be equal to rows*columns')
             self.mat = map(lambda i: sympify(i), mat)
-        else:
-            if len(args) == 1:
-                mat = args[0]
-            else:
-                mat = args
+        elif len(args) == 1:
+            mat = args[0]
             if isinstance(mat, Matrix):
                 self.lines = mat.lines
                 self.cols = mat.cols
@@ -84,17 +106,54 @@ class Matrix(object):
                 # NumPy array or matrix or some other object that implements
                 # __array__. So let's first use this method to get a
                 # numpy.array() and then make a python list out of it.
-                mat = list(mat.__array__())
-            elif not isinstance(mat[0], (list, tuple)):
+                arr = mat.__array__()
+                if len(arr.shape) == 2:
+                    self.lines, self.cols = arr.shape[0], arr.shape[1]
+                    self.mat = map(lambda i: sympify(i), arr.ravel())
+                    return
+                elif len(arr.shape) == 1:
+                    self.lines, self.cols = 1, arr.shape[0]
+                    self.mat = [0]*self.cols
+                    for i in xrange(len(arr)):
+                        self.mat[i] = sympify(arr[i])
+                    return
+                else:
+                    raise NotImplementedError("Sympy supports just 1D and 2D matrices")
+            elif not isinstance(mat, (list, tuple)):
+                raise TypeError("Matrix constructor doesn't accept %s as input" % str(type(mat)))
+            self.lines = len(mat)
+            if len(mat) != 0:
+                if not isinstance(mat[0], (list, tuple)):
+                    self.cols = 1
+                    self.mat = map(lambda i: sympify(i), mat)
+                    return
+                self.cols = len(mat[0])
+            else:
+                self.cols = 0
+            self.mat = []
+            for j in xrange(self.lines):
+                assert len(mat[j])==self.cols
+                for i in xrange(self.cols):
+                    self.mat.append(sympify(mat[j][i]))
+        elif len(args) == 0:
+            # Empty Matrix
+            self.lines = self.cols = 0
+            self.mat = []
+        else:
+            # TODO: on 0.7.0 delete this and uncomment the last line
+            mat = args
+            if not isinstance(mat[0], (list, tuple)):
                 # make each element a singleton
                 mat = [ [element] for element in mat ]
+            warnings.warn("Deprecated constructor, use brackets: Matrix(%s)" % str(mat))
             self.lines=len(mat)
             self.cols=len(mat[0])
             self.mat=[]
-            for j in range(self.lines):
+            for j in xrange(self.lines):
                 assert len(mat[j])==self.cols
-                for i in range(self.cols):
+                for i in xrange(self.cols):
                     self.mat.append(sympify(mat[j][i]))
+            #raise TypeError("Data type not understood")
 
     def key2ij(self,key):
         """Converts key=(4,6) to 4,6 and ensures the key is correct."""
@@ -174,27 +233,48 @@ class Matrix(object):
         2 - I
 
         """
-        if isinstance(key, (list, tuple)):
-            if len(key) == 2:
-                # proper 2-index access
-                if isinstance(key[0], slice) or isinstance(key[1], slice):
-                    return self.submatrix(key)
+        if type(key) is tuple:
+            i, j = key
+            if type(i) is slice or type(j) is slice:
+                return self.submatrix(key)
+
+            else:
+                # a2idx inlined
+                try:
+                    i = i.__int__()
+                except AttributeError:
+                    try:
+                        i = i.__index__()
+                    except AttributeError:
+                        raise IndexError("Invalid index a[%r]" % (key,))
+
+                # a2idx inlined
+                try:
+                    j = j.__int__()
+                except AttributeError:
+                    try:
+                        j = j.__index__()
+                    except AttributeError:
+                        raise IndexError("Invalid index a[%r]" % (key,))
+
+
+                if not (i>=0 and i<self.lines and j>=0 and j < self.cols):
+                    raise IndexError("Index out of range: a[%s]" % (key,))
                 else:
-                    k1, k2 = a2idx(key[0]), a2idx(key[1])
-                    if k1 is not None and k2 is not None:
-                        i,j=self.key2ij([k1, k2])
-                        return self.mat[i*self.cols+j]
+                    return self.mat[i*self.cols + j]
+
+
         else:
             # row-wise decomposition of matrix
-            if isinstance(key, slice):
+            if type(key) is slice:
                 return self.mat[key]
             else:
                 k = a2idx(key)
                 if k is not None:
                     return self.mat[k]
-        raise IndexError("Invalid index: a[%s]"%repr(key))
+        raise IndexError("Invalid index: a[%s]" % repr(key))
 
-    def __setitem__(self,key,value):
+    def __setitem__(self, key, value):
         """
         >>> from sympy import *
         >>> m=Matrix(((1,2+I),(3,4)))
@@ -207,25 +287,44 @@ class Matrix(object):
         [9,     4]
 
         """
-        if isinstance(key, (list, tuple)):
-            if len(key) == 2:
-                # proper 2-index access
-                if isinstance(key[0], slice) or isinstance(key[1], slice):
-                    if isinstance(value, Matrix):
-                        self.copyin_matrix(key, value)
-                        return
-                    if isinstance(value, (list, tuple)):
-                        self.copyin_list(key, value)
-                        return
+        if type(key) is tuple:
+            i, j = key
+            if type(i) is slice or type(j) is slice:
+                if isinstance(value, Matrix):
+                    self.copyin_matrix(key, value)
+                    return
+                if isinstance(value, (list, tuple)):
+                    self.copyin_list(key, value)
+                    return
+            else:
+                # a2idx inlined
+                try:
+                    i = i.__int__()
+                except AttributeError:
+                    try:
+                        i = i.__index__()
+                    except AttributeError:
+                        raise IndexError("Invalid index a[%r]" % (key,))
+
+                # a2idx inlined
+                try:
+                    j = j.__int__()
+                except AttributeError:
+                    try:
+                        j = j.__index__()
+                    except AttributeError:
+                        raise IndexError("Invalid index a[%r]" % (key,))
+
+
+                if not (i>=0 and i<self.lines and j>=0 and j < self.cols):
+                    raise IndexError("Index out of range: a[%s]" % (key,))
                 else:
-                    k1, k2 = a2idx(key[0]), a2idx(key[1])
-                    if k1 is not None and k2 is not None:
-                        i,j=self.key2ij([k1, k2])
-                        self.mat[i*self.cols + j] = sympify(value)
-                        return
+                    self.mat[i*self.cols + j] = sympify(value)
+                    return
+
         else:
             # row-wise decomposition of matrix
-            if isinstance(key, slice):
+            if type(key) is slice:
                 raise IndexError("Vector slices not implemented yet.")
             else:
                 k = a2idx(key)
@@ -359,13 +458,6 @@ class Matrix(object):
         else:
             return True
 
-    # hook Basic.__repr__ & Basic.__str__
-    def __repr__(self):
-        return _MatrixAsBasic(self).__repr__()
-
-    def __str__(self):
-        return _MatrixAsBasic(self).__str__()
-
     def _format_str(self, strfunc, rowsep='\n'):
         # Build table of string representations of the elements
         res = []
@@ -385,15 +477,13 @@ class Matrix(object):
             res[i] = "[" + ", ".join(row) + "]"
         return rowsep.join(res)
 
-    def torepr(self):
-        s = self._format_str(lambda elem: repr(elem), ',\n  ')
+    def __str__(self):
+        return StrPrinter.doprint(self)
 
-        return '%s([\n  %s,\n])' % (self.__class__.__name__, s)
+    def __repr__(self):
+        return StrPrinter.doprint(self)
 
-    def tostr(self):
-        return self._format_str(lambda elem: str(elem))
-
-    def inv(self, method="GE"):
+    def inv(self, method="GE", iszerofunc=_iszero):
         """
         Calculates the matrix inverse.
 
@@ -403,12 +493,18 @@ class Matrix(object):
           LU .... inverse_LU()
           ADJ ... inverse_ADJ()
 
+        Note, the GE and LU methods may require the matrix to be simplified
+        before it is inverted in order to properly detect zeros during
+        pivoting. In difficult cases a custom zero detection function can
+        be provided by setting the iszerosfunc argument to a function that
+        should return True if its argument is zero.
+
         """
         assert self.cols==self.lines
         if method == "GE":
-            return self.inverse_GE()
+            return self.inverse_GE(iszerofunc=iszerofunc)
         elif method == "LU":
-            return self.inverse_LU()
+            return self.inverse_LU(iszerofunc=iszerofunc)
         elif method == "ADJ":
             return self.inverse_ADJ()
         else:
@@ -473,7 +569,7 @@ class Matrix(object):
         [2, 3, 4, 5]
         """
         assert self.lines == rhs.lines
-        newmat = self.zeronm(self.lines, self.cols + rhs.cols)
+        newmat = self.zeros((self.lines, self.cols + rhs.cols))
         newmat[:,:self.cols] = self[:,:]
         newmat[:,self.cols:] = rhs
         return newmat
@@ -492,7 +588,7 @@ class Matrix(object):
         [3, 4, 5]
         """
         assert self.cols == bott.cols
-        newmat = self.zeronm(self.lines+bott.lines, self.cols)
+        newmat = self.zeros((self.lines+bott.lines, self.cols))
         newmat[:self.lines,:] = self[:,:]
         newmat[self.lines:,:] = bott
         return newmat
@@ -505,7 +601,7 @@ class Matrix(object):
         [0, 1, 2]
         [1, 2, 3]
         [2, 3, 4]
-        >>> V = zeronm(1,3)
+        >>> V = zeros((1, 3))
         >>> V
         [0, 0, 0]
         >>> M.row_insert(1,V)
@@ -517,7 +613,7 @@ class Matrix(object):
         if pos is 0:
             return mti.col_join(self)
         assert self.cols == mti.cols
-        newmat = self.zeronm(self.lines + mti.lines, self.cols)
+        newmat = self.zeros((self.lines + mti.lines, self.cols))
         newmat[:pos,:] = self[:pos,:]
         newmat[pos:pos+mti.lines,:] = mti[:,:]
         newmat[pos+mti.lines:,:] = self[pos:,:]
@@ -531,7 +627,7 @@ class Matrix(object):
         [0, 1, 2]
         [1, 2, 3]
         [2, 3, 4]
-        >>> V = zeronm(3,1)
+        >>> V = zeros((3, 1))
         >>> V
         [0]
         [0]
@@ -544,7 +640,7 @@ class Matrix(object):
         if pos is 0:
             return mti.row_join(self)
         assert self.lines == mti.lines
-        newmat = self.zeronm(self.lines, self.cols + mti.cols)
+        newmat = self.zeros((self.lines, self.cols + mti.cols))
         newmat[:,:pos] = self[:,:pos]
         newmat[:,pos:pos+mti.cols] = mti[:,:]
         newmat[:,pos+mti.cols:] = self[:,pos:]
@@ -627,9 +723,11 @@ class Matrix(object):
         out = Matrix(self.lines,self.cols,map(f,self.mat))
         return out
 
-    def evalf(self):
-        out = self.applyfunc(Basic.evalf)
-        return out
+    def evalf(self, prec=None, **options):
+        if prec is None:
+            return self.applyfunc(lambda i: i.evalf(**options))
+        else:
+            return self.applyfunc(lambda i: i.evalf(prec, **options))
 
     def reshape(self, _rows, _cols):
         """
@@ -678,13 +776,13 @@ class Matrix(object):
             s+="]\n"
         print s
 
-    def LUsolve(self, rhs):
+    def LUsolve(self, rhs, iszerofunc=_iszero):
         """
         Solve the linear system Ax = b.
         self is the coefficient matrix A and rhs is the right side b.
         """
         assert rhs.lines == self.lines
-        A, perm = self.LUdecomposition_Simple()
+        A, perm = self.LUdecomposition_Simple(iszerofunc=_iszero)
         n = self.lines
         b = rhs.permuteFwd(perm)
         # forward substitution, all diag entries are scaled to 1
@@ -698,13 +796,13 @@ class Matrix(object):
             b.row(i, lambda x,k: x / A[i,i])
         return b
 
-    def LUdecomposition(self):
+    def LUdecomposition(self, iszerofunc=_iszero):
         """
         Returns the decompositon LU and the row swaps p.
         """
-        combined, p = self.LUdecomposition_Simple()
-        L = self.zero(self.lines)
-        U = self.zero(self.lines)
+        combined, p = self.LUdecomposition_Simple(iszerofunc=_iszero)
+        L = self.zeros(self.lines)
+        U = self.zeros(self.lines)
         for i in range(self.lines):
             for j in range(self.lines):
                 if i > j:
@@ -715,7 +813,7 @@ class Matrix(object):
                     U[i,j] = combined[i,j]
         return L, U, p
 
-    def LUdecomposition_Simple(self):
+    def LUdecomposition_Simple(self, iszerofunc=_iszero):
         """
         Returns A compused of L,U (L's diag entries are 1) and
         p which is the list of the row swaps (in order).
@@ -734,14 +832,14 @@ class Matrix(object):
                 for k in range(j):
                     A[i,j] = A[i,j] - A[i,k]*A[k,j]
                 # find the first non-zero pivot, includes any expression
-                if pivot == -1 and A[i,j] != 0:
+                if pivot == -1 and not iszerofunc(A[i,j]):
                     pivot = i
             if pivot < 0:
                 raise "Error: non-invertible matrix passed to LUdecomposition_Simple()"
             if pivot != j: # row must be swapped
                 A.row_swap(pivot,j)
                 p.append([pivot,j])
-            assert A[j,j] != 0
+            assert not iszerofunc(A[j,j])
             scale = 1 / A[j,j]
             for i in range(j+1,n):
                 A[i,j] = A[i,j] * scale
@@ -756,7 +854,7 @@ class Matrix(object):
         """
         n, m = self.lines, self.cols
         U, L, P = self[:,:], eye(n), eye(n)
-        DD = zero(n) # store it smarter since it's just diagonal
+        DD = zeros(n) # store it smarter since it's just diagonal
         oldpivot = 1
 
         for k in range(n-1):
@@ -811,33 +909,43 @@ class Matrix(object):
         else:
             return -1 * self.minorEntry(i, j, method)
 
-    def jacobian(self, varlist):
+    def jacobian(self, X):
         """
         Calculates the Jacobian matrix (derivative of a vectorial function).
 
-        self is a vector of expression representing functions f_i(x_1, ...,
-        x_n).  varlist is the set of x_i's in order.
+        self ... a vector of expressions representing functions f_i(x_1, ...,
+                        x_n).
+        X ...... is the set of x_i's in order, it can be a list or a Matrix
+
+        Both self and X can be a row or a column matrix in any order
+        (jacobian() should always work).
+
+        Example:
+        >>> from sympy import symbols, sin, cos
+        >>> rho, phi = symbols("rho phi")
+        >>> X = Matrix([rho*cos(phi), rho*sin(phi)])
+        >>> Y = Matrix([rho, phi])
+        >>> X.jacobian(Y)
+        [cos(phi), -rho*sin(phi)]
+        [sin(phi),  rho*cos(phi)]
         """
-        assert self.lines == 1
-        m = self.cols
-        if isinstance(varlist, Matrix):
-            assert varlist.lines == 1
-            n = varlist.cols
-        elif isinstance(varlist, (list, tuple)):
-            n = len(varlist)
-        assert n > 0 # need to diff by something
-        J = self.zeronm(m,n) # maintain subclass type
-        for i in range(m):
-            if isinstance(self[i], (float, int)):
-                continue    # constant function, jacobian row is zero
-            try:
-                tmp = self[i].diff(varlist[0])   # check differentiability
-                J[i,0] = tmp
-            except AttributeError:
-                raise "Function %d is not differentiable" % i
-            for j in range(1,n):
-                J[i,j] = self[i].diff(varlist[j])
-        return J
+        if not isinstance(X, Matrix):
+            X = Matrix(X)
+        # Both X and self can be a row or a column matrix, so we need to make
+        # sure all valid combinations work, but everything else fails:
+        assert len(self.shape) == 2
+        assert len(X.shape) == 2
+        if self.shape[0] == 1:
+            n = self.shape[1]
+        else:
+            n = self.shape[0]
+        if X.shape[0] == 1:
+            assert X.shape[1] == n
+        else:
+            assert X.shape[0] == n
+
+        # n is the dimension of the matrix, computing the Jacobian is now easy:
+        return Matrix(n, n, lambda j, i: self[j].diff(X[i]))
 
     def QRdecomposition(self):
         """
@@ -847,7 +955,7 @@ class Matrix(object):
         """
         assert self.lines == self.cols
         n = self.lines
-        Q, R = self.zero(n), self.zero(n)
+        Q, R = self.zeros(n), self.zeros(n)
         for j in range(n):      # for each column vector
             tmp = self[:,j]     # take original v
             for i in range(j):
@@ -937,15 +1045,22 @@ class Matrix(object):
     def zeronm(self, n, m):
         # used so that certain functions above can use this
         # then only this func need be overloaded in subclasses
+        warnings.warn( 'Deprecated: use zeros() instead.' )
         return Matrix(n,m,[S.Zero]*n*m)
 
     def zero(self, n):
         """Returns a n x n matrix of zeros."""
+        warnings.warn( 'Deprecated: use zeros() instead.' )
         return Matrix(n,n,[S.Zero]*n*n)
 
+    def zeros(self, dims):
+        """Returns a dims = (d1,d2) matrix of zeros."""
+        n, m = _dims_to_nm( dims )
+        return Matrix(n,m,[S.Zero]*n*m)
+
     def eye(self, n):
         """Returns the identity matrix of size n."""
-        tmp = self.zero(n)
+        tmp = self.zeros(n)
         for i in range(tmp.lines):
             tmp[i,i] = S.One
         return tmp
@@ -1060,20 +1175,20 @@ class Matrix(object):
         return self.cofactorMatrix(method).T
 
 
-    def inverse_LU(self):
+    def inverse_LU(self, iszerofunc=_iszero):
         """
         Calculates the inverse using LU decomposition.
         """
-        return self.LUsolve(self.eye(self.lines))
+        return self.LUsolve(self.eye(self.lines), iszerofunc=_iszero)
 
-    def inverse_GE(self):
+    def inverse_GE(self, iszerofunc=_iszero):
         """
         Calculates the inverse using Gaussian elimination.
         """
         assert self.lines == self.cols
         assert self.det() != 0
         big = self.row_join(self.eye(self.lines))
-        red = big.rref()
+        red = big.rref(iszerofunc=iszerofunc)
         return red[0][:,big.lines:]
 
     def inverse_ADJ(self):
@@ -1085,7 +1200,7 @@ class Matrix(object):
         assert d != 0
         return self.adjugate()/d
 
-    def rref(self,simplified=False):
+    def rref(self,simplified=False, iszerofunc=_iszero):
         """
         Take any matrix and return reduced row-echelon form and indices of pivot vars
 
@@ -1099,13 +1214,13 @@ class Matrix(object):
                 break
             if simplified:
                 r[pivots,i] = simplify(r[pivots,i])
-            if r[pivots,i] == 0:
+            if iszerofunc(r[pivots,i]):
                 for k in range(pivots, r.lines):
                     if simplified and k>pivots:
                         r[k,i] = simplify(r[k,i])
-                    if r[k,i] != 0:
+                    if not iszerofunc(r[k,i]):
                         break
-                if k == r.lines - 1 and r[k,i] == 0:
+                if k == r.lines - 1 and iszerofunc(r[k,i]):
                     continue
                 r.row_swap(pivots,k)
             scale = r[pivots,i]
@@ -1128,7 +1243,7 @@ class Matrix(object):
         basis = []
         # create a set of vectors for the basis
         for i in range(self.cols - len(pivots)):
-            basis.append(zeronm(self.cols,1))
+            basis.append(zeros((self.cols, 1)))
         # contains the variable index to which the vector corresponds
         basiskey, cur = [-1]*len(basis), 0
         for i in range(self.cols):
@@ -1203,7 +1318,7 @@ class Matrix(object):
         transforms = [0] * (N-1)
 
         for n in xrange(N, 1, -1):
-            T, k = zeronm(n+1,n), n - 1
+            T, k = zeros((n+1,n)), n - 1
 
             R, C = -A[k,:k], A[:k,k]
             A, a = A[:k,:k], -A[k,k]
@@ -1223,7 +1338,7 @@ class Matrix(object):
 
             transforms[k-1] = T
 
-        polys = [ Matrix(S.One, -A[0,0]) ]
+        polys = [ Matrix([S.One, -A[0,0]]) ]
 
         for i, T in enumerate(transforms):
             polys.append(T * polys[i])
@@ -1279,6 +1394,19 @@ class Matrix(object):
             out.append((r, k, basis))
         return out
 
+    def fill(self, value):
+        """Fill the matrix with the scalar value."""
+        self.mat = [value] * self.lines * self.cols
+
+    def __getattr__(self, attr):
+        if attr in ('diff','integrate','limit'):
+            def doit(*args):
+                item_doit = lambda item: getattr(item, attr)(*args)
+                return self.applyfunc( item_doit )
+            return doit
+        else:
+            raise AttributeError()
+
 def matrix_multiply(A,B):
     """
     Return  A*B.
@@ -1307,25 +1435,35 @@ def matrix_add(A,B):
 
 def zero(n):
     """Create square zero matrix n x n"""
+    warnings.warn( 'Deprecated: use zeros() instead.' )
     return zeronm(n,n)
 
 def zeronm(n,m):
     """Create zero matrix n x m"""
+    warnings.warn( 'Deprecated: use zeros() instead.' )
     assert n>0
     assert m>0
     return Matrix(n,m,[S.Zero]*m*n)
 
+def zeros(dims):
+    """Create zero matrix of dimensions dims = (d1,d2)"""
+    n, m = _dims_to_nm( dims )
+    return Matrix(n,m,[S.Zero]*m*n)
+
 def one(n):
     """Create square all-one matrix n x n"""
-    m = zero(n)
-    for i in range(n):
-        m[i,i]=1
-    return m
+    warnings.warn( 'Deprecated: use ones() instead.' )
+    return Matrix(n,n,[S.One]*n*n)
+
+def ones(dims):
+    """Create all-one matrix of dimensions dims = (d1,d2)"""
+    n, m = _dims_to_nm( dims )
+    return Matrix(n,m,[S.One]*m*n)
 
 def eye(n):
     """Create square identity matrix n x n"""
     assert n>0
-    out = zeronm(n,n)
+    out = zeros(n)
     for i in range(n):
         out[i,i]=S.One
     return out
@@ -1356,7 +1494,7 @@ def hessian(f, varlist):
         f.diff(varlist[0])   # check differentiability
     except AttributeError:
         raise "Function %d is not differentiable" % i
-    out = zero(m)
+    out = zeros(m)
     for i in range(m):
         for j in range(i,m):
             out[i,j] = f.diff(varlist[i]).diff(varlist[j])
@@ -1543,17 +1681,6 @@ class SMatrix(Matrix):
             elif self.mat.has_key((i,j)):
                 del self.mat[(i,j)]
 
-    def __str__(self):
-        s = ""
-        for i in range(self.lines):
-            for j in range(self.cols):
-                if self.mat.has_key((i,j)):
-                    s += "%s " % repr(self[i,j])
-                else:
-                    s += "0 "
-            s += "\n"
-        return s
-
     def row_del(self, k):
         newD = {}
         for (i,j) in self.mat.keys():
@@ -1578,6 +1705,18 @@ class SMatrix(Matrix):
         self.mat = newD
         self.cols -= 1
 
+    def toMatrix(self):
+        l = []
+        for i in range(self.lines):
+            c = []
+            l.append(c)
+            for j in range(self.cols):
+                if self.mat.has_key((i, j)):
+                    c.append(self[i, j])
+                else:
+                    c.append(0)
+        return Matrix(l)
+
     # from here to end all functions are same as in matrices.py
     # with Matrix replaced with SMatrix
     def copyin_list(self, key, value):
@@ -1631,11 +1770,18 @@ class SMatrix(Matrix):
                                (self[0]*b[1] - self[1]*b[0])))
 
     def zeronm(self,n,m):
+        warnings.warn( 'Deprecated: use zeros() instead.' )
         return SMatrix(n,m,{})
 
     def zero(self, n):
+        warnings.warn( 'Deprecated: use zeros() instead.' )
         return SMatrix(n,n,{})
 
+    def zeros(self, dims):
+        """Returns a dims = (d1,d2) matrix of zeros."""
+        n, m = _dims_to_nm( dims )
+        return SMatrix(n,m,{})
+
     def eye(self, n):
         tmp = SMatrix(n,n,lambda i,j:0)
         for i in range(tmp.lines):
