@@ -1,8 +1,9 @@
 from sympy import symbols, Matrix, eye, I, Symbol, Rational, wronskian, cos, \
-        sin, exp, hessian, sqrt, zero, randMatrix, Poly, S, pi
+        sin, exp, hessian, sqrt, zeros, ones, randMatrix, Poly, S, pi, \
+        integrate, oo, raises, trigsimp
 from sympy.matrices.matrices import ShapeError, MatrixError
-from sympy.utilities.test import REPR0
-import py
+from sympy.printing import srepr
+from sympy.utilities.pytest import XFAIL
 
 def test_division():
     x, y, z = symbols('x','y','z')
@@ -66,10 +67,10 @@ def test_power():
     assert eye(2)**10000000 == eye(2)
 
 def test_creation():
-    py.test.raises(MatrixError, 'Matrix(5,5,range(20))')
+    raises(MatrixError, 'Matrix(5,5,range(20))')
 
     x = Symbol("x")
-    a = Matrix([x, 0], [0, 0])
+    a = Matrix([[x, 0], [0, 0]])
     m = a
     assert m.cols == m.lines
     assert m.cols == 2
@@ -179,32 +180,32 @@ def test_determinant():
 def test_submatrix():
     m0 = eye(4)
     assert m0[0:3, 0:3] == eye(3)
-    assert m0[2:4, 0:2] == zero(2)
+    assert m0[2:4, 0:2] == zeros(2)
 
     m1 = Matrix(3,3, lambda i,j: i+j)
     assert m1[0,:] == Matrix(1,3,(0,1,2))
     assert m1[1:3, 1] == Matrix(2,1,(2,3))
 
-    m2 = Matrix([0,1,2,3],[4,5,6,7],[8,9,10,11],[12,13,14,15])
+    m2 = Matrix([[0,1,2,3],[4,5,6,7],[8,9,10,11],[12,13,14,15]])
     assert m2[:,-1] == Matrix(4,1,[3,7,11,15])
     assert m2[-2:,:] == Matrix([[8,9,10,11],[12,13,14,15]])
 
 def test_submatrix_assignment():
-    m = zero(4)
+    m = zeros(4)
     m[2:4, 2:4] = eye(2)
-    assert m == Matrix((0,0,0,0),
+    assert m == Matrix(((0,0,0,0),
                         (0,0,0,0),
                         (0,0,1,0),
-                        (0,0,0,1))
+                        (0,0,0,1)))
     m[0:2, 0:2] = eye(2)
     assert m == eye(4)
     m[:,0] = Matrix(4,1,(1,2,3,4))
-    assert m == Matrix((1,0,0,0),
+    assert m == Matrix(((1,0,0,0),
                         (2,1,0,0),
                         (3,0,1,0),
-                        (4,0,0,1))
-    m[:,:] = zero(4)
-    assert m == zero(4)
+                        (4,0,0,1)))
+    m[:,:] = zeros(4)
+    assert m == zeros(4)
     m[:,:] = ((1,2,3,4),(5,6,7,8),(9, 10, 11, 12),(13,14,15,16))
     assert m == Matrix(((1,2,3,4),
                         (5,6,7,8),
@@ -220,13 +221,13 @@ def test_reshape():
     m0 = eye(3)
     assert m0.reshape(1,9) == Matrix(1,9,(1,0,0,0,1,0,0,0,1))
     m1 = Matrix(3,4, lambda i,j: i+j)
-    assert m1.reshape(4,3) == Matrix((0,1,2), (3,1,2), (3,4,2), (3,4,5))
-    assert m1.reshape(2,6) == Matrix((0,1,2,3,1,2), (3,4,2,3,4,5))
+    assert m1.reshape(4,3) == Matrix(((0,1,2), (3,1,2), (3,4,2), (3,4,5)))
+    assert m1.reshape(2,6) == Matrix(((0,1,2,3,1,2), (3,4,2,3,4,5)))
 
 def test_applyfunc():
     m0 = eye(3)
     assert m0.applyfunc(lambda x:2*x) == eye(3)*2
-    assert m0.applyfunc(lambda x: 0 ) == zero(3)
+    assert m0.applyfunc(lambda x: 0 ) == zeros(3)
 
 def test_expand():
     x,y = symbols('xy')
@@ -248,7 +249,7 @@ def test_LUdecomp():
     L,U,p = testmat.LUdecomposition()
     assert L.is_lower()
     assert U.is_upper()
-    assert (L*U).permuteBkwd(p)-testmat == zero(4)
+    assert (L*U).permuteBkwd(p)-testmat == zeros(4)
 
     testmat = Matrix([[6,-2,7,4],
                       [0,3,6,7],
@@ -257,14 +258,14 @@ def test_LUdecomp():
     L,U,p = testmat.LUdecomposition()
     assert L.is_lower()
     assert U.is_upper()
-    assert (L*U).permuteBkwd(p)-testmat == zero(4)
+    assert (L*U).permuteBkwd(p)-testmat == zeros(4)
 
     x, y, z = Symbol('x'), Symbol('y'), Symbol('z')
-    M = Matrix((1, x, 1), (2, y, 0), (y, 0, z))
+    M = Matrix(((1, x, 1), (2, y, 0), (y, 0, z)))
     L, U, p = M.LUdecomposition()
     assert L.is_lower()
     assert U.is_upper()
-    assert (L*U).permuteBkwd(p)-M == zero(3)
+    assert (L*U).permuteBkwd(p)-M == zeros(3)
 
     # test FF LUdecomp
     M = Matrix([[1, 3, 3],
@@ -400,6 +401,19 @@ def test_wronskian():
         -6*exp(x)*sin(x)*x + 6*cos(x)*exp(x)*x**2 - 6*exp(x)*cos(x)*x - \
         exp(x)*cos(x)*x**3 + exp(x)*sin(x)*x**3
 
+def canonicalize(v):
+    """
+    Takes the output of eigenvects() and makes it canonical, so that we can
+    compare it across platforms.
+
+    It converts Matrices to lists, and uses set() to list the outer list in a
+    platform independent way.
+    """
+    def c(x):
+        a, b, c = x
+        return (S(a), S(b), tuple(c[0]))
+    return tuple(set([c(x) for x in v]))
+
 def test_eigen():
     x,y = symbols('xy')
 
@@ -414,10 +428,10 @@ def test_eigen():
 
     assert M.eigenvals() == {S.One: 3}
 
-    assert M.eigenvects() == \
+    assert canonicalize(M.eigenvects()) == canonicalize(
         [(1, 3, [Matrix([1,0,0]),
                  Matrix([0,1,0]),
-                 Matrix([0,0,1])])]
+                 Matrix([0,0,1])])])
 
     M = Matrix([[0,1,1],
                 [1,0,0],
@@ -425,19 +439,20 @@ def test_eigen():
 
     assert M.eigenvals() == {2*S.One: 1, -S.One: 1, S.Zero: 1}
 
-    assert M.eigenvects() == \
+    assert canonicalize(M.eigenvects()) == canonicalize(
         [( 2, 1, [Matrix([R(2,3), R(1,3), 1])]),
          (-1, 1, [Matrix([-1, 1, 0])]),
-         ( 0, 1, [Matrix([ 0,-1, 1])])]
+         ( 0, 1, [Matrix([ 0,-1, 1])])])
 
     eps = Symbol('eps',real=True)
 
-    M = Matrix([abs(eps), I*eps    ],
-               [-I*eps,   abs(eps) ])
+    M = Matrix([[abs(eps), I*eps    ],
+               [-I*eps,   abs(eps) ]])
+
+    assert canonicalize(M.eigenvects()) == canonicalize(
+        [( 2*abs(eps), 1, [ Matrix([[I*eps/abs(eps)],[1]]) ] ),
+         ( 0, 1, [Matrix([[-I*eps/abs(eps)],[1]])]) ])
 
-    assert M.eigenvects() ==  \
-        [( 2*abs(eps), 1, [ Matrix([I*eps/abs(eps)],[1]) ] ),
-         ( 0, 1, [Matrix([-I*eps/abs(eps)],[1])]) ]
 def test_sparse_matrix():
     return
     def eye(n):
@@ -445,7 +460,7 @@ def test_sparse_matrix():
         for i in range(tmp.lines):
             tmp[i,i] = 1
         return tmp
-    def zero(n):
+    def zeros(n):
         return SMatrix(n,n,lambda i,j:0)
 
     # test_multiplication
@@ -558,7 +573,7 @@ def test_sparse_matrix():
     # test_submatrix
     m0 = eye(4)
     assert m0[0:3, 0:3] == eye(3)
-    assert m0[2:4, 0:2] == zero(2)
+    assert m0[2:4, 0:2] == zeros(2)
 
     m1 = SMatrix(3,3, lambda i,j: i+j)
     assert m1[0,:] == SMatrix(1,3,(0,1,2))
@@ -569,7 +584,7 @@ def test_sparse_matrix():
     assert m2[-2:,:] == SMatrix([[8,9,10,11],[12,13,14,15]])
 
     # test_submatrix_assignment
-    m = zero(4)
+    m = zeros(4)
     m[2:4, 2:4] = eye(2)
     assert m == SMatrix((0,0,0,0),
                         (0,0,0,0),
@@ -582,8 +597,8 @@ def test_sparse_matrix():
                         (2,1,0,0),
                         (3,0,1,0),
                         (4,0,0,1))
-    m[:,:] = zero(4)
-    assert m == zero(4)
+    m[:,:] = zeros(4)
+    assert m == zeros(4)
     m[:,:] = ((1,2,3,4),(5,6,7,8),(9, 10, 11, 12),(13,14,15,16))
     assert m == SMatrix(((1,2,3,4),
                         (5,6,7,8),
@@ -605,7 +620,7 @@ def test_sparse_matrix():
     # test_applyfunc
     m0 = eye(3)
     assert m0.applyfunc(lambda x:2*x) == eye(3)*2
-    assert m0.applyfunc(lambda x: 0 ) == zero(3)
+    assert m0.applyfunc(lambda x: 0 ) == zeros(3)
 
     # test_LUdecomp
     testmat = SMatrix([[0,2,5,3],
@@ -615,7 +630,7 @@ def test_sparse_matrix():
     L,U,p = testmat.LUdecomposition()
     assert L.is_lower()
     assert U.is_upper()
-    assert (L*U).permuteBkwd(p)-testmat == zero(4)
+    assert (L*U).permuteBkwd(p)-testmat == zeros(4)
 
     testmat = SMatrix([[6,-2,7,4],
                       [0,3,6,7],
@@ -624,14 +639,14 @@ def test_sparse_matrix():
     L,U,p = testmat.LUdecomposition()
     assert L.is_lower()
     assert U.is_upper()
-    assert (L*U).permuteBkwd(p)-testmat == zero(4)
+    assert (L*U).permuteBkwd(p)-testmat == zeros(4)
 
     x, y, z = Symbol('x'), Symbol('y'), Symbol('z')
-    M = Matrix((1, x, 1), (2, y, 0), (y, 0, z))
+    M = Matrix(((1, x, 1), (2, y, 0), (y, 0, z)))
     L, U, p = M.LUdecomposition()
     assert L.is_lower()
     assert U.is_upper()
-    assert (L*U).permuteBkwd(p)-M == zero(3)
+    assert (L*U).permuteBkwd(p)-M == zeros(3)
 
     # test_LUsolve
     A = SMatrix([[2,3,5],
@@ -747,6 +762,8 @@ def test_sparse_matrix():
                               [2, 1, [Matrix(1,3,[0,1,0])]],
                               [5, 1, [Matrix(1,3,[1,1,0])]]]
 
+    assert M.zeros((3, 5)) == SMatrix(3, 5, {})
+
 def test_subs():
     x = Symbol('x')
     assert Matrix([[1,x],[x,4]]).subs(x, 5) == Matrix([[1,5],[5,4]])
@@ -798,7 +815,7 @@ def test_conjugate():
                            [5,0]])
 
 def test_conj_dirac():
-    py.test.raises(ShapeError, "eye(3).D")
+    raises(ShapeError, "eye(3).D")
 
     M = Matrix([ [1,I,I,I],
                  [0,1,I,I],
@@ -835,36 +852,6 @@ def test_col_row():
     assert M == Matrix([[x+1,0,0],
                         [1+y,y+2,3]])
 
-
-@REPR0
-def test_str():
-    x, y = symbols("xy")
-    M = Matrix([[x**+1, 1], [y, x+y]])
-
-    assert repr(M) in [
-"""\
-Matrix([
-  [Symbol('x'),                        One(1)],
-  [Symbol('y'), Add(Symbol('x'), Symbol('y'))],
-])\
-""",
-"""\
-Matrix([
-  [Symbol('x'),                        One(1)],
-  [Symbol('y'), Add(Symbol('y'), Symbol('x'))],
-])\
-""" ]
-
-    assert str(M) in [
-"""\
-[x,     1]
-[y, x + y]\
-""",
-"""\
-[x,     1]
-[y, y + x]\
-""" ]
-
 def test_issue851():
     m = Matrix([1, 2, 3])
     a = Matrix([1, 2, 3])
@@ -902,7 +889,7 @@ def test_issue882():
     m[index2] = 5
     assert m[2] == 5
 
-    m = Matrix([1, 2, 3], [4, 5, 6])
+    m = Matrix([[1, 2, 3], [4, 5, 6]])
     assert m[int1,int2] == 6
     assert m[index1,int2] == 6
     assert m[int1,index2] == 6
@@ -936,9 +923,49 @@ def test_evalf():
 
 def test_is_symbolic():
     x = Symbol('x')
-    a = Matrix([x,x],[x,x])
+    a = Matrix([[x,x],[x,x]])
     assert a.is_symbolic() == True
-    a = Matrix([1,2],[3,4])
+    a = Matrix([[1,2],[3,4]])
     assert a.is_symbolic() == False
-    a = Matrix([1,x],[3,4])
+    a = Matrix([[1,x],[3,4]])
     assert a.is_symbolic() == True
+
+def test_zeros_ones_fill():
+    n, m = 3, 5
+
+    a = zeros( (n, m) )
+    a.fill( 5 )
+
+    b = 5 * ones( (n, m) )
+
+    assert a == b
+    assert a.lines == b.lines == 3
+    assert a.cols == b.cols == 5
+    assert a.shape == b.shape == (3, 5)
+
+def test_issue650():
+    x, y = symbols('x','y')
+    a = Matrix([[x**2, x*y],[x*sin(y), x*cos(y)]])
+    assert a.diff(x) == Matrix([[2*x, y],[sin(y), cos(y)]])
+    assert Matrix([[x, -x, x**2],[exp(x),1/x-exp(-x), x+1/x]]).limit(x, oo) == Matrix([[oo, -oo, oo],[oo, 0, oo]])
+    assert Matrix([[(exp(x)-1)/x, 2*x + y*x, x**x ],
+                    [1/x, abs(x) , abs(sin(x+1))]]).limit(x, 0) == Matrix([[1, 0, 1],[oo, 0, sin(1)]])
+    assert a.integrate(x) == Matrix([[Rational(1,3)*x**3, y*x**2/2],[x**2*sin(y)/2, x**2*cos(y)/2]])
+
+def test_inv_iszerofunc():
+    A = eye(4)
+    A.col_swap(0,1)
+    for method in "GE", "LU":
+        assert A.inv(method, iszerofunc=lambda x: x==0) == A.inv("ADJ")
+
+def test_jacobian_metrics():
+    rho, phi = symbols("rho phi")
+    X = Matrix([rho*cos(phi), rho*sin(phi)])
+    Y = Matrix([rho, phi])
+    J = X.jacobian(Y)
+    assert J == X.jacobian(Y.T)
+    assert J == (X.T).jacobian(Y)
+    assert J == (X.T).jacobian(Y.T)
+    g = J.T*eye(J.shape[0])*J
+    g = g.applyfunc(trigsimp)
+    assert g == Matrix([[1, 0], [0, rho**2]])
