@@ -37,9 +37,10 @@ class Pauli(Symbol):
     1
     >>> Pauli(1)*Pauli(2)*Pauli(3)
     I
-
     """
 
+    __slots__ = ["i"]
+
     def __new__(cls, i):
         if not i in [1,2,3]:
             raise "Invalid Pauli index"
@@ -47,6 +48,9 @@ class Pauli(Symbol):
         obj.i=i
         return obj
 
+    def __getnewargs__(self):
+        return (self.i,)
+
     # FIXME don't work for -I*Pauli(2)*Pauli(3)
     def __mul__(self, other):
         if isinstance(other, Pauli):
@@ -59,7 +63,5 @@ class Pauli(Symbol):
         return super(Pauli, self).__mul__(other)
 
     def _eval_power(b, e):
-        if isinstance(e, Integer) and e.is_positive:
+        if e.is_Integer and e.is_positive:
             return super(Pauli, b).__pow__(int(e) % 2)
-
-
