@@ -41,6 +41,7 @@ from random import uniform
 from math import ceil, sqrt, log
 
 def gf_degree(f):
+    """Returns leading degree of f. """
     return len(f)-1
 
 def gf_LC(f):
@@ -99,6 +100,14 @@ def gf_to_dict(f, p):
 
     return result
 
+def gf_from_int_poly(f, p):
+    """Create GF(p)[x] polynomial from Z[x]. """
+    return gf_normal(f, p)
+
+def gf_to_int_poly(f, p):
+    """Convert GF(p)[x] polynomial to Z[x]. """
+    return [ zp_int(c, p) for c in f ]
+
 def gf_neg(f, p):
     """Negate a polynomial over GF(p)[x]. """
     return [ -coeff % p for coeff in f ]
@@ -276,13 +285,6 @@ def gf_div(f, g, p):
        >>> gf_add_mul([1], [1, 1], [1, 1, 0], 2)
        [1, 0, 1, 1]
 
-       Two division methods are implemented:
-
-        1. Classical O(n**2) in-place method due to Monagan (see [1]).
-
-        2. Newton iteration for polynomials of large degree. Complexity
-           of this method is O(M(n)) where M(n) is multiplication time.
-
        For more details on the implemented algorithms refer to:
 
        [1] Michael Monagan, In-place Arithmetic for Polynomials over
@@ -293,10 +295,6 @@ def gf_div(f, g, p):
            First Edition, Cambridge University Press, 1999, pp. 247
 
     """
-    return gf_div_classic(f, g, p)
-
-def gf_div_classic(f, g, p):
-    """Division with remainder over GF(p)[x] using classical method. """
     df = gf_degree(f)
     dg = gf_degree(g)
 
@@ -322,8 +320,8 @@ def gf_div_classic(f, g, p):
 
     return h[:dq+1], gf_strip(h[dq+1:])
 
-def gf_quo_classic(f, g, p):
-    """Computes polynomial quotient over GF(p)[x] using classical method. """
+def gf_quo(f, g, p):
+    """Computes polynomial quotient over GF(p)[x]. """
     df = gf_degree(f)
     dg = gf_degree(g)
 
@@ -346,74 +344,6 @@ def gf_quo_classic(f, g, p):
 
     return h[:dq+1]
 
-def gf_inv_newton(f, n, p):
-    """Compute f**(-1) mod x**n using Newton iteration. """
-    g, h = [zp_inv(gf_TC(f), p)], [1,0,0]
-
-    N = int(ceil(log(n, 2)))
-
-    for i in xrange(1, N+1):
-        a = gf_mul_const(g, 2, p)
-        b = gf_mul(f, gf_sqr(g, p), p)
-        g = gf_rem(gf_sub(a, b, p), h, p)
-
-        h = gf_lshift(h, gf_degree(h))
-
-    return g
-
-def gf_div_newton(f, g, p):
-    """Division with remainder over GF(p)[x] using Newton iteration. """
-    df = gf_degree(f)
-    dg = gf_degree(g)
-
-    if not g:
-        raise ZeroDivisionError("polynomial division")
-    elif df < dg:
-        return [], f
-
-    dq = df - dg
-
-    F = gf_reverse(f)
-    G = gf_reverse(g)
-
-    n = dq + 1
-
-    H = gf_inv_newton(G, n, p)
-    Q = gf_rshift(gf_mul(F, H, p), n)[1]
-
-    q = gf_reverse(Q)
-    r = gf_sub_mul(f, g, q, p)
-
-    return q, r
-
-def gf_quo_newton(f, g, p):
-    """Computes polynomial quotient over GF(p)[x] using Newton iteration. """
-    df = gf_degree(f)
-    dg = gf_degree(g)
-
-    if not g:
-        raise ZeroDivisionError("polynomial division")
-    elif df < dg:
-        return [], f
-
-    dq = df - dg
-
-    F = gf_reverse(f)
-    G = gf_reverse(g)
-
-    n = dq + 1
-
-    H = gf_inv_newton(G, n, p)
-    Q = gf_rshift(gf_mul(F, H, p), n)[1]
-
-    q = gf_reverse(Q)
-
-    return q
-
-def gf_quo(f, g, p):
-    """Returns polynomial quotient over GF(p)[x]. """
-    return gf_div(f, g, p)[0]
-
 def gf_rem(f, g, p):
     """Returns polynomial remainder over GF(p)[x]. """
     return gf_div(f, g, p)[1]
@@ -919,6 +849,9 @@ def gf_ddf_shoup(f, p):
        list of pairs (f_i, e_i) where deg(f_i) > 0 and e_i > 0 is
        an argument to the equal degree factorization routine.
 
+       This algorithm is an improved version of Zassenhaus algorithm
+       for large deg(f) and modulus p (especially for deg(f) ~ lg(p)).
+
        For more details on the implemented algorithm refer to:
 
        [1] E. Kaltofen, V. Shoup, Subquadratic-time Factoring of
@@ -980,8 +913,13 @@ def gf_ddf_shoup(f, p):
 def gf_edf_shoup(f, n, p):
     """Gathen-Shoup: Probabilistic Equal Degree Factorization
 
+       Given a monic square-free polynomial f in GF(p)[x] and integer n
+       such that n divides deg(f),  returns all irreducible factors f_1
+       ... f_d of f, each of degree n. This is a complete factorization
+       over Galois fields.
 
-       Given a monic square-free polynomial f in GF(p)[x] ...
+       This algorithm is an improved version of Zassenhaus algorithm
+       for large deg(f) and modulus p (especially for deg(f) ~ lg(p)).
 
        For more details on the implemented algorithm refer to:
 
@@ -1097,12 +1035,8 @@ def gf_factor(f, p, **flags):
        for i != j. The result is given as a tuple consisting of the leading
        coefficient of f and a list of factors with their multiplicities.
 
-       The algorithm proceeds by iteratively, for i = 1, 2, ...,  computing
-       one step of distinct degree factorization obtaining product g = g_1
-       g_2 ... g_s of all irreducible factors of f, each of degree i, which
-       is then factored into irreducibles using equal degree factorization
-       algorithm. For each factor g_k, its multiplicity e_k is obtained by
-       trial division and then g_k**e_k is removed from f.
+       The algorithm proceeds by first computing square-free decomposition
+       of f and then iteratively factoring each of the square-free factors.
 
        Consider a non square-free polynomial f = (7*x + 1) (x + 2)**2 over
        GF(11)[x]. We obtain its factorization into irreducibles as follows:
@@ -1120,32 +1054,21 @@ def gf_factor(f, p, **flags):
            First Edition, Cambridge University Press, 1999, pp. 365
 
     """
-    i, factors, h = 1, [], [1, 0]
-
     LC, f = gf_monic(f, p)
 
     if gf_degree(f) < 1:
         return LC, []
 
-    while f != [1]:
-        if gf_degree(f) < 2*i:
-            factors.append((f, 1))
-            break
+    factors = []
 
-        h = gf_pow_mod(h, p, f, p)
-        g = gf_gcd(f, gf_sub(h, [1, 0], p), p)
+    for g, k in gf_sqf(f, p)[1]:
+        n = gf_degree(g)
 
-        if g != [1]:
-            for F in gf_edf(g, i, p, **flags):
-                (q, r), e = gf_div(f, F, p), 0
+        if n == 0:
+            continue
 
-                while not r:
-                    f, e = q, e + 1
-                    q, r = gf_div(f, F, p)
-
-                factors.append((F, e))
-
-        i += 1
+        for h in gf_factor_sqf(g, p)[1]:
+            factors.append((h, k))
 
     def compare((f_a, e_a), (f_b, e_b)):
         i = len(f_a) - len(f_b)
@@ -1175,3 +1098,4 @@ def gf_factor_sqf(f, p, **flags):
         factors += gf_edf(factor, n, p, **flags)
 
     return LC, factors
+
