@@ -6,7 +6,6 @@ from sympy.core.sympify import sympify
 from sympy.core.basic import Basic, S, C
 from sympy.core.symbol import Symbol, Wild
 from sympy.core.numbers import Integer, igcd, ilcm
-from sympy.core.methods import RelMeths, ArithMeths
 
 from sympy.utilities import all, any
 
@@ -54,7 +53,7 @@ class MultivariatePolyError(PolynomialError):
 ##  [7] Concept of Monomial.
 ##
 
-class Poly(Basic, RelMeths, ArithMeths):
+class Poly(Basic):
     """Represents polynomials with symbolic coefficients.
 
        Polynomials are internally represented as two lists containing
@@ -223,6 +222,8 @@ class Poly(Basic, RelMeths, ArithMeths):
           [9.10] [U-] diff          --> efficient polynomial differentiation
           [9.11] [U-] integrate     --> efficient polynomial integration
 
+          [9.12] [U-] invert        --> computes multiplicative inverse in K[t]
+
        [10] Operations on terms:
 
           [10.1] [U-] add_term --> add a term to a polynomial
@@ -248,8 +249,8 @@ class Poly(Basic, RelMeths, ArithMeths):
 
        [13] String representation functions:
 
-          [13.1] [U-] torepr      --> returns represtation of 'self'
-          [13.3] [U-] tostr       --> returns pretty representation
+          [13.1] [U-] repr      --> returns represantation of 'self'
+          [13.3] [U-] str       --> returns pretty representation
 
        [14] Other (derived from Basic) functionality:
 
@@ -419,6 +420,10 @@ class Poly(Basic, RelMeths, ArithMeths):
 
         return Basic.__new__(cls, *args, **flags)
 
+    def __getnewargs__(self):
+        coeffs, monoms, symbols, order = self.args[:4]
+        return ((coeffs,monoms),) + symbols
+
     @staticmethod
     def _permute(poly, *symbols):
         terms, N = {}, len(symbols)
@@ -492,7 +497,7 @@ class Poly(Basic, RelMeths, ArithMeths):
            >>> x, y = symbols('xy')
 
            >>> Poly._decompose(y*x**2 + 3, x)
-           {(2,): y, (0,): 3}
+           {(0,): 3, (2,): y}
 
            >>> Poly._decompose(y*x**2 + 3, x, y)
            {(0, 0): 3, (2, 1): 1}
@@ -639,6 +644,15 @@ class Poly(Basic, RelMeths, ArithMeths):
         p = sympy.polys.algorithms.poly_div(p, g)[0]
         q = sympy.polys.algorithms.poly_div(q, g)[0]
 
+        pc = int(p.content)
+        qc = int(q.content)
+
+        cont = igcd(pc, qc)
+
+        if cont != 1:
+            p = p.div_term(cont)
+            q = q.div_term(cont)
+
         return p.as_basic() / q.as_basic()
 
     def as_basic(self):
@@ -659,8 +673,8 @@ class Poly(Basic, RelMeths, ArithMeths):
            3 + x**2
 
         """
-        return Add(*[ coeff.as_basic() * monomial_as_basic(monom,
-            *self.symbols) for coeff, monom in self.iter_terms() ])
+        multinomial = dict(zip(self.monoms, self.coeffs))
+        return multinomial_as_basic(multinomial, *self.symbols)
 
     def as_dict(self):
         """Convert list representation to dictionary representation.
@@ -679,7 +693,7 @@ class Poly(Basic, RelMeths, ArithMeths):
 
            >>> p = Poly(x**2+3, x)
            >>> p.as_dict()
-           {(2,): 1, (0,): 3}
+           {(0,): 3, (2,): 1}
 
         """
         return dict(zip(self.monoms, self.coeffs))
@@ -1064,7 +1078,7 @@ class Poly(Basic, RelMeths, ArithMeths):
             N, Q = abs(int(other)), self
 
             while True:
-                N, M = N/2, N
+                N, M = N//2, N
 
                 if M & 1:
                     P *= Q
@@ -1269,24 +1283,26 @@ class Poly(Basic, RelMeths, ArithMeths):
            (2, Poly(6*x**2 + x, x))
 
         """
-        denom = 1
+        denom, coeffs = 1, []
 
         for coeff in self.iter_coeffs():
             if coeff.is_Rational:
+                coeffs.append(coeff)
+
                 if not coeff.is_Integer:
                     denom = ilcm(denom, coeff.q)
+            elif coeff.is_Real and int(coeff) == coeff:
+                coeffs.append(Integer(int(coeff)))
             else:
                 raise CoefficientError("%s is not a rational number" % coeff)
 
         denom = sympify(denom)
 
-        if denom is S.One:
-            return denom, self
-        else:
+        if denom is not S(1):
             coeffs = [ coeff * denom for coeff in self.iter_coeffs() ]
 
-            return denom, self.__class__((coeffs, self.monoms),
-                *self.symbols, **self.flags)
+        return denom, self.__class__((coeffs, self.monoms),
+            *self.symbols, **self.flags)
 
     @property
     def content(self):
@@ -1628,13 +1644,13 @@ class Poly(Basic, RelMeths, ArithMeths):
            Poly(1/3*x**3*y + z**2*x, x, y)
 
            >>> p.integrate(x, 2)
-           Poly(1/12*x**4*y + (1/2)*z**2*x**2, x, y)
+           Poly(1/12*x**4*y + z**2/2*x**2, x, y)
 
            >>> p.integrate(x, 2, y)
-           Poly(1/24*x**4*y**2 + (1/2)*z**2*x**2*y, x, y)
+           Poly(1/24*x**4*y**2 + z**2/2*x**2*y, x, y)
 
            >>> p.integrate(z)
-           Poly(z*x**2*y + (1/3)*z**3, x, y)
+           Poly(z*x**2*y + z**3/3, x, y)
 
         """
         if self.is_zero:
@@ -1693,6 +1709,21 @@ class Poly(Basic, RelMeths, ArithMeths):
 
         return self.__class__(poly, *self.symbols, **self.flags)
 
+    def invert(self, modulus):
+        """Compute multiplicative inverse in K[x]. """
+        self, modulus = self.unify_with(modulus)
+
+        if self.is_multivariate:
+            raise PolynomialError("only univariate polynomials can be inverted")
+
+        s, g = sympy.polys.algorithms.poly_half_gcdex(self, modulus)
+
+        if g.is_one:
+            return sympy.polys.algorithms.poly_div(s, modulus)[1]
+        else:
+            raise ZeroDivisionError("polynomial division")
+
+
     def add_term(self, coeff, monom):
         """Efficiently add a single term to 'self'.
 
@@ -2075,10 +2106,10 @@ class Poly(Basic, RelMeths, ArithMeths):
                 else:
                     return self.evaluate((old, new))
         elif not new.has_any_symbols(*symbols):
-            coeffs = [ sympify(coeff).subs(old, new) for coeff in self.coeffs ]
-            return self.__class__((coeffs, self.monoms), *symbols, **self.flags)
+            coeffs = [ sympify(coeff)._eval_subs(old, new) for coeff in self.coeffs ]
+            return self.__class__(zip(coeffs, self.monoms), *symbols, **self.flags)
 
-        result = self.as_basic().subs(old, new)
+        result = self.as_basic()._eval_subs(old, new)
 
         try:
             return self.__class__(result, *symbols, **self.flags)
@@ -2128,75 +2159,6 @@ class Poly(Basic, RelMeths, ArithMeths):
     def __nonzero__(self):
         return self.coeffs not in ((S.Zero,), (0,))
 
-    def torepr(self):
-        terms = []
-
-        for coeff, monom in self.iter_terms():
-            terms.append("(%s, %s)" % (coeff.torepr(), monom))
-
-        format = self.__class__.__name__ + "([%s], %s, order='%s')"
-
-        symbols = [ s.torepr() for s in self.symbols ]
-
-        return format % (', '.join(terms),
-            ', '.join(symbols), self.order)
-
-    def tostr(self, level=0):
-        terms, symbols = [], [ s.tostr() for s in self.symbols ]
-
-        for coeff, monom in self.iter_terms():
-            s_monom = []
-
-            for i, exp in enumerate(monom):
-                if exp > 0:
-                    if exp == 1:
-                        s_monom.append(symbols[i])
-                    else:
-                        s_monom.append(symbols[i] + "**%d" % exp)
-
-            s_monom = "*".join(s_monom)
-
-            if coeff.is_Add:
-                if s_monom:
-                    s_coeff = "(" + coeff.tostr() + ")"
-                else:
-                    s_coeff = coeff.tostr()
-            else:
-                if s_monom and abs(coeff) is S.One:
-                    if coeff.is_negative:
-                        terms.extend(['-', s_monom])
-                    else:
-                        terms.extend(['+', s_monom])
-
-                    continue
-                else:
-                    s_coeff = coeff.tostr()
-
-            if not s_monom:
-                s_term = s_coeff
-            else:
-                s_term = s_coeff + "*" + s_monom
-
-            if s_term.startswith('-'):
-                terms.extend(['-', s_term[1:]])
-            else:
-                terms.extend(['+', s_term])
-
-        if terms[0] in ['-', '+']:
-            modifier = terms.pop(0)
-
-            if modifier == '-':
-                terms[0] = '-' + terms[0]
-
-        format = self.__class__.__name__ + "(%s, %s"
-
-        if self.is_multivariate and self.order != 'grlex':
-            format += ", order='%s')" % self.order
-        else:
-            format += ")"
-
-        return format % (' '.join(terms), ', '.join(symbols))
-
     def _eval_is_polynomial(self, symbols):
         try:
             self.__class__(self, *symbols, **self.flags)
@@ -2207,3 +2169,20 @@ class Poly(Basic, RelMeths, ArithMeths):
 
 class IntegerPoly(Poly):
     pass
+
+def multinomial_as_basic(multinomial, *symbols):
+    """
+    Converts the multinomial to Add/Mul/Pow instances.
+
+    multinomial is a dict of {powers: coefficient} pairs, powers is a tuple of
+    python integers, coefficient is a python integer.
+    """
+    l = []
+    for powers, k in multinomial.iteritems():
+        term = [k]
+        for i,e in enumerate(powers):
+            term.append(Pow(symbols[i], powers[i]))
+        l.append(Mul(*term))
+    result = Add(*l)
+    return result
+
