@@ -8,6 +8,7 @@ from sympy.core.numbers import Rational
 from sympy.polys.polynomial import Poly, SymbolsError, \
     PolynomialError, CoefficientError, MultivariatePolyError
 from sympy.polys.algorithms import poly_decompose, poly_sqf, poly_div
+from sympy.polys.factortools import poly_factors
 
 from sympy.ntheory import divisors
 from sympy.functions import exp
@@ -30,11 +31,16 @@ def roots_quadratic(f):
         (-b - d) / (2*a),
     ]
 
-    return [ r.expand() for r in roots ]
+    from sympy.simplify import simplify
+
+    return [ simplify(r) for r in roots ]
 
 def roots_cubic(f):
     """Returns a list of  roots of a cubic polynomial."""
-    a, b, c, d = f.iter_all_coeffs()
+    if isinstance(f, (tuple, list)):
+        a, b, c, d = f
+    else:
+        a, b, c, d = f.iter_all_coeffs()
 
     b, c, d = b/a, c/a, d/a
 
@@ -63,14 +69,25 @@ def roots_cubic(f):
     return [ r.expand() for r in roots ]
 
 def roots_quartic(f):
-    """Returns a list of roots of a quartic polynomial."""
+    """Returns a list of roots of a quartic polynomial.
+
+    References:
+      http://planetmath.org/encyclopedia/GaloisTheoreticDerivationOfTheQuarticFormula.html
+    """
     a, b, c, d, e = f.iter_all_coeffs()
 
+    # normalize
+    t = a
+    a = b/t
+    b = c/t
+    c = d/t
+    d = e/t
+
     p = -2*b
     q = b**2 + a*c - 4*d
     r = c**2 + a**2*d - a*b*c
 
-    r = cubic(1, p, q, r)
+    r = roots_cubic((1, p, q, r))
 
     u = r[1] + r[2] - r[0]
     v = (u**2 - 16*d)**S.Half
@@ -147,9 +164,9 @@ def roots(f, *symbols, **flags):
 
        Only roots expressible via radicals will be returned.  To get
        a complete set of roots use RootOf class or numerical methods
-       instead. By default cubic and quartic formulas aren't used in
-       the algorithm because of unreadable output. To enable them
-       set cubics=True or quartics=True respectively.
+       instead. By default cubic and quartic formulas are used in
+       the algorithm. To disable them because of unreadable output
+       set cubics=False or quartics=False respectively.
 
        To get roots from a specific domain set the 'domain' flag with
        one of the following specifiers: Z, Q, R, I, C. By default all
@@ -177,57 +194,67 @@ def roots(f, *symbols, **flags):
     if f.is_multivariate:
         raise MultivariatePolyError(f)
 
-    def roots_trivial(g):
-        if g.length == 1:
-            if g.is_constant:
+    def _update_dict(result, root, k):
+        if result.has_key(root):
+            result[root] += k
+        else:
+            result[root] = k
+
+    def _try_decompose(f):
+        """Find roots using functional decomposition. """
+        factors = poly_decompose(f)
+        result, g = {}, factors[0]
+
+        for i, h in enumerate(poly_sqf(g)):
+            for r in _try_heuristics(h):
+                _update_dict(result, r, i+1)
+
+        for factor in factors[1:]:
+            last, result = result.copy(), {}
+
+            for last_r, i in last.iteritems():
+                g = factor.sub_term(last_r, (0,))
+
+                for j, h in enumerate(poly_sqf(g)):
+                    for r in _try_heuristics(h):
+                        _update_dict(result, r, i*(j+1))
+
+        return result
+
+    def _try_heuristics(f):
+        """Find roots using formulas and some tricks. """
+        if f.length == 1:
+            if f.is_constant:
                 return []
             else:
-                return [S.Zero] * g.degree
+                return [S(0)] * f.degree
 
-        (k,), g = g.as_reduced()
+        if f.length == 2:
+            if f.degree == 1:
+                return roots_linear(f)
+            else:
+                return roots_binomial(f)
 
-        if k == 0:
-            zeros = []
-        else:
-            zeros = [S.Zero] * k
+        x, result = f.symbols[0], []
 
-        if g.length == 2:
-            zeros += roots_binomial(g)
-        else:
-            x = g.symbols[0]
+        for i in [S(-1), S(1)]:
+            if f(i).expand().is_zero:
+                f = poly_div(f, x-i)[0]
+                result.append(i)
+                break
 
-            for i in [S.NegativeOne, S.One]:
-                if g(i).expand() is S.Zero:
-                    g = poly_div(g, x-i)[0]
-                    zeros.append(i)
-                    break
+        n = f.degree
 
-            n = g.degree
+        if n == 1:
+            result += roots_linear(f)
+        elif n == 2:
+            result += roots_quadratic(f)
+        elif n == 3 and flags.get('cubics', True):
+            result += roots_cubic(f)
+        elif n == 4 and flags.get('quartics', False):
+            result += roots_quartic(f)
 
-            if n == 1:
-                zeros += roots_linear(g)
-            elif n == 2:
-                zeros += roots_quadratic(g)
-            elif n == 3:
-                if flags.get('cubics', False):
-                    from sympy.polynomials.factor_ import factor
-                    from sympy.polynomials.base import PolynomialException
-
-                    try:
-                        factors = factor(g.as_basic(), g.symbols)
-
-                        if len(factors) <= 2:
-                            raise PolynomialException
-
-                        for term in factors:
-                            zeros += roots(term.as_poly(x))
-                    except PolynomialException:
-                        zeros += roots_cubic(g)
-            elif n == 4:
-                if flags.get('quartics', False):
-                    zeros += roots_quartic(g)
-
-        return zeros
+        return result
 
     multiple = flags.get('multiple', False)
 
@@ -238,43 +265,40 @@ def roots(f, *symbols, **flags):
             else:
                 return {}
         else:
-            result = { S.Zero : f.degree }
+            result = { S(0) : f.degree }
     else:
         (k,), f = f.as_reduced()
 
         if k == 0:
-            result = {}
+            zeros = {}
         else:
-            result = { S.Zero : k }
+            zeros = { S(0) : k }
 
-        if f.degree == 1:
-            result[roots_linear(f)[0]] = 1
+        result = {}
+
+        if f.length == 2:
+            if f.degree == 1:
+                result[roots_linear(f)[0]] = 1
+            else:
+                for r in roots_binomial(f):
+                    _update_dict(result, r, 1)
+        elif f.degree == 2:
+            for r in roots_quadratic(f):
+                _update_dict(result, r, 1)
         else:
-            factors = poly_decompose(f)
+            try:
+                _, factors = poly_factors(f)
 
-            zeros, g = {}, factors[0]
+                if len(factors) == 1 and factors[0][1] == 1:
+                    raise CoefficientError
 
-            for i, h in enumerate(poly_sqf(g)):
-                for zero in roots_trivial(h):
-                    if zeros.has_key(zero):
-                        zeros[zero] += i+1
-                    else:
-                        zeros[zero] = i+1
+                for factor, k in factors:
+                    for r in _try_heuristics(factor):
+                        _update_dict(result, r, k)
+            except CoefficientError:
+                result = _try_decompose(f)
 
-            for factor in factors[1:]:
-                previous, zeros = zeros.copy(), {}
-
-                for zero, i in previous.iteritems():
-                    g = factor.sub_term(zero, (0,))
-
-                    for j, h in enumerate(poly_sqf(g)):
-                        for zero in roots_trivial(h):
-                            if zeros.has_key(zero):
-                                zeros[zero] += i*(j+1)
-                            else:
-                                zeros[zero] = i*(j+1)
-
-            result.update(zeros)
+        result.update(zeros)
 
     domain = flags.get('domain', None)
 
@@ -312,13 +336,13 @@ def roots(f, *symbols, **flags):
 
         return zeros
 
-def poly_factors(f, *symbols, **flags):
+def poly_root_factors(f, *symbols, **flags):
     """Returns all factors of an univariate polynomial.
 
        >>> from sympy import *
        >>> x,y = symbols('xy')
 
-       >>> factors = poly_factors(x**2-y, x)
+       >>> factors = poly_root_factors(x**2-y, x)
 
        >>> set(f.as_basic() for f in factors)
        set([x + y**(1/2), x - y**(1/2)])
@@ -426,6 +450,17 @@ def number_of_real_roots(f, *symbols, **flags):
 
         return count
 
+    if not isinstance(f, Poly):
+        f = Poly(f, *symbols)
+    elif symbols:
+        raise SymbolsError("Redundant symbols were given")
+
+    if f.is_multivariate:
+        raise MultivariatePolyError(f)
+
+    if f.degree < 1:
+        return 0
+
     inf = flags.get('inf', None)
 
     if inf is not None:
@@ -446,7 +481,7 @@ def number_of_real_roots(f, *symbols, **flags):
         elif abs(sup) is S.Infinity:
             sup = None
 
-    sturm = poly_sturm(f, *symbols)
+    sturm = poly_sturm(f)
 
     if inf is None:
         signs_inf = sign_changes([ s.LC * (-1)**s.LM[0] for s in sturm ])
@@ -614,3 +649,4 @@ class RootSum(Basic):
                 result += self.function(root)
 
             return result
+
