@@ -3,13 +3,19 @@ from sympy import symbols, expand, sin, sqrt, re, im, I, Rational, Lambda, \
 
 from sympy.polys.monomial import monomial_lex_cmp, monomial_grlex_cmp, \
         monomial_grevlex_cmp, monomial_1_el_cmp
-from sympy.polys.algorithms import poly_groebner, poly_subresultants, \
+
+from sympy.polys.algorithms import poly_groebner, poly_subresultants,    \
         poly_resultant, poly_half_gcdex, poly_gcdex, poly_gcd, poly_lcm, \
-        poly_pdiv
-from sympy.polys.rootfinding import poly_factors, roots_linear, \
-        roots_quadratic, roots_cubic, roots_binomial, roots_rational, roots, \
-        number_of_real_roots, poly_sturm, PolynomialError, poly_decompose, \
-        poly_sqf, poly_div, CoefficientError, SymbolsError
+        poly_div, poly_pdiv, poly_decompose, poly_sqf, poly_reduce
+
+from sympy.polys.rootfinding import poly_root_factors, roots_linear,  \
+        roots_quadratic, roots_cubic, roots_quartic, roots_binomial, \
+        roots_rational, roots, number_of_real_roots, poly_sturm
+
+from sympy.polys.factortools import poly_factors, factors, factor
+
+from sympy.polys.polynomial import Poly, PolynomialError, \
+        CoefficientError, SymbolsError
 
 a,b,c,d,x,y,z,u,v,t = symbols('abcdxyzuvt')
 
@@ -179,6 +185,10 @@ def test_poly_cancel():
 
     assert Poly.cancel(f) == -2 + sqrt(2)
 
+    a, b = x, 1/(1/y + 1/(x+y))
+
+    assert Poly.cancel(y/(x+y) * b/(a+b), x, y) == y**2/(x**2 + 3*x*y + y**2 )
+
     raises(SymbolsError, "Poly.cancel((x**2-y**2, x-y))")
 
 def test_poly_characteristics():
@@ -381,7 +391,11 @@ def test_as_integer():
     assert Poly(x**2 + x/2, x).as_integer() == \
         (Integer(2), Poly(2*x**2 + x, x))
 
+    assert Poly(25.0*x**7 + 5.0*x**2, x).as_integer() == \
+        (Integer(1), Poly(25*x**7 + 5*x**2, x))
+
     raises(CoefficientError, "Poly(x**2 + t*x, x).as_integer()")
+    raises(CoefficientError, "Poly(x**2 + 0.1, x).as_integer()")
 
 def test_poly_add():
     f = -Rational(1,6)*x**2-Rational(5,36)+Rational(17,18)
@@ -412,6 +426,14 @@ def test_poly_mul():
 
     assert p.symbols == (x, y, z)
 
+def test_poly_pow():
+    assert Poly(x**3-12*x**2-42, x)**2 == \
+        Poly(x**6 - 24*x**5 + 144*x**4 - 84*x**3 + 1008*x**2 + 1764, x)
+
+    assert Poly(x**3*y-2*x*y**2-3*z+1, x)**2 == \
+        Poly(y**2*x**6 - 4*y**3*x**4 + (2*y - 6*y*z)*x**3 + \
+        4*y**4*x**2 + (-4*y**2 + 12*z*y**2)*x + 1 - 6*z + 9*z**2, x)
+
 def test_poly_div():
     f = Poly(x**3-12*x**2-42, x)
     g = Poly(x**2+x-3, x)
@@ -554,14 +576,14 @@ def test_poly_gcdex():
     g = x**3 + x**2 - 4*x - 4
 
     assert poly_half_gcdex(f, g, x) == \
-        (Poly(-x+3, x), Poly(5*x+5, x))
+        (Poly(-x/5+Rational(3,5), x), Poly(x+1, x))
     assert poly_half_gcdex(Poly(f, x), Poly(g, x)) == \
-        (Poly(-x+3, x), Poly(5*x+5, x))
+        (Poly(-x/5+Rational(3,5), x), Poly(x+1, x))
 
     assert poly_gcdex(f, g, x) == \
-        (Poly(-x+3, x), Poly(x**2-6*x+10, x), Poly(5*x+5, x))
+        (Poly(-x/5+Rational(3,5), x), Poly(x**2/5-6*x/5+2, x), Poly(x+1, x))
     assert poly_gcdex(Poly(f, x), Poly(g, x)) == \
-        (Poly(-x+3, x), Poly(x**2-6*x+10, x), Poly(5*x+5, x))
+        (Poly(-x/5+Rational(3,5), x), Poly(x**2/5-6*x/5+2, x), Poly(x+1, x))
 
     f = x**4 + 4*x**3 - x + 1
     g = x**3 - x + 1
@@ -572,6 +594,9 @@ def test_poly_gcdex():
     assert s*f + t*g == h
     assert S*g + T*f == H
 
+    assert poly_gcdex(2*x, x**2-16, x) == \
+        (Poly(x/32, x), Poly(-Rational(1,16), x), Poly(1, x))
+
 def test_poly_resultant():
     assert poly_resultant(x**2-1, x**3-x**2+2, x) == 0
     assert poly_resultant(3*x**3-x, 5*x**2+1, x) == 64
@@ -743,6 +768,18 @@ def test_squarefree():
     assert poly_sqf(x**8+6*x**6+12*x**4+8*x**2, x) == \
         [Poly(1, x), Poly(x, x), Poly(x**2+2, x)]
 
+    # Bronstein, Symbolic Integration, pp. 52
+
+    A = Poly(x**4 - 3*x**2 + 6, x)
+    D = Poly(x**6 - 5*x**4 + 5*x**2 + 4, x)
+
+    f, g = D, A - D.diff(x)*t
+
+    R = poly_subresultants(f, g)
+    S = poly_sqf(Poly(R[-1], t))
+
+    assert S == [Poly(45796, t), Poly(1, t), Poly(4*t**2 + 1, t)]
+
 def test_decompose():
     assert poly_decompose(1, x) == [Poly(1, x)]
     assert poly_decompose(x, x) == [Poly(x, x)]
@@ -761,6 +798,15 @@ def test_decompose():
     assert poly_decompose(2*f.subs(x, g), x) == [Poly(2*f, x), Poly(g, x)]
     assert poly_decompose(f.subs(x, g-2), x) == [Poly(f.subs(x, x-2), x), Poly(g, x)]
 
+def test_reduce():
+    f = Poly(2930944*x**6 + 2198208*x**4 + 549552*x**2 + 45796, x)
+    g = Poly(17585664*x**5 + 8792832*x**3 + 1099104*x, x)
+
+    F, G = poly_reduce(f, g)
+
+    assert F == Poly(64*x**6 + 48*x**4 + 12*x**2 + 1, x)
+    assert G == Poly(384*x**5 + 192*x**3 + 24*x, x)
+
 def test_evaluate():
     f = x**2*y*z + 2*x*y*z**3 + 3*x*y + 4*y*z
 
@@ -822,6 +868,13 @@ def test_subs():
     assert p.subs(x, sin(x)) == \
         t*y**2*sin(x) + y*sin(x) + t**2
 
+    coeffs = [ Symbol('a' + str(i)) for i in [3,2,1,0] ]
+    values = [ 4, 0, 0, 1 ]
+
+    f = Poly(zip(coeffs, [3,2,1,0]), x)
+
+    assert f.subs(dict(zip(coeffs, values))) == Poly(4*x**3+1, x)
+
 def test_unify():
     p = Poly(x**2+x*y, x, y)
     q = Poly(x**2+2*x*y+1, x)
@@ -855,6 +908,9 @@ def test_diff():
 
     assert g.diff() == g
 
+def test_invert():
+    assert Poly(2*x, x).invert(x**2-16) == Poly(x/32, x)
+
 def test_eq_ne():
     p = Poly(x**2+x*y, x, y)
     q = Poly(x**2+x*y+1, x, y)
@@ -905,6 +961,9 @@ def test_sturm():
          Poly(Rational(-3303,100), x)]
 
 def test_number_of_real_roots():
+    assert number_of_real_roots(0, x) == 0
+    assert number_of_real_roots(7, x) == 0
+
     f = Poly(x - 1, x)
 
     assert number_of_real_roots(f) == 1
@@ -923,9 +982,10 @@ def test_number_of_real_roots():
     raises(ValueError, "number_of_real_roots(f, x, inf=t)")
     raises(ValueError, "number_of_real_roots(f, x, sup=t)")
 
-def test_roots():
+def test_roots_linear():
     assert roots_linear(Poly(2*x+1, x)) == [-Rational(1, 2)]
 
+def test_roots_quadratic():
     assert roots_quadratic(Poly(2*x**2, x)) == [0, 0]
     assert roots_quadratic(Poly(2*x**2+3*x, x)) == [0, -Rational(3, 2)]
     assert roots_quadratic(Poly(2*x**2+3, x)) == [I*sqrt(6)/2, -I*sqrt(6)/2]
@@ -933,12 +993,30 @@ def test_roots():
     assert roots_quadratic(Poly(2*x**2+4*x+3, x)) == \
         [-1 + I*sqrt(2)/2, -1 - I*sqrt(2)/2]
 
+def test_roots_cubic():
     assert roots_cubic(Poly(2*x**3, x)) == [0, 0, 0]
     assert roots_cubic(Poly(x**3-3*x**2+3*x-1, x)) == [1, 1, 1]
 
     assert roots_cubic(Poly(x**3+1, x)) == \
         [-1, S.Half - I*sqrt(3)/2, S.Half + I*sqrt(3)/2]
 
+def test_roots_quartic():
+    assert roots_quartic(Poly(x**4, x)) == [0, 0, 0, 0]
+    assert roots_quartic(Poly(x**4 + x**3, x)) in [
+        [-1,0,0,0],
+        [0,-1,0,0],
+        [0,0,-1,0],
+        [0,0,0,-1]
+    ]
+    assert roots_quartic(Poly(x**4 - x**3, x)) in [
+        [1,0,0,0],
+        [0,1,0,0],
+        [0,0,1,0],
+        [0,0,0,1]
+    ]
+    assert roots_quartic(Poly(x**4 + x, x)) == [S.Half + I*sqrt(3)/2, S.Half - I*sqrt(3)/2, 0, -1]
+
+def test_roots_binomial():
     assert roots_binomial(Poly(5*x, x)) == [0]
     assert roots_binomial(Poly(5*x**4, x)) == [0, 0, 0, 0]
     assert roots_binomial(Poly(5*x+2, x)) == [-Rational(2, 5)]
@@ -957,6 +1035,7 @@ def test_roots():
     assert powsimp(r0[0]) == powsimp(r1[0])
     assert powsimp(r0[1]) == powsimp(r1[1])
 
+def test_roots_rational():
     assert roots_rational(Poly(x**2-1, x)) == [S.One, -S.One]
     assert roots_rational(Poly(x**2-x, x)) == [S.Zero, S.One]
 
@@ -965,9 +1044,11 @@ def test_roots():
 
     assert roots_rational(Poly(t*x**2-x, x)) == []
 
+def test_roots():
     assert roots(1, x) == {}
     assert roots(x, x) == {S.Zero: 1}
     assert roots(x**9, x) == {S.Zero: 9}
+    assert roots(((x-2)*(x+3)*(x-4)).expand(), x) == {-S(3): 1, S(2): 1, S(4): 1}
 
     assert roots(2*x+1, x) == {-S.Half: 1}
     assert roots((2*x+1)**2, x) == {-S.Half: 2}
@@ -1002,10 +1083,22 @@ def test_roots():
         S.One: 1, -S.One: 1, I: 1, -I: 1
     }
 
+    assert roots(-2016*x**2 - 5616*x**3 - 2056*x**4 + 3324*x**5 + 2176*x**6 \
+        - 224*x**7 - 384*x**8 - 64*x**9, x) == {S(0): 2, -S(2): 2, S(2): 1, -S(7)/2: 1,\
+                                            -S(3)/2: 1, -S(1)/2: 1, S(3)/2: 1}
+
     assert roots((a+b+c)*x + a+b+c+d, x) == \
         { (-a-b-c-d) / (a+b+c) : 1 }
 
     assert roots(x**3+x**2-x+1, x, cubics=False) == {}
+    assert roots(((x-2)*(x+3)*(x-4)).expand(), x, cubics=False) == {-S(3): 1, S(2): 1, S(4): 1}
+    assert roots(((x-2)*(x+3)*(x-4)*(x-5)).expand(), x, cubics=False) == \
+            {-S(3): 1, S(2): 1, S(4): 1, S(5): 1}
+    assert roots(x**3 + 2*x**2 + 4*x + 8, x) == {-S(2): 1, -2*I: 1, 2*I: 1}
+    assert roots(x**3 + 2*x**2 + 4*x + 8, x, cubics=True) == \
+                {-2*I: 1, 2*I: 1, -S(2): 1}
+    assert roots((x**2 - x)*(x**3 + 2*x**2 + 4*x + 8), x ) == \
+                {S(1): 1, S(0): 1, -S(2): 1, -2*I: 1, 2*I: 1}
 
     r1_2, r1_3, r1_9, r4_9, r19_27 = [ Rational(*r) \
         for r in ((1,2), (1,3), (1,9), (4,9), (19,27)) ]
@@ -1039,14 +1132,14 @@ def test_roots():
 
     f = (x**2+2*x+3).subs(x, 2*x**2 + 3*x).subs(x, 5*x-4)
 
-    r1_2, r2_25, r13_20, r1_100 = [ Rational(*r) \
-        for r in ((1,2), (2,25), (13,20), (1,100)) ]
+    r1_2, r13_20, r1_100 = [ Rational(*r) \
+        for r in ((1,2), (13,20), (1,100)) ]
 
     assert roots(f, x) == {
-        r13_20 + r1_2*(r1_100 - r2_25*I*2**r1_2)**r1_2: 1,
-        r13_20 - r1_2*(r1_100 - r2_25*I*2**r1_2)**r1_2: 1,
-        r13_20 + r1_2*(r1_100 + r2_25*I*2**r1_2)**r1_2: 1,
-        r13_20 - r1_2*(r1_100 + r2_25*I*2**r1_2)**r1_2: 1,
+        r13_20 + r1_100*(25 - 200*I*2**r1_2)**r1_2: 1,
+        r13_20 - r1_100*(25 - 200*I*2**r1_2)**r1_2: 1,
+        r13_20 + r1_100*(25 + 200*I*2**r1_2)**r1_2: 1,
+        r13_20 - r1_100*(25 + 200*I*2**r1_2)**r1_2: 1,
     }
 
     p = Poly(z**3 + (-2 - y)*z**2 + (1 + 2*y - 2*x**2)*z - y + 2*x**2, z)
@@ -1057,11 +1150,7 @@ def test_roots():
         S.Half + S.Half*y - S.Half*(1 - 2*y + y**2 + 8*x**2)**S.Half: 1,
     }
 
-    assert roots(x**3 + 2*x**2 + 4*x + 8, x) == {}
-    assert roots(x**3 + 2*x**2 + 4*x + 8, x, cubics=True) == \
-        {-2*I: 1, 2*I: 1, -Integer(2): 1}
-
-    assert roots(a*b*c*x**3 + 2*x**2 + 4*x + 8, x) == {}
+    assert roots(a*b*c*x**3 + 2*x**2 + 4*x + 8, x, cubics=False) == {}
     assert roots(a*b*c*x**3 + 2*x**2 + 4*x + 8, x, cubics=True) != {}
 
     assert roots(x**4-1, x, domain='Z') == {S.One: 1, -S.One: 1}
@@ -1076,13 +1165,23 @@ def test_roots():
     assert roots(x**3, x, multiple=True) == [S.Zero, S.Zero, S.Zero]
     assert roots(1234, x, multiple=True) == []
 
-def test_factors():
-    assert poly_factors(1, x) == [Poly(1, x)]
-    assert poly_factors(x, x) == [Poly(x, x)]
+def test_roots2():
+    """Just test that calculating these roots does not hang
+    (final result is not checked)
+    """
+    a, b, c, d, x = symbols("a b c d x")
+    f1 = x**2*c + (a/b) + x*c*d - a
+    f2 = x**2*(a + b*(c-d)*a) + x*a*b*c/(b*d-d) + (a*d-c/d)
+    assert roots(f1, x).values() == [1, 1]
+    assert roots(f2, x).values() == [1, 1]
 
-    assert poly_factors(x**2-1, x) == [Poly(x-1, x), Poly(x+1, x)]
+def test_root_factors():
+    assert poly_root_factors(1, x) == [Poly(1, x)]
+    assert poly_root_factors(x, x) == [Poly(x, x)]
 
-    assert set( poly_factors((x**4 - 1)**2, x) ) == \
+    assert poly_root_factors(x**2-1, x) == [Poly(x-1, x), Poly(x+1, x)]
+
+    assert set( poly_root_factors((x**4 - 1)**2, x) ) == \
     set([Poly(((S.One, -I),     ((1,), (0,))), x),
          Poly(((S.One, -I),     ((1,), (0,))), x),
          Poly(((S.One, -S.One), ((1,), (0,))), x),
@@ -1092,7 +1191,7 @@ def test_factors():
          Poly(((S.One, I),      ((1,), (0,))), x),
          Poly(((S.One, I),      ((1,), (0,))), x)])
 
-    assert poly_factors(x**4-1, x, domain='Z') == \
+    assert poly_root_factors(x**4-1, x, domain='Z') == \
         [Poly(x-1, x), Poly(x+1, x), Poly(x**2+1, x)]
 
 def test_RootsOf():
@@ -1123,3 +1222,56 @@ def test_RootsOf():
     assert RootSum(Lambda(x, x), Poly(0, x), evaluate=False) != S.Zero
 
     assert RootSum(Lambda(x, x), Poly(x-1, x), evaluate=False).doit() == S.One
+
+def test_factor():
+    assert factor(-2) == -2
+    assert factor(-x) == -x
+
+    assert factor(Rational(3, 8)*x**2 - Rational(3, 2)) == \
+        Rational(3, 8)*((x - 2)*(x + 2))
+
+    assert factor(x**3 - 1) == (x - 1)*(x**2 + x + 1)
+    assert factor(x**3 - x) == x*(x - 1)*(x + 1)
+
+    assert factor(x**2 + 2*x + 1) == (x + 1)**2
+    assert factor(x**2 +   x - 2) == (x - 1)*(x + 2)
+
+    assert factor(2*x**2 + 5*x + 2) == (x + 2)*(2*x + 1)
+
+    assert factor(x**2 + y**2) == x**2 + y**2
+    assert factor(x**5 - y**2) == x**5 - y**2
+
+    assert factor(x*y + x*z + y*z) == x*y + x*z + y*z
+    assert factor(x*y + x*z + x**2) == x*(x + y + z)
+
+    assert poly_factors((a*x - b)**5, x) == \
+        (1, [(Poly(a*x - b, x), 5)])
+
+    assert poly_factors((a*x - b)**5, x, a) == \
+        (1, [(Poly(x*a - b, x, a), 5)])
+
+    assert poly_factors((a*x - b)**5, x, a, b) == \
+        (1, [(Poly(x*a - b, x, a, b), 5)])
+
+    assert poly_factors(-2*x**2 + x, x) == \
+        (-1, [(Poly(x, x), 1),
+              (Poly(2*x - 1, x), 1)])
+
+    assert poly_factors(x**3 - 3*x**2 + 3*x - 1, x) == \
+        (1, [(Poly(x - 1, x), 3)])
+
+    assert poly_factors(x**6 - 1, x) == \
+        (1, [(Poly(x - 1, x), 1),
+             (Poly(x + 1, x), 1),
+             (Poly(x**2 - x + 1, x), 1),
+             (Poly(x**2 + x + 1, x), 1)])
+
+    assert poly_factors(2*x**3*y - 2*a**2*x*y - 3*a**2*x**2 + 3*a**4, a, x, y) == \
+        (1, [(Poly(a - x, a, x, y), 1),
+             (Poly(a + x, a, x, y), 1),
+             (Poly(3*a**2 - 2*x*y, a, x, y), 1)])
+
+    assert poly_factors(x**20 - z**5*y**20, x, y, z) == \
+        (1, [(Poly(-y**4*z + x**4, x, y, z), 1),
+             (Poly(y**16*z**4 + x**4*y**12*z**3 + x**8*y**8*z**2 + x**12*y**4*z + x**16, x, y, z) , 1)])
+
