@@ -1,25 +1,77 @@
-from sympy.core import S, C
+"""
+A Printer which converts an expression into its LaTeX equivalent.
+"""
+
+from sympy.core import S, C, Basic, Symbol
 from printer import Printer
 from sympy.simplify import fraction
+import re
 
 class LatexPrinter(Printer):
-    """A printer which converts an expression into its LaTeX equivalent."""
+    printmethod = "_latex_"
 
-    def __init__(self, inline=True):
+    def __init__(self, profile=None):
         Printer.__init__(self)
-        self._inline = inline
+
+        self._settings = {
+            "inline" : True,
+            "fold_frac_powers" : False,
+            "fold_func_brackets" : False,
+            "mul_symbol" : None,
+            "inv_trig_style" : "abbreviated"
+        }
+
+        if profile is not None:
+            self._settings.update(profile)
 
     def doprint(self, expr):
         tex = Printer.doprint(self, expr)
 
-        if self._inline:
+        if self._settings['inline']:
             return r"$%s$" % tex
         else:
             return r"\begin{equation*}%s\end{equation*}" % tex
 
     def _needs_brackets(self, expr):
+        """
+        Returns True if the expression needs to be wrapped in brackets when
+        printed, False otherwise. For example: a + b => True; a => False;
+        10 => False; -10 => True.
+        """
         return not ((expr.is_Integer and expr.is_nonnegative) or expr.is_Atom)
 
+    def _needs_function_brackets(self, expr):
+        """
+        Returns True if the expression needs to be wrapped in brackets when
+        passed as an argument to a function, False otherwise. This is a more
+        liberal version of _needs_brackets, in that many expressions which need
+        to be wrapped in brackets when added/substracted/raised to a power do
+        not need them when passed to a function. Such an example is a*b.
+        """
+        if not self._needs_brackets(expr):
+            return False
+        else:
+            # Muls of the form a*b*c... can be folded
+            if expr.is_Mul and not self._mul_is_clean(expr):
+                return True
+            # Pows which don't need brackets can be folded
+            elif expr.is_Pow and not self._pow_is_clean(expr):
+                return True
+            # Add and Function always need brackets
+            elif expr.is_Add or expr.is_Function:
+                return True
+            else:
+                return False
+
+    def _mul_is_clean(self, expr):
+        for arg in expr.args:
+            if arg.is_Function:
+                return False
+        return True
+
+    def _pow_is_clean(self, expr):
+        return not self._needs_brackets(expr.base)
+
     def _do_exponent(self, expr, exp):
         if exp is not None:
             return r"\left(%s\right)^{%s}" % (expr, exp)
@@ -27,9 +79,12 @@ class LatexPrinter(Printer):
             return expr
 
     def _print_Add(self, expr):
-        tex = str(self._print(expr.args[0]))
+        args = list(expr.args)
+        args.sort(Basic._compare_pretty)
 
-        for term in expr.args[1:]:
+        tex = str(self._print(args[0]))
+
+        for term in args[1:]:
             coeff = term.as_coeff_terms()[0]
 
             if coeff.is_negative:
@@ -50,6 +105,15 @@ class LatexPrinter(Printer):
 
         numer, denom = fraction(C.Mul(*terms))
 
+        mul_symbol_table = {
+            None : r" ",
+            "ldot" : r" \,.\, ",
+            "dot" : r" \cdot ",
+            "times" : r" \times "
+        }
+
+        seperator = mul_symbol_table[self._settings['mul_symbol']]
+
         def convert(terms):
             product = []
 
@@ -64,11 +128,11 @@ class LatexPrinter(Printer):
                     else:
                         product.append(str(pretty))
 
-                return r" ".join(product)
+                return seperator.join(product)
 
         if denom is S.One:
             if coeff is not S.One:
-                tex += str(self._print(coeff)) + " "
+                tex += str(self._print(coeff)) + seperator
 
             if numer.is_Add:
                 tex += r"\left(%s\right)" % convert(numer)
@@ -109,14 +173,29 @@ class LatexPrinter(Printer):
                 return r"\frac{1}{%s}" % tex
             else:
                 return tex
+        elif self._settings['fold_frac_powers'] \
+             and expr.exp.is_Rational \
+             and expr.exp.q != 1:
+            base, p, q = self._print(expr.base), expr.exp.p, expr.exp.q
+            return r"%s^{%s/%s}" % (base, p, q)
         else:
             if expr.base.is_Function:
                 return self._print(expr.base, self._print(expr.exp))
             else:
+                if expr.exp == S.NegativeOne:
+                    #solves issue 1030
+                    #As Mul always simplify 1/x to x**-1
+                    #The objective is achieved with this hack
+                    #first we get the latex for -1 * expr,
+                    #which is a Mul expression
+                    tex = self._print(S.NegativeOne * expr).strip()
+                    #the result comes with a minus and a space, so we remove
+                    if tex[:1] == "-":
+                        return tex[1:].strip()
                 if self._needs_brackets(expr.base):
                     tex = r"\left(%s\right)^{%s}"
                 else:
-                    tex = r"{%s}^{%s}"
+                    tex = r"%s^{%s}"
 
                 return tex % (self._print(expr.base),
                               self._print(expr.exp))
@@ -160,7 +239,7 @@ class LatexPrinter(Printer):
             tex += r"\int"
 
             if limits is not None:
-                if not self._inline:
+                if not self._settings['inline']:
                     tex += r"\limits"
 
                 tex += "_{%s}^{%s}" % (self._print(limits[0]),
@@ -187,13 +266,48 @@ class LatexPrinter(Printer):
             return getattr(self, '_print_' + func)(expr, exp)
         else:
             args = [ str(self._print(arg)) for arg in expr.args ]
+            # How inverse trig functions should be displayed, formats are:
+            # abbreviated: asin, full: arcsin, power: sin^-1
+            inv_trig_style = self._settings['inv_trig_style']
+            # If we are dealing with a power-style inverse trig function
+            inv_trig_power_case = False
+            # If it is applicable to fold the argument brackets
+            can_fold_brackets = self._settings['fold_func_brackets'] and \
+                                len(args) == 1 and \
+                                not self._needs_function_brackets(expr.args[0])
 
-            if exp is not None:
+            inv_trig_table = ["asin", "acos", "atan", "acot"]
+
+            # If the function is an inverse trig function, handle the style
+            if func in inv_trig_table:
+                if inv_trig_style == "abbreviated":
+                    func = func
+                elif inv_trig_style == "full":
+                    func = "arc" + func[1:]
+                elif inv_trig_style == "power":
+                    func = func[1:]
+                    inv_trig_power_case = True
+
+                    # Can never fold brackets if we're raised to a power
+                    if exp is not None:
+                        can_fold_brackets = False
+
+            if inv_trig_power_case:
+                name = r"\operatorname{%s}^{-1}" % func
+            elif exp is not None:
                 name = r"\operatorname{%s}^{%s}" % (func, exp)
             else:
                 name = r"\operatorname{%s}" % func
 
-            return name + r"\left(%s\right)" % ",".join(args)
+            if can_fold_brackets:
+                name += r"%s"
+            else:
+                name += r"\left(%s\right)"
+
+            if inv_trig_power_case and exp is not None:
+                name += r"^{%s}" % exp
+
+            return name % ",".join(args)
 
     def _print_floor(self, expr, exp=None):
         tex = r"\lfloor{%s}\rfloor" % self._print(expr.args[0])
@@ -244,7 +358,7 @@ class LatexPrinter(Printer):
             return tex
 
     def _print_exp(self, expr, exp=None):
-        tex = r"{e}^{%s}" % self._print(expr.args[0])
+        tex = r"e^{%s}" % self._print(expr.args[0])
         return self._do_exponent(tex, exp)
 
     def _print_gamma(self, expr, exp=None):
@@ -290,7 +404,14 @@ class LatexPrinter(Printer):
 
     def _print_Rational(self, expr):
         if expr.q != 1:
-            return r"\frac{%d}{%d}" % (expr.p, expr.q)
+            sign = ""
+            p = expr.p
+            if expr.p < 0:
+                sign = "- "
+                p = -p
+            return r"%s\frac{%d}{%d}" % (sign, p, expr.q)
+        else:
+            return self._print(expr.p)
 
     def _print_Infinity(self, expr):
         return r"\infty"
@@ -324,6 +445,22 @@ class LatexPrinter(Printer):
         if len(expr.name) == 1:
             return expr.name
 
+        #convert trailing digits to subscript
+        m = re.match('(^[a-zA-Z]+)([0-9]+)$', expr.name)
+        if m is not None:
+            name, sub=m.groups()
+            tex=self._print_Symbol(Symbol(name))
+            tex="%s_{%s}" %(tex, sub)
+            return tex
+
+        # insert braces to expresions containing '_' or '^'
+        m = re.match('(^[a-zA-Z0-9]+)([_\^]{1})([a-zA-Z0-9]+)$', expr.name)
+        if m is not None:
+            name, sep, rest=m.groups()
+            tex=self._print_Symbol(Symbol(name))
+            tex="%s%s{%s}" %(tex, sep, rest)
+            return tex
+
         greek = set([ 'alpha', 'beta', 'gamma', 'delta', 'epsilon', 'zeta',
                       'eta', 'theta', 'iota', 'kappa', 'lambda', 'mu', 'nu',
                       'xi', 'omicron', 'pi', 'rho', 'sigma', 'tau', 'upsilon',
@@ -350,13 +487,26 @@ class LatexPrinter(Printer):
         return "%s %s %s" % (self._print(expr.lhs),
             charmap[expr.rel_op], self._print(expr.rhs))
 
+    def _print_Piecewise(self, expr):
+        ecpairs = [r"%s & for %s" % (self._print(e), self._print(c)) \
+                       for e, c in expr.args[:-1]]
+        if expr.args[-1].cond is S.One:
+            ecpairs.append(r"%s & \textrm{otherwise}" % \
+                               self._print(expr.args[-1].expr))
+        else:
+            ecpairs.append(r"%s & for %s" % \
+                           (self._print(expr.args[-1].cond),
+                            self._print(expr.args[-1].expr)))
+        tex = r"\left\{\begin{array}{cl} %s \end{array}\right."
+        return tex % r" \\".join(ecpairs)
+
     def _print_Matrix(self, expr):
         lines = []
 
         for line in range(expr.lines): # horrible, should be 'rows'
             lines.append(" & ".join([ self._print(i) for i in expr[line,:] ]))
 
-        if self._inline:
+        if self._settings['inline']:
             tex = r"\left(\begin{smallmatrix}%s\end{smallmatrix}\right)"
         else:
             tex = r"\begin{pmatrix}%s\end{pmatrix}"
@@ -374,12 +524,23 @@ class LatexPrinter(Printer):
     def _print_dict(self, expr):
         items = []
 
-        for key, val in expr.iteritems():
+        keys = expr.keys()
+        keys.sort(Basic.compare_pretty)
+        for key in keys:
+            val = expr[key]
             items.append("%s : %s" % (self._print(key), self._print(val)))
 
         return r"\begin{Bmatrix}%s\end{Bmatrix}" % r", & ".join(items)
 
-def latex(expr, inline=True):
+    def _print_DiracDelta(self, expr):
+        if len(expr.args) == 1 or expr.args[1] == 0:
+            tex = r"\delta\left(%s\right)" % self._print(expr.args[0])
+        else:
+            tex = r"\delta^{\left( %s \right)}\left( %s \right)" % (\
+            self._print(expr.args[1]), self._print(expr.args[0]))
+        return tex
+
+def latex(expr, profile=None, **kargs):
     r"""Convert the given expression to LaTeX representation.
 
         You can specify how the generated code will be delimited.
@@ -405,7 +566,12 @@ def latex(expr, inline=True):
 
     """
 
-    return LatexPrinter(inline).doprint(expr)
+    if profile is not None:
+        profile.update(kargs)
+    else:
+        profile = kargs
+
+    return LatexPrinter(profile).doprint(expr)
 
 def print_latex(expr):
     """Prints LaTeX representation of the given expression."""
