@@ -1,16 +1,26 @@
+"""
+A MathML printer.
+"""
+
 from sympy import Basic, sympify
 from printer import Printer
 
-
 class MathMLPrinter(Printer):
-    """A MathML printer."""
+    """Prints an expression to the MathML markup language
+
+    Whenever possible tries to use Content markup and not Presentation markup.
+
+    References: http://www.w3.org/TR/MathML2/
+    """
+    printmethod = "_mathml_"
+
     def __init__(self, *args, **kwargs):
         Printer.__init__(self, *args, **kwargs)
         from xml.dom.minidom import Document
         self.dom = Document()
 
     def doprint(self, e):
-        return self._print(e).toprettyxml()
+        return self._print(e).toxml()
 
     def mathml_tag(self, e):
         """Returns the MathML tag for an expression."""
@@ -19,6 +29,7 @@ class MathMLPrinter(Printer):
             'Mul': 'times',
             'Derivative': 'diff',
             'Number': 'cn',
+            'int': 'cn',
             'Pow': 'power',
             'Symbol': 'ci',
             'Integral': 'int'
@@ -48,34 +59,40 @@ class MathMLPrinter(Printer):
 
         x_1 = self.dom.createElement('bvar')
         x_2 = self.dom.createElement('lowlimit')
-        x_1.appendChild(self._print(e.x))
-        x_2.appendChild(self._print(e.x0))
+        x_1.appendChild(self._print(e.args[1]))
+        x_2.appendChild(self._print(e.args[2]))
 
         x.appendChild(x_1)
         x.appendChild(x_2)
-        x.appendChild(self._print(e.e))
+        x.appendChild(self._print(e.args[0]))
 
         return x
 
+
     def _print_Integral(self, e):
-        # FIXME doesn't work -- needs to be updated to the new Integral class
-        x = self.dom.createElement('apply')
-        x.appendChild(self.dom.createElement(self.mathml_tag(e)))
+        def lime_recur(limits):
+            x = self.dom.createElement('apply')
+            x.appendChild(self.dom.createElement(self.mathml_tag(e)))
+            bvar_elem = self.dom.createElement('bvar')
+            bvar_elem.appendChild(self._print(limits[0][0]))
+            x.appendChild(bvar_elem)
 
-        x_1 = self.dom.createElement('bvar')
-        x_2 = self.dom.createElement('lowlimit')
-        x_3 = self.dom.createElement('uplimit')
+            if limits[0][1]:
+                low_elem = self.dom.createElement('lowlimit')
+                low_elem.appendChild(self._print(limits[0][1][0]))
+                x.appendChild(low_elem)
+                up_elem = self.dom.createElement('uplimit')
+                up_elem.appendChild(self._print(limits[0][1][1]))
+                x.appendChild(up_elem)
+            if len(limits) == 1:
+                x.appendChild(self._print(e.function))
+            else:
+                x.appendChild(lime_recur(limits[1:]))
+            return x
 
-        #x_1.appendChild(self._print(e.x))
-        #x_2.appendChild(self._print(e.a))
-        #x_3.appendChild(self._print(e.b))
-
-        x.appendChild(x_1)
-        x.appendChild(x_2)
-        x.appendChild(x_3)
-        x.appendChild(self._print(e.f))
-
-        return x
+        limits = list(e.limits)
+        limits.reverse()
+        return lime_recur(limits)
 
     def _print_Symbol(self, sym):
         x = self.dom.createElement(self.mathml_tag(sym))
@@ -128,6 +145,19 @@ class MathMLPrinter(Printer):
             x.appendChild(self._print(arg))
         return x
 
+    def _print_list(self, seq):
+        """MathML reference for the <list> element:
+        http://www.w3.org/TR/MathML2/chapter4.html#contm.list"""
+        dom_element = self.dom.createElement('list')
+        for item in seq:
+            dom_element.appendChild(self._print(item))
+        return dom_element
+
+    def _print_int(self, p):
+        dom_element = self.dom.createElement(self.mathml_tag(p))
+        dom_element.appendChild(self.dom.createTextNode(str(p)))
+        return dom_element
+
 def mathml(expr):
     """Returns the MathML representation of expr"""
     s = MathMLPrinter()
@@ -143,12 +173,13 @@ def print_mathml(expr):
     >>> print_mathml(x+1) #doctest: +NORMALIZE_WHITESPACE
     <apply>
         <plus/>
-        <ci>
-                x
-        </ci>
         <cn>
                 1
         </cn>
+        <ci>
+                x
+        </ci>
     </apply>
     """
-    print mathml(expr)
+    s = MathMLPrinter()
+    print s._print(sympify(expr)).toprettyxml()
