@@ -1,8 +1,9 @@
-from sympy.core import S, C
+from sympy.core import S, C, Basic
 from sympy.printing.printer import Printer
+from sympy.printing.str import sstr
 from stringpict import prettyForm, stringPict
 
-from pretty_symbology import xstr, hobj, vobj, xobj, xrel, pretty_symbol,\
+from pretty_symbology import xstr, hobj, vobj, xobj, xsym, pretty_symbol,\
         pretty_atom, pretty_use_unicode, pretty_try_use_unicode, greek
 
 # rename for usage from outside
@@ -13,10 +14,18 @@ pprint_try_use_unicode = pretty_try_use_unicode
 class PrettyPrinter(Printer):
     """Printer, which converts an expression into 2D ascii-art figure."""
 
-    def __init__(self, use_unicode=None):
+    def __init__(self, profile=None):
         Printer.__init__(self)
         self.emptyPrinter = lambda x : prettyForm(xstr(x))
 
+        self._settings = {
+                "full_prec" : "auto",
+                "use_unicode" : True,
+        }
+
+        if profile is not None:
+            self._settings.update(profile)
+
     def doprint(self, expr):
         return self._print(expr).terminal_string()
 
@@ -24,22 +33,27 @@ class PrettyPrinter(Printer):
     def _print_stringPict(self, e):
         return e
 
-    def _print_str(self, e):
-        return prettyForm(repr(e))
-
-    def _print_unicode(self, e):
+    def _print_basestring(self, e):
         return prettyForm(repr(e))
 
     def _print_Symbol(self, e):
         symb = pretty_symbol(e.name)
         return prettyForm(symb)
 
+    def _print_Real(self, e):
+        # we will use StrPrinter's Real printer, but we need to handle the
+        # full_prec ourselves, according to the self._print_level
+        full_prec = self._settings["full_prec"]
+        if  full_prec == "auto":
+            full_prec = self._print_level == 1
+        return prettyForm(sstr(e, full_prec=full_prec))
+
     def _print_Atom(self, e):
         try:
             # print atoms like Exp1 or Pi
             return prettyForm(pretty_atom(e.__class__.__name__))
         except KeyError:
-            pass
+            return self.emptyPrinter(e)
 
     # Infinity inherits from Rational, so we have to override _print_XXX order
     _print_Infinity         = _print_Atom
@@ -55,7 +69,7 @@ class PrettyPrinter(Printer):
         return pform
 
     def _print_Relational(self, e):
-        op = prettyForm(' ' + xrel(e.rel_op) + ' ')
+        op = prettyForm(' ' + xsym(e.rel_op) + ' ')
 
         l = self._print(e.lhs)
         r = self._print(e.rhs)
@@ -113,6 +127,23 @@ class PrettyPrinter(Printer):
         pform = prettyForm(*stringPict.next(pform, f))
         return pform
 
+    def _print_PDF(self, pdf):
+        lim = self._print(pdf.pdf.args[0])
+        lim = prettyForm(*lim.right(', '))
+        lim = prettyForm(*lim.right(self._print(pdf.domain[0])))
+        lim = prettyForm(*lim.right(', '))
+        lim = prettyForm(*lim.right(self._print(pdf.domain[1])))
+        lim = prettyForm(*lim.parens())
+
+        f = self._print(pdf.pdf.args[1])
+        f = prettyForm(*f.right(', '))
+        f = prettyForm(*f.right(lim))
+        f = prettyForm(*f.parens())
+
+        pform = prettyForm('PDF')
+        pform = prettyForm(*pform.right(f))
+        return pform
+
     def _print_Integral(self, integral):
         f   = integral.function
 
@@ -223,44 +254,116 @@ class PrettyPrinter(Printer):
             for j in range(M.cols):
                 S[i,j] = self._print(M[i,j])
 
-        # max w/h for elements
-        maxw = max([s.width()  for s in S.values()])
-        maxh = max([s.height() for s in S.values()])
+        # h- and v- spacers
+        hsep = 2
+        vsep = 1
+
+        # max width for columns
+        maxw = [-1] * M.cols
+
+        for j in range(M.cols):
+            maxw[j] = max([S[i,j].width()  for i in range(M.lines)])
+
 
         # drawing result
         D = None
 
-        # XXX at present, we reshape each cell to be of the same size,
-        # XXX and it is ugly most of the time!
         for i in range(M.lines):
 
             D_row = None
             for j in range(M.cols):
                 s = S[i,j]
 
-                # reshape s to maxw/maxh
+                # reshape s to maxw
                 # XXX this should be generalized, and go to stringPict.reshape ?
-                while s.width() < maxw:
-                    s = prettyForm(*s.left(' '))    # right align
-                while s.height() < maxh:
-                    s = prettyForm(*s.above(' '))   # down align
+                assert s.width()  <= maxw[j]
+
+                # hcenter it, +0.5 to the right                        2
+                # ( it's better to align formula starts for say 0 and r )
+                # XXX this is not good in all cases -- maybe introduce vbaseline?
+                wdelta = maxw[j] - s.width()
+                wleft  = wdelta // 2
+                wright = wdelta - wleft
+
+                s = prettyForm(*s.right(' '*wright))
+                s = prettyForm(*s.left (' '*wleft))
+
+                # we don't need vcenter cells -- this is automatically done in
+                # a pretty way because when their baselines are taking into
+                # account in .right()
 
                 if D_row is None:
                     D_row = s   # first box in a row
                     continue
 
-                D_row = prettyForm(*D_row.right(' '))
+                D_row = prettyForm(*D_row.right(' '*hsep))  # h-spacer
                 D_row = prettyForm(*D_row.right(s))
 
             if D is None:
                 D = D_row       # first row in a picture
                 continue
 
+            # v-spacer
+            for _ in range(vsep):
+                D = prettyForm(*D.below(' '))
+
             D = prettyForm(*D.below(D_row))
 
         D = prettyForm(*D.parens('[',']'))
         return D
 
+    def _print_Piecewise(self, pexpr):
+
+        P = {}
+        for n, ec in enumerate(pexpr.args):
+            P[n,0] = self._print(ec.expr)
+            if ec.cond is S.One:
+                P[n,1] = prettyForm('otherwise')
+            else:
+                P[n,1] = prettyForm(*prettyForm('for ').right(self._print(ec.cond)))
+        hsep = 2
+        vsep = 1
+        len_args = len(pexpr.args)
+
+        # max widths
+        maxw = [max([P[i,j].width() for i in xrange(len_args)]) \
+                    for j in xrange(2)]
+
+        # FIXME: Refactor this code and matrix into some tabular environment.
+        # drawing result
+        D = None
+
+        for i in xrange(len_args):
+            D_row = None
+            for j in xrange(2):
+                p = P[i,j]
+                assert p.width() <= maxw[j]
+
+                wdelta = maxw[j] - p.width()
+                wleft  = wdelta // 2
+                wright = wdelta - wleft
+
+                p = prettyForm(*p.right(' '*wright))
+                p = prettyForm(*p.left (' '*wleft))
+
+                if D_row is None:
+                    D_row = p
+                    continue
+
+                D_row = prettyForm(*D_row.right(' '*hsep))  # h-spacer
+                D_row = prettyForm(*D_row.right(p))
+            if D is None:
+                D = D_row       # first row in a picture
+                continue
+
+            # v-spacer
+            for _ in range(vsep):
+                D = prettyForm(*D.below(' '))
+
+            D = prettyForm(*D.below(D_row))
+
+        D = prettyForm(*D.parens('{',''))
+        return D
 
     def _print_exp(self, e):
         base = prettyForm(pretty_atom('Exp1', 'e'))
@@ -296,7 +399,7 @@ class PrettyPrinter(Printer):
 
     def _print_Add(self, sum):
         args = list(sum.args)
-        args.sort(sum.compare_terms)
+        args.sort(Basic._compare_pretty)
         pforms = []
         for x in args:
             # Check for negative "things" so that this information can be enforce upon
@@ -335,7 +438,7 @@ class PrettyPrinter(Printer):
         for item in product.args:
             if item.is_Pow and item.exp.is_Rational and item.exp.is_negative:
                 b.append(C.Pow(item.base, -item.exp))
-            elif item.is_Rational:
+            elif item.is_Rational and item is not S.Infinity:
                 if item.p != 1:
                     a.append( C.Rational(item.p) )
                 if item.q != 1:
@@ -426,6 +529,8 @@ class PrettyPrinter(Printer):
                 return prettyForm(binding=prettyForm.NEG, *pform.left('- '))
             else:
                 return prettyForm(str(r.p))/prettyForm(str(r.q))
+        else:
+            return self.emptyPrinter(r)
 
 
     def _print_seq(self, seq, left=None, right=None):
@@ -459,25 +564,44 @@ class PrettyPrinter(Printer):
 
     def _print_dict(self, d):
         items = []
-        for k,v in d.items():
+
+        keys = d.keys()
+        keys.sort( Basic.compare_pretty )
+
+        for k in keys:
             K = self._print(k)
-            V = self._print(v)
+            V = self._print(d[k])
             S = prettyForm(*stringPict.next(K, ': ', V))
 
             items.append(S)
 
         return self._print_seq(items, '{', '}')
 
+    def __print_set(self, s):
+        items = list(s)
+        items.sort( Basic.compare_pretty )
 
-def pretty(expr, use_unicode=None):
+        S = self._print_seq(items, '(', ')')
+        S = prettyForm(*stringPict.next(type(s).__name__, S))
+        return S
+
+    _print_set       = __print_set
+    _print_frozenset = __print_set
+
+
+def pretty(expr, profile=None, **kargs):
     """
     Returns a string containing the prettified form of expr. If use_unicode
     is set to True then certain expressions will use unicode characters,
     such as the greek letter pi for Pi instances.
     """
-    uflag = pretty_use_unicode(use_unicode)
+    if profile is not None:
+        profile.update(kargs)
+    else:
+        profile = kargs
+    uflag = pretty_use_unicode(kargs.get("use_unicode", None))
     try:
-        pp = PrettyPrinter()
+        pp = PrettyPrinter(profile)
         return pp.doprint(expr)
     finally:
         pretty_use_unicode(uflag)
