@@ -1,5 +1,4 @@
-from sympy.core import S, Add, sympify, Basic
-from sympy.core.methods import NoRelMeths, ArithMeths
+from sympy.core import S, Add, sympify, Basic, PoleError, Mul, oo, C
 from gruntz import gruntz
 
 def limit(e, z, z0, dir="+"):
@@ -14,6 +13,8 @@ def limit(e, z, z0, dir="+"):
 
     Examples:
 
+    >>> from sympy import limit, sin, Symbol
+    >>> x = Symbol('x')
     >>> limit(sin(x)/x, x, 0)
     1
     >>> limit(1/x, x, 0, dir="+")
@@ -62,7 +63,7 @@ def limit(e, z, z0, dir="+"):
                         return -S.Infinity
 
     if e.is_Add:
-        if e.is_polynomial():
+        if e.is_polynomial() and z0.is_finite:
             return Add(*[limit(term, z, z0, dir) for term in e.args])
         else:
             # this is a case like limit(x*y+x*z, z, 2) == x*y+2*x
@@ -72,15 +73,42 @@ def limit(e, z, z0, dir="+"):
             if r is not S.NaN:
                 return r
 
-    return gruntz(e, z, z0, dir)
+    try:
+        r = gruntz(e, z, z0, dir)
+    except PoleError:
+        r = heuristics(e, z, z0, dir)
+    return r
 
-class Limit(Basic, NoRelMeths, ArithMeths):
+def heuristics(e, z, z0, dir):
+    if z0 == oo:
+        return heuristics(e.subs(z, 1/z), z, sympify(0), "+")
+    elif e.is_Mul:
+        r = []
+        for a in e.args:
+            if not a.is_bounded:
+                r.append(a.limit(z, z0, dir))
+        if not (r is []):
+            return Mul(*r)
+    elif e.is_Add:
+        r = []
+        for a in e.args:
+            r.append(a.limit(z, z0, dir))
+        return Add(*r)
+    elif isinstance(e, C.Function):
+        return e.subs(e.args[0], heuristics(e.args[0], z, z0, dir))
+    msg = "Don't know how to calculate the limit(%s, %s, %s, dir=%s), sorry."
+    raise PoleError(msg % (e, z, z0, dir))
+
+
+class Limit(Basic):
     """Represents unevaluated limit.
 
     Examples:
 
+    >>> from sympy import limit, sin, Symbol
+    >>> x = Symbol('x')
     >>> Limit(sin(x)/x, x, 0)
-    Limit(1/x*sin(x), x, 0, dir='+')
+    Limit(sin(x)/x, x, 0)
     >>> Limit(1/x, x, 0, dir="-")
     Limit(1/x, x, 0, dir='-')
     """
@@ -96,8 +124,3 @@ class Limit(Basic, NoRelMeths, ArithMeths):
     def doit(self):
         e, z, z0, dir = self.args
         return limit(e, z, z0, dir)
-
-    def tostr(self, level=0):
-        e, z, z0, dir = self.args
-        return "Limit(%s, %s, %s, dir='%s')" % (e, z, z0, dir)
-
