@@ -1,3 +1,4 @@
+from sympy import SYMPY_DEBUG
 
 from sympy.core import Basic, S, C, Add, Mul, Pow, Rational, Integer, \
         Derivative, Wild, Symbol, sympify
@@ -7,9 +8,11 @@ from sympy.core.numbers import igcd
 from sympy.utilities import make_list, all
 from sympy.functions import gamma, exp, sqrt
 
+from sympy.simplify.cse_main import cse
+
 from sympy.polys import Poly
 
-from sys import maxint
+import sympy.mpmath as mpmath
 
 def fraction(expr, exact=False):
     """Returns a pair with expression's numerator and denominator.
@@ -60,10 +63,6 @@ def fraction(expr, exact=False):
     """
     expr = sympify(expr)
 
-    #XXX this only works sometimes (caching bug?)
-    if expr == exp(-Symbol("x")) and exact:
-        return (expr, 1)
-
     numer, denom = [], []
 
     for term in make_list(expr, Mul):
@@ -160,7 +159,6 @@ def separate(expr, deep=False):
 
     if expr.is_Pow:
         terms, expo = [], separate(expr.exp, deep)
-        #print expr, terms, expo, expr.base
 
         if expr.base.is_Mul:
             t = [ separate(C.Pow(t,expo), deep) for t in expr.base.args ]
@@ -208,7 +206,7 @@ def together(expr, deep=False):
        #(z*x + x*y + z*y)/(y*x*z)
 
        >>> together(1/(x*y) + 1/y**2)
-       1/x*y**(-2)*(x + y)
+       (x + y)/(x*y**2)
 
        Or you can just denest multi-level fractional expressions:
 
@@ -248,7 +246,6 @@ def together(expr, deep=False):
                     expo = S.One
                     coeff = S.One
 
-
                     if term.is_Pow:
                         if term.exp.is_Rational:
                             term, expo = term.base, term.exp
@@ -259,10 +256,10 @@ def together(expr, deep=False):
                                 term, expo = Pow(term.base, tail), coeff
                         coeff = S.One
                     elif term.func is C.exp:
-                        if term[0].is_Rational:
-                            term, expo = C.E, term[0]
-                        elif term[0].is_Mul:
-                            coeff, tail = term[0].as_coeff_terms()
+                        if term.args[0].is_Rational:
+                            term, expo = S.Exp1, term.args[0]
+                        elif term.args[0].is_Mul:
+                            coeff, tail = term.args[0].as_coeff_terms()
                             if coeff.is_Rational:
                                 tail = C.Mul(*tail)
                                 term, expo = C.exp(tail), coeff
@@ -295,7 +292,7 @@ def together(expr, deep=False):
                 basis[term] = (total, total-maxi)
 
                 if term.func is C.exp:
-                    denominator.append(C.exp(maxi*term[:]))
+                    denominator.append(C.exp(maxi*term.args[0]))
                 else:
                     if maxi is S.One:
                         denominator.append(term)
@@ -323,7 +320,7 @@ def together(expr, deep=False):
                         expo = total-sub
 
                     if term.func is C.exp:
-                        expr.append(C.exp(expo*term[:]))
+                        expr.append(C.exp(expo*term.args[0]))
                     else:
                         if expo is S.One:
                             expr.append(term)
@@ -392,39 +389,36 @@ def collect(expr, syms, evaluate=True, exact=False):
        >>> collect(x**2 + y*x**2 + x*y + y + a*y, [x, y])
        x*y + y*(1 + a) + x**2*(1 + y)
 
-       >>> collect(x**2*y**4 + z*(x*y**2)**2 + z + a*z, [x*y**2, z])
-       z*(1 + a) + x**2*y**4*(1 + z)
-
        Also more complicated expressions can be used as patterns:
 
        >>> collect(a*sin(2*x) + b*sin(2*x), sin(2*x))
        (a + b)*sin(2*x)
 
-       >>> collect(a*x**2*log(x)**2 + b*(x*log(x))**2, x*log(x))
-       x**2*log(x)**2*(a + b)
+       >>> collect(a*x*log(x) + b*(x*log(x)), x*log(x))
+       x*(a + b)*log(x)
 
        It is also possible to work with symbolic powers, although
        it has more complicated behaviour, because in this case
        power's base and symbolic part of the exponent are treated
        as a single symbol:
 
-       #>>> collect(a*x**c + b*x**c, x)
-       #a*x**c + b*x**c
+       >>> collect(a*x**c + b*x**c, x)
+       a*x**c + b*x**c
 
-       #>>> collect(a*x**c + b*x**c, x**c)
-       #x**c*(a + b)
+       >>> collect(a*x**c + b*x**c, x**c)
+       x**c*(a + b)
 
        However if you incorporate rationals to the exponents, then
        you will get well known behaviour:
 
-       #>>> collect(a*x**(2*c) + b*x**(2*c), x**c)
-       #x**(2*c)*(a + b)
+       >>> collect(a*x**(2*c) + b*x**(2*c), x**c)
+       (x**2)**c*(a + b)
 
        Note also that all previously stated facts about 'collect'
        function apply to the exponential function, so you can get:
 
-       #>>> collect(a*exp(2*x) + b*exp(2*x), exp(x))
-       #(a+b)*exp(2*x)
+       >>> collect(a*exp(2*x) + b*exp(2*x), exp(x))
+       (a + b)*exp(2*x)
 
        If you are interested only in collecting specific powers
        of some symbols then set 'exact' flag in arguments:
@@ -438,26 +432,32 @@ def collect(expr, syms, evaluate=True, exact=False):
        You can also apply this function to differential equations, where
        derivatives of arbitary order can be collected:
 
-       #>>> from sympy import Derivative as D
-       #>>> f = Function(x)
+       >>> from sympy import Derivative as D
+       >>> f = Function('f') (x)
 
-       #>>> collect(a*D(f,x) + b*D(f,x), D(f,x))
-       #(a+b)*Function'(x)
+       >>> collect(a*D(f,x) + b*D(f,x), D(f,x))
+       (a + b)*D(f(x), x)
 
-       #>>> collect(a*D(D(f,x),x) + b*D(D(f,x),x), D(f,x))
-       #(a+b)*(Function'(x))'
+       >>> collect(a*D(D(f,x),x) + b*D(D(f,x),x), D(f,x))
+       (a + b)*D(f(x), x, x)
 
-       #>>> collect(a*D(D(f,x),x) + b*D(D(f,x),x), D(f,x), exact=True)
-       #a*(Function'(x))'+b*(Function'(x))'
+       >>> collect(a*D(D(f,x),x) + b*D(D(f,x),x), D(f,x), exact=True)
+       a*D(f(x), x, x) + b*D(f(x), x, x)
 
-       #>>> collect(a*D(D(f,x),x) + b*D(D(f,x),x) + a*D(f,x) + b*D(f,x), D(f,x))
-       #(a+b)*Function'(x)+(a+b)*(Function'(x))'
+       >>> collect(a*D(D(f,x),x) + b*D(D(f,x),x) + a*D(f,x) + b*D(f,x), D(f,x))
+       (a + b)*D(f(x), x) + (a + b)*D(f(x), x, x)
 
        Or you can even match both derivative order and exponent at time:
 
-       #>>> collect(a*D(D(f,x),x)**2 + b*D(D(f,x),x)**2, D(f,x))
-       #(a+b)*(Function'(x))'**2
+       >>> collect(a*D(D(f,x),x)**2 + b*D(D(f,x),x)**2, D(f,x)) == \
+           (a + b)*D(D(f,x),x)**2
+       True
 
+
+    Notes
+    =====
+        - arguments are expected to be in expanded form, so you might have to call
+           .expand prior to calling this function.
     """
     def make_expression(terms):
         product = []
@@ -482,14 +482,39 @@ def collect(expr, syms, evaluate=True, exact=False):
     def parse_derivative(deriv):
         # scan derivatives tower in the input expression and return
         # underlying function and maximal differentiation order
-        expr, sym, order = deriv.f, deriv.x, 1
+        expr, sym, order = deriv.expr, deriv.symbols[0], 1
 
-        while isinstance(expr, Derivative) and expr.x == sym:
-            expr, order = expr.f, order+1
+        for s in deriv.symbols[1:]:
+            if s == sym:
+                order += 1
+            else:
+                raise NotImplementedError('Improve MV Derivative support in collect')
+
+        while isinstance(expr, Derivative):
+            s0 = expr.symbols[0]
+
+            for s in expr.symbols:
+                if s != s0:
+                    raise NotImplementedError('Improve MV Derivative support in collect')
+
+            if s0 == sym:
+                expr, order = expr.expr, order+len(expr.symbols)
+            else:
+                break
 
         return expr, (sym, Rational(order))
 
     def parse_term(expr):
+        """Parses expression expr and outputs tuple (sexpr, rat_expo, sym_expo, deriv)
+        where:
+         - sexpr is the base expression
+         - rat_expo is the rational exponent that sexpr is raised to
+         - sym_expo is the symbolic exponent that sexpr is raised to
+         - deriv contains the derivatives the the expression
+
+         for example, the output of x would be (x, 1, None, None)
+         the output of 2**x would be (2, 1, x, None)
+        """
         rat_expo, sym_expo = S.One, None
         sexpr, deriv = expr, None
 
@@ -502,7 +527,7 @@ def collect(expr, syms, evaluate=True, exact=False):
             if expr.exp.is_Rational:
                 rat_expo = expr.exp
             elif expr.exp.is_Mul:
-                coeff, tail = term.exp.as_coeff_terms()
+                coeff, tail = expr.exp.as_coeff_terms()
 
                 if coeff.is_Rational:
                     rat_expo, sym_expo = coeff, C.Mul(*tail)
@@ -512,7 +537,7 @@ def collect(expr, syms, evaluate=True, exact=False):
                 sym_expo = expr.exp
         elif expr.func is C.exp:
             if expr.args[0].is_Rational:
-                sexpr, rat_expo = S.Exp1, expr[0]
+                sexpr, rat_expo = S.Exp1, expr.args[0]
             elif expr.args[0].is_Mul:
                 coeff, tail = expr.args[0].as_coeff_terms()
 
@@ -524,6 +549,10 @@ def collect(expr, syms, evaluate=True, exact=False):
         return sexpr, rat_expo, sym_expo, deriv
 
     def parse_expression(terms, pattern):
+        """Parse terms searching for a pattern.
+        terms is a list of tuples as returned by parse_terms
+        pattern is an expression
+        """
         pattern = make_list(pattern, Mul)
 
         if len(terms) < len(pattern):
@@ -533,7 +562,7 @@ def collect(expr, syms, evaluate=True, exact=False):
         else:
             pattern = [ parse_term(elem) for elem in pattern ]
 
-            elems, common_expo, has_deriv = [], S.One, False
+            elems, common_expo, has_deriv = [], None, False
 
             for elem, e_rat, e_sym, e_ord in pattern:
                 if e_ord is not None:
@@ -544,20 +573,25 @@ def collect(expr, syms, evaluate=True, exact=False):
                 for j in range(len(terms)):
                     term, t_rat, t_sym, t_ord = terms[j]
 
+                    if elem.is_Number:
+                        # a constant is a match for everything
+                        break
+
                     if elem == term and e_sym == t_sym:
                         if exact == False:
                             # we don't have to exact so find common exponent
                             # for both expression's term and pattern's element
                             expo = t_rat / e_rat
 
-                            if common_expo is S.One:
+                            if common_expo is None:
+                                # first time
                                 common_expo = expo
                             else:
                                 # common exponent was negotiated before so
                                 # teher is no chance for pattern match unless
                                 # common and current exponents are equal
                                 if common_expo != expo:
-                                    return None
+                                    common_expo = 1
                         else:
                             # we ought to be exact so all fields of
                             # interest must match in very details
@@ -570,10 +604,10 @@ def collect(expr, syms, evaluate=True, exact=False):
                         del terms[j]
 
                         break
+
                 else:
                     # pattern element not found
                     return None
-
             return terms, elems, common_expo, has_deriv
 
     if evaluate:
@@ -599,22 +633,32 @@ def collect(expr, syms, evaluate=True, exact=False):
         terms = [ parse_term(i) for i in make_list(product, Mul) ]
 
         for symbol in syms:
+            if SYMPY_DEBUG:
+                print "DEBUG: parsing of expression %s with symbol %s " % (str(terms), str(symbol))
+
             result = parse_expression(terms, symbol)
 
+            if SYMPY_DEBUG:
+                print "DEBUG: returned %s" %  str(result)
+
             if result is not None:
                 terms, elems, common_expo, has_deriv = result
 
                 # when there was derivative in current pattern we
                 # will need to rebuild its expression from scratch
                 if not has_deriv:
-                    index = Pow(symbol, common_expo)
+                    index = 1
+                    for elem in elems:
+                        index *= Pow(elem[0], elem[1])
+                        if elem[2] is not None:
+                            index **= elem[2]
                 else:
                     index = make_expression(elems)
 
                 terms = separate(make_expression(terms))
                 index = separate(index)
 
-                if index in collected:
+                if index in collected.keys():
                     collected[index] += terms
                 else:
                     collected[index] = terms
@@ -689,15 +733,19 @@ def ratsimp(expr):
 
     return num/denum
 
-def trigsimp(expr, deep=False):
+def trigsimp(expr, deep=False, recursive=False):
     """
     Usage
     =====
-        trig(expr) -> reduces expression by using known trig identities
+        trigsimp(expr) -> reduces expression by using known trig identities
 
     Notes
     =====
 
+    deep ........ apply trigsimp inside functions
+    recursive ... use common subexpression elimination (cse()) and apply
+                  trigsimp recursively (recursively==True is quite expensive
+                  operation if the expression is large)
 
     Examples
     ========
@@ -714,21 +762,80 @@ def trigsimp(expr, deep=False):
     """
     from sympy.core.basic import S
     sin, cos, tan, cot = C.sin, C.cos, C.tan, C.cot
+    if recursive:
+        w, g = cse(expr)
+        g = trigsimp_nonrecursive(g[0])
+        for sub in reversed(w):
+            g = g.subs(sub[0], sub[1])
+            g = trigsimp_nonrecursive(g)
+        result = g
+    else:
+        result = trigsimp_nonrecursive(expr, deep)
 
-    #XXX this stopped working:
-    if expr == 1/cos(Symbol("x"))**2 - 1:
-        return tan(Symbol("x"))**2
+    # do some final simplifications like sin/cos -> tan:
+    a,b,c = map(Wild, 'abc')
+    matchers = (
+            (a*sin(b)**c/cos(b)**c, a*tan(b)**c),
+    )
+    for pattern, simp in matchers:
+        res = result.match(pattern)
+        if res is not None:
+            # if c is missing or zero, do nothing:
+            if (not c in res) or res[c] == 0:
+                continue
+            # if "a" contains the argument of sin/cos "b", skip the
+            # simplification:
+            if res[a].has(res[b]):
+                continue
+            # simplify and finish:
+            result = simp.subs(res)
+            break
+
+    return result
+
+
+def trigsimp_nonrecursive(expr, deep=False):
+    """
+    A nonrecursive trig simplifier, used from trigsimp.
+
+    Usage
+    =====
+        trigsimp_nonrecursive(expr) -> reduces expression by using known trig
+                                       identities
+
+    Notes
+    =====
+
+    deep ........ apply trigsimp inside functions
+
+    Examples
+    ========
+        >>> from sympy import *
+        >>> x = Symbol('x')
+        >>> y = Symbol('y')
+        >>> e = 2*sin(x)**2 + 2*cos(x)**2
+        >>> trigsimp(e)
+        2
+        >>> trigsimp_nonrecursive(log(e))
+        log(2*cos(x)**2 + 2*sin(x)**2)
+        >>> trigsimp_nonrecursive(log(e), deep=True)
+        log(2)
+    """
+    from sympy.core.basic import S
+    sin, cos, tan, cot = C.sin, C.cos, C.tan, C.cot
 
     if expr.is_Function:
         if deep:
-            return expr.func( trigsimp(expr.args[0], deep) )
+            return expr.func( trigsimp_nonrecursive(expr.args[0], deep) )
     elif expr.is_Mul:
         ret = S.One
         for x in expr.args:
-            ret *= trigsimp(x, deep)
+            ret *= trigsimp_nonrecursive(x, deep)
+
         return ret
     elif expr.is_Pow:
-        return Pow(trigsimp(expr.base, deep), trigsimp(expr.exp, deep))
+        return Pow(trigsimp_nonrecursive(expr.base, deep),
+                trigsimp_nonrecursive(expr.exp, deep))
     elif expr.is_Add:
         # TODO this needs to be faster
 
@@ -743,7 +850,7 @@ def trigsimp(expr, deep=False):
         # Scan for the terms we need
         ret = S.Zero
         for term in expr.args:
-            term = trigsimp(term, deep)
+            term = trigsimp_nonrecursive(term, deep)
             res = None
             for pattern, result in matchers:
                 res = term.match(pattern)
@@ -769,6 +876,8 @@ def trigsimp(expr, deep=False):
             a_t = Wild('a', exclude=[ex])
             pattern = pattern.subs(a, a_t)
             result = result.subs(a, a_t)
+            if expr.is_number:
+                continue
 
             m = expr.match(pattern)
             while m is not None:
@@ -788,7 +897,7 @@ def radsimp(expr):
     =========
         >>> from sympy import *
         >>> radsimp(1/(2+sqrt(2)))
-        1 - 1/2*2**(1/2)
+        1 - 2**(1/2)/2
         >>> x,y = map(Symbol, 'xy')
         >>> e = ( (2+2*sqrt(2))*x+(2+sqrt(8))*y )/( 2+sqrt(2) )
         >>> radsimp(e)
@@ -969,3 +1078,81 @@ def simplify(expr):
     """
     expr = Poly.cancel(powsimp(expr))
     return together(expr.expand())
+
+def nsimplify(expr, constants=[], tolerance=None, full=False):
+    """
+    Numerical simplification -- tries to find a simple formula
+    that numerically matches the given expression. The input should
+    be possible to evalf to a precision of at least 30 digits.
+
+    Optionally, a list of (rationally independent) constants to
+    include in the formula may be given.
+
+    A lower tolerance may be set to find less exact matches.
+
+    With full=True, a more extensive search is performed
+    (this is useful to find simpler numbers when the tolerance
+    is set low).
+
+    Examples:
+
+        >>> from sympy import *
+        >>> nsimplify(4/(1+sqrt(5)), [GoldenRatio])
+        -2 + 2*GoldenRatio
+        >>> nsimplify((1/(exp(3*pi*I/5)+1)))
+        1/2 - I*(1/4 + 5**(1/2)/10)**(1/2)
+        >>> nsimplify(I**I, [pi])
+        exp(-pi/2)
+        >>> nsimplify(pi, tolerance=0.01)
+        22/7
+
+    """
+    expr = sympify(expr)
+
+    prec = 30
+    bprec = int(prec*3.33)
+
+    constants_dict = {}
+    for constant in constants:
+        constant = sympify(constant)
+        v = constant.evalf(prec)
+        if not v.is_Real:
+            raise ValueError("constants must be real-valued")
+        constants_dict[str(constant)] = v._to_mpmath(bprec)
+
+    exprval = expr.evalf(prec, chop=True)
+    re, im = exprval.as_real_imag()
+
+    # Must be numerical
+    if not ((re.is_Real or re.is_Integer) and (im.is_Real or im.is_Integer)):
+        return expr
+
+    def nsimplify_real(x):
+        orig = mpmath.mp.dps
+        xv = x._to_mpmath(bprec)
+        try:
+            # We'll be happy with low precision if a simple fraction
+            if not (tolerance or full):
+                mpmath.mp.dps = 15
+                rat = mpmath.findpoly(xv, 1)
+                if rat is not None:
+                    return Rational(-int(rat[1]), int(rat[0]))
+            mpmath.mp.dps = prec
+            newexpr = mpmath.identify(xv, constants=constants_dict,
+                tol=tolerance, full=full)
+            if not newexpr:
+                raise ValueError
+            if full:
+                newexpr = newexpr[0]
+            return sympify(newexpr)
+        finally:
+            mpmath.mp.dps = orig
+    try:
+        if re: re = nsimplify_real(re)
+        if im: im = nsimplify_real(im)
+    except ValueError:
+        return expr
+
+    return re + im*S.ImaginaryUnit
+
+
