@@ -205,6 +205,34 @@ def test_lambdify_matrix():
     f = lambdify(x, Matrix([[x, 2*x],[1, 2]]), "numpy")
     assert (f(1) == matrix([[1,2],[1,2]])).all()
 
+def test_lambdify_matrix_multi_input():
+    x,y,z=sympy.symbols('x,y,z')
+    M=sympy.Matrix([[x**2, x*y, x*z],
+                    [y*x, y**2, y*z],
+                    [z*x, z*y, z**2]])
+    f = lambdify((x,y,z), M, "numpy")
+
+    xh,yh,zh = 1.0, 2.0, 3.0
+    expected = matrix([[xh**2, xh*yh, xh*zh],
+                       [yh*xh, yh**2, yh*zh],
+                       [zh*xh, zh*yh, zh**2]])
+    actual = f(xh,yh,zh)
+    assert numpy.allclose(actual,expected)
+
+def test_lambdify_matrix_vec_input():
+    X=sympy.DeferredVector('X')
+    M=Matrix([[X[0]**2, X[0]*X[1], X[0]*X[2]],
+              [X[1]*X[0], X[1]**2, X[1]*X[2]],
+              [X[2]*X[0], X[2]*X[1], X[2]**2]])
+    f = lambdify(X, M, "numpy")
+
+    Xh = array([1.0, 2.0, 3.0])
+    expected = matrix([[Xh[0]**2, Xh[0]*Xh[1], Xh[0]*Xh[2]],
+                       [Xh[1]*Xh[0], Xh[1]**2, Xh[1]*Xh[2]],
+                       [Xh[2]*Xh[0], Xh[2]*Xh[1], Xh[2]**2]])
+    actual = f(Xh)
+    assert numpy.allclose(actual,expected)
+
 def test_lambdify_transl():
     from sympy.utilities.lambdify import NUMPY_TRANSLATIONS
     for sym, mat in NUMPY_TRANSLATIONS.iteritems():
