@@ -2,7 +2,7 @@
 """
 import sys
 
-from iterables import make_list, flatten
+from iterables import make_list, flatten, subsets
 
 if sys.version_info[1] < 5:
     from iterables import any, all
@@ -15,6 +15,7 @@ from source import source
 
 from decorator import threaded
 
-from runtests import test
+from runtests import test, doctest
 
 from pytest import raises
+
