@@ -3,7 +3,8 @@ This is our testing framework.
 
 Goals:
 
-* it should behave very similar (or identical) to py.test
+* it should be compatible with py.test and operate very similarly (or
+identically)
 * doesn't require any external dependencies
 * preferably all the functionality should be in this file only
 * no magic, just import the test file and execute the test functions, that's it
@@ -68,6 +69,49 @@ def test(*paths, **kwargs):
         t.add_paths(["sympy"])
     return t.test()
 
+def doctest(*paths, **kwargs):
+    """
+    Runs the doctests specified by paths, or all tests if paths=[].
+
+    Note: paths are specified relative to the sympy root directory in a unix
+    format (on all platforms including windows).
+
+    Examples:
+
+    Run all tests:
+    >> import sympy
+    >> sympy.doctest()
+
+    Run one file:
+    >> import sympy
+    >> sympy.doctest("sympy/core/tests/test_basic.py")
+
+    Run all tests in sympy/functions/ and some particular file:
+    >> import sympy
+    >> sympy.doctest("sympy/core/tests/test_basic.py", "sympy/functions")
+    """
+    verbose = kwargs.get("verbose", False)
+    blacklist = kwargs.get("blacklist", [])
+    blacklist.extend([
+        "sympy/thirdparty/pyglet", # segfaults
+        "sympy/mpmath", # needs to be fixed upstream
+        "sympy/plotting", # generates live plots
+        "sympy/utilities/compilef.py", # needs tcc
+        "sympy/galgebra/GA.py", # needs numpy
+        "sympy/galgebra/latex_ex.py", # needs numpy
+        "sympy/conftest.py", # needs py.test
+        "sympy/utilities/benchmarking.py", # needs py.test
+        ])
+
+    r = PyTestReporter(verbose)
+    t = SymPyDocTests(r, blacklist=blacklist)
+    if len(paths) > 0:
+        t.add_paths(paths)
+    else:
+        t.add_paths(["sympy"])
+    return t.test()
+
+
 class SymPyTests(object):
 
     def __init__(self, reporter, kw="", post_mortem=False):
@@ -103,53 +147,46 @@ class SymPyTests(object):
         return self._reporter.finish()
 
     def test_file(self, filename):
-        import sympy
-        import imp
         name = "test%d" % self._count
         name = os.path.splitext(os.path.basename(filename))[0]
         self._count += 1
+        gl = {'__file__':filename}
         try:
-            #module = __import__(filename, globals(), locals())
-            module = imp.load_source(name, filename)
+            execfile(filename, gl)
         except (ImportError, SyntaxError):
             self._reporter.import_error(filename, sys.exc_info())
             return
-        disabled = getattr(module, "disabled", False)
+        pytestfile = ""
+        if gl.has_key("XFAIL"):
+            pytestfile = inspect.getsourcefile(gl["XFAIL"])
+        disabled = gl.get("disabled", False)
         if disabled:
             funcs = []
         else:
-            funcs = sorted(module.__dict__.keys())
             # we need to filter only those functions that begin with 'test_'
-            funcs = [f for f in funcs if f.startswith("test_")]
-            # and also that are defined in this module (i.e. not imported from
-            # other modules using the import statement, like "from sympy import
-            # *"). This is tricky to achieve. The easiest is to compare m1 and
-            # m2 below, if they are equal, we are sure that the "f" is from
-            # module. However, when one uses the XFAIL decorator, the m1
-            # appears from "sympy.utilities.pytest", so we check this one
-            # explicitly. This is not robust, as it will stop working when we
-            # move XFAIL to another module, or if we use some decorator defined
-            # elsewhere. Any help with this is welcomed.
-            funcs2 = []
-            m2 = module.__name__
-            for f in funcs:
-                f = module.__dict__[f]
-                m1 = f.__module__
-                if m1 == m2 or m1 == "sympy.utilities.pytest":
-                    if isgeneratorfunction(f):
-                        # some tests can be generators, that return the actual
-                        # test functions. We unpack it below:
-                        for fg in f():
-                            func = fg[0]
-                            args = fg[1:]
-                            fgw = lambda: func(*args)
-                            funcs2.append((fgw, inspect.getsourcelines(f)[1]))
-                    else:
-                        funcs2.append((f, inspect.getsourcelines(f)[1]))
-            # 'func2' now contains all candidates of functions to test.
-            # sort them according to the line of occurence:
-            funcs2.sort(key=lambda x: x[1])
-            funcs = [x[0] for x in funcs2]
+            # that are defined in the testing file or in the file where
+            # is defined the XFAIL decorator
+            funcs = [gl[f] for f in gl.keys() if f.startswith("test_") and
+                                                 (inspect.isfunction(gl[f])
+                                                    or inspect.ismethod(gl[f])) and
+                                                 (inspect.getsourcefile(gl[f]) == filename or
+                                                   inspect.getsourcefile(gl[f]) == pytestfile)]
+            # Sorting of XFAILed functions isn't fixed yet :-(
+            funcs.sort(key=lambda x: inspect.getsourcelines(x)[1])
+            i = 0
+            while i is not len(funcs):
+                if isgeneratorfunction(funcs[i]):
+                # some tests can be generators, that return the actual
+                # test functions. We unpack it below:
+                    f = funcs.pop(i)
+                    for fg in f():
+                        func = fg[0]
+                        args = fg[1:]
+                        fgw = lambda: func(*args)
+                        funcs.insert(i, fgw)
+                        i += 1
+                else:
+                    i += 1
             # drop functions that are not selected with the keyword expression:
             funcs = [x for x in funcs if self.matches(x)]
 
@@ -171,7 +208,7 @@ class SymPyTests(object):
                 elif t.__name__ == "XFail":
                     self._reporter.test_xfail()
                 elif t.__name__ == "XPass":
-                    self._reporter.test_xpass()
+                    self._reporter.test_xpass(v)
                 else:
                     self._reporter.test_exception((t, v, tr))
                     if self._post_mortem:
@@ -228,6 +265,149 @@ class SymPyTests(object):
         g.sort()
         return g
 
+class SymPyDocTests(object):
+
+    def __init__(self, reporter, blacklist=[]):
+        self._count = 0
+        self._root_dir = self.get_sympy_dir()
+        self._reporter = reporter
+        self._reporter.root_dir(self._root_dir)
+        self._tests = []
+        self._blacklist = blacklist
+
+    def add_paths(self, paths):
+        for path in paths:
+            path2 = os.path.join(self._root_dir, *path.split("/"))
+            if path2.endswith(".py"):
+                self._tests.append(path2)
+            else:
+                self._tests.extend(self.get_tests(path2))
+
+    def test(self):
+        """
+        Runs the tests.
+
+        Returns True if all tests pass, otherwise False.
+        """
+        self._reporter.start()
+        for f in self._tests:
+            try:
+                self.test_file(f)
+            except KeyboardInterrupt:
+                print " interrupted by user"
+                break
+        return self._reporter.finish()
+
+    def test_file(self, filename):
+        def setup_pprint():
+            from sympy import pprint_use_unicode
+            # force pprint to be in ascii mode in doctests
+            pprint_use_unicode(False)
+
+            # hook our nice, hash-stable strprinter
+            from sympy.interactive import init_printing
+            from sympy.printing import sstrrepr
+            init_printing(sstrrepr)
+
+        import doctest
+        import unittest
+        from StringIO import StringIO
+
+        rel_name = filename[len(self._root_dir)+1:]
+        module = rel_name.replace('/', '.')[:-3]
+        setup_pprint()
+        try:
+            module = doctest._normalize_module(module)
+            tests = doctest.DocTestFinder().find(module)
+        except:
+            self._reporter.import_error(filename, sys.exc_info())
+            return
+
+        tests.sort()
+        tests = [test for test in tests if len(test.examples) > 0]
+        self._reporter.entering_filename(filename, len(tests))
+        for test in tests:
+            assert len(test.examples) != 0
+            runner = doctest.DocTestRunner()
+            old = sys.stdout
+            new = StringIO()
+            sys.stdout = new
+            try:
+                f, t = runner.run(test, out=new.write, clear_globs=False)
+            finally:
+                sys.stdout = old
+            if f > 0:
+                self._reporter.doctest_fail(test.name, new.getvalue())
+            else:
+                self._reporter.test_pass()
+        self._reporter.leaving_filename()
+
+    def get_sympy_dir(self):
+        """
+        Returns the root sympy directory.
+        """
+        this_file = os.path.abspath(__file__)
+        sympy_dir = os.path.join(os.path.dirname(this_file), "..", "..")
+        sympy_dir = os.path.normpath(sympy_dir)
+        return sympy_dir
+
+
+    def get_paths(self, dir="", level=15):
+        """
+        Generates a set of paths for testfiles searching.
+
+        Example:
+        >> get_paths(2)
+        ['sympy/test_*.py', 'sympy/*/test_*.py', 'sympy/*/*/test_*.py']
+        >> get_paths(6)
+        ['sympy/test_*.py', 'sympy/*/test_*.py', 'sympy/*/*/test_*.py',
+        'sympy/*/*/*/test_*.py', 'sympy/*/*/*/*/test_*.py',
+        'sympy/*/*/*/*/*/test_*.py', 'sympy/*/*/*/*/*/*/test_*.py']
+        """
+        wildcards = [dir]
+        for i in range(level):
+            wildcards.append(os.path.join(wildcards[-1], "*"))
+        p = [os.path.join(x, "*.py") for x in wildcards]
+        return p
+
+    def is_on_blacklist(self, x):
+        """
+        Returns True if "x" is on the blacklist. Otherwise False.
+        """
+        for p in self._blacklist:
+            if x.find(p) != -1:
+                return True
+        return False
+
+    def get_tests(self, dir):
+        """
+        Returns the list of tests.
+        """
+        def importable(x):
+            """
+            Checks if given pathname x is an importable module by checking for
+            __init__.py file.
+
+            Returns True/False.
+
+            Currently we only test if the __init__.py file exists in the
+            directory with the file "x" (in theory we should also test all the
+            parent dirs) and if "x" is not on self._blacklist.
+            """
+            if self.is_on_blacklist(x):
+                return False
+            init_py = os.path.dirname(x) + os.path.sep + "__init__.py"
+            return os.path.exists(init_py)
+
+        g = []
+        for x in self.get_paths(dir):
+            g.extend(glob(x))
+        g = list(set(g))
+        g.sort()
+        # skip files that are not importable (i.e. missing __init__.py)
+        g = [x for x in g if importable(x)]
+        return g
+
 class Reporter(object):
     """
     Parent class for all reporters.
@@ -246,6 +426,7 @@ class PyTestReporter(Reporter):
         self._xfailed = 0
         self._xpassed = []
         self._failed = []
+        self._failed_doctest = []
         self._passed = 0
         self._skipped = 0
         self._exceptions = []
@@ -253,6 +434,7 @@ class PyTestReporter(Reporter):
         # this tracks the x-position of the cursor (useful for positioning
         # things on the screen), without the need for any readline library:
         self._write_pos = 0
+        self._line_wrap = False
 
     def root_dir(self, dir):
         self._root_dir = dir
@@ -306,6 +488,10 @@ class PyTestReporter(Reporter):
             # don't use any colors.
             color = ""
 
+        if self._line_wrap:
+            if text[0] != "\n":
+                sys.stdout.write("\n")
+
         if color == "":
             sys.stdout.write(text)
         else:
@@ -316,6 +502,7 @@ class PyTestReporter(Reporter):
             self._write_pos += len(text)
         else:
             self._write_pos = len(text)-l-1
+        self._line_wrap = self._write_pos >= width
         self._write_pos %= width
 
     def write_center(self, text, delim="="):
@@ -349,6 +536,8 @@ class PyTestReporter(Reporter):
         text = "tests finished: %d passed" % self._passed
         if len(self._failed) > 0:
             text += ", %d failed" % len(self._failed)
+        if len(self._failed_doctest) > 0:
+            text += ", %d failed" % len(self._failed_doctest)
         if self._skipped > 0:
             text += ", %d skipped" % self._skipped
         if self._xfailed > 0:
@@ -363,7 +552,7 @@ class PyTestReporter(Reporter):
         if len(self._xpassed) > 0:
             self.write_center("xpassed tests", "_")
             for e in self._xpassed:
-                self.write("%s:%s\n" % (e[0], e[1].__name__))
+                self.write("%s:%s\n" % (e[0], e[1]))
             self.write("\n")
 
         if self._tb_style != "no" and len(self._exceptions) > 0:
@@ -388,8 +577,18 @@ class PyTestReporter(Reporter):
                 self.write_exception(t, val, tb)
             self.write("\n")
 
+        if self._tb_style != "no" and len(self._failed_doctest) > 0:
+            #self.write_center("Failed", "_")
+            for e in self._failed_doctest:
+                filename, msg = e
+                self.write_center("", "_")
+                self.write_center("%s" % filename, "_")
+                self.write(msg)
+            self.write("\n")
+
         self.write_center(text)
-        ok = len(self._failed) == 0 and len(self._exceptions) == 0
+        ok = len(self._failed) == 0 and len(self._exceptions) == 0 and \
+                len(self._failed_doctest) == 0
         if not ok:
             self.write("DO *NOT* COMMIT!\n")
         return ok
@@ -421,8 +620,8 @@ class PyTestReporter(Reporter):
         self._xfailed += 1
         self.write("f")
 
-    def test_xpass(self):
-        self._xpassed.append((self._active_file, self._active_f))
+    def test_xpass(self, fname):
+        self._xpassed.append((self._active_file, fname))
         self.write("X")
 
     def test_fail(self, exc_info):
@@ -430,6 +629,13 @@ class PyTestReporter(Reporter):
         self.write("F")
         self._active_file_error = True
 
+    def doctest_fail(self, name, error_msg):
+        # the first line contains "******", remove it:
+        error_msg = "\n".join(error_msg.split("\n")[1:])
+        self._failed_doctest.append((name, error_msg))
+        self.write("F")
+        self._active_file_error = True
+
     def test_pass(self):
         self._passed += 1
         if self._verbose:
@@ -450,7 +656,7 @@ class PyTestReporter(Reporter):
         self._exceptions.append((filename, None, exc_info))
         rel_name = filename[len(self._root_dir)+1:]
         self.write(rel_name)
-        self.write("[?] Failed to import")
+        self.write("[?]   Failed to import")
         if self._colors:
             self.write(" ")
             self.write("[FAIL]", "Red", align="right")
