@@ -1,7 +1,7 @@
-from sympy import symbols, lambdify, sqrt, sin, cos, pi
 from sympy.utilities.pytest import XFAIL
-from sympy import symbols, lambdify, sqrt, sin, cos, pi, atan, Rational, Real
-from sympy.thirdparty import mpmath
+from sympy import (symbols, lambdify, sqrt, sin, cos, pi, atan, Rational, Real,
+        Matrix, Lambda, exp, Integral, oo)
+from sympy import mpmath
 import math, sympy
 
 # high precision output of sin(0.2*pi) is used to detect if precision is lost unwanted
@@ -209,3 +209,16 @@ def test_sin():
     assert isinstance(f(2), float)
     f = lambdify(x, sin(x)**2, modules="math")
     assert isinstance(f(2), float)
+
+def test_matrix():
+    A = Matrix([[x, x*y], [sin(z)+4, x**z]])
+    sol = Matrix([[1, 2], [sin(3)+4, 1]])
+    f = lambdify((x,y,z), A, modules="sympy")
+    assert f(1,2,3) == sol
+    f = lambdify((x,y,z), (A, [A]), modules="sympy")
+    assert f(1,2,3) == (sol,[sol])
+
+def test_integral():
+    f = Lambda(x, exp(-x**2))
+    l = lambdify(x, Integral(f(x), (x, -oo, oo)), modules="sympy")
+    assert l(x) == Integral(exp(-x**2), (x, -oo, oo))
